@@ -46,19 +46,20 @@
 #include <netdb.h>		/* for gethostbyname */
 #include <netinet/in.h>
 #include <pthread.h>
+#include <signal.h>
 #include <stdarg.h>
-#include <stdlib.h>
 #include <stdio.h>
+#include <stdlib.h>
 #include <string.h>
-#include <sys/param.h>
-#include <sys/socket.h>
 #include <sys/stat.h>
 #include <sys/types.h>
-#include <sys/un.h>
 #include <unistd.h>
 
 #include "slurm/slurm.h"
 
+#include "src/api/pmi_server.h"
+#include "src/api/step_launch.h"
+
 #include "src/common/cpu_frequency.h"
 #include "src/common/eio.h"
 #include "src/common/fd.h"
@@ -68,9 +69,6 @@
 #include "src/common/macros.h"
 #include "src/common/net.h"
 #include "src/common/read_config.h"
-#include "src/interfaces/auth.h"
-#include "src/interfaces/cred.h"
-#include "src/interfaces/mpi.h"
 #include "src/common/slurm_protocol_api.h"
 #include "src/common/slurm_protocol_defs.h"
 #include "src/common/slurm_time.h"
@@ -80,8 +78,12 @@
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
 
-#include "src/api/step_launch.h"
-#include "src/api/pmi_server.h"
+#include "src/interfaces/auth.h"
+#include "src/interfaces/certgen.h"
+#include "src/interfaces/certmgr.h"
+#include "src/interfaces/conn.h"
+#include "src/interfaces/cred.h"
+#include "src/interfaces/mpi.h"
 
 #include "src/srun/step_ctx.h"
 
@@ -94,7 +96,7 @@ extern char **environ;
  **********************************************************************/
 static int _launch_tasks(slurm_step_ctx_t *ctx,
 			 launch_tasks_request_msg_t *launch_msg,
-			 uint32_t timeout, char *nodelist);
+			 uint32_t timeout, uint16_t tree_width, char *nodelist);
 static char *_lookup_cwd(void);
 static void _print_launch_msg(launch_tasks_request_msg_t *msg,
 			      char *hostname, int nodeid);
@@ -169,10 +171,6 @@ static void _rebuild_mpi_layout(slurm_step_ctx_t *ctx,
 	new_step_layout = xmalloc(sizeof(slurm_step_layout_t));
 	orig_step_layout = ctx->launch_state->mpi_step->step_layout;
 	ctx->launch_state->mpi_step->step_layout = new_step_layout;
-	if (orig_step_layout->front_end) {
-		new_step_layout->front_end =
-			xstrdup(orig_step_layout->front_end);
-	}
 	new_step_layout->node_cnt = params->het_job_nnodes;
 	new_step_layout->node_list = xstrdup(params->het_job_node_list);
 	new_step_layout->plane_size = orig_step_layout->plane_size;
@@ -201,13 +199,14 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 	int rc = SLURM_SUCCESS;
 	bool preserve_env = params->preserve_env;
 	uint32_t mpi_plugin_id;
+	char *io_key = NULL;
 
 	debug("Entering %s", __func__);
 	memset(&launch, 0, sizeof(launch));
 
 	if ((ctx == NULL) || (ctx->magic != STEP_CTX_MAGIC)) {
 		error("%s: Not a valid slurm_step_ctx_t", __func__);
-		slurm_seterrno(EINVAL);
+		errno = EINVAL;
 		return SLURM_ERROR;
 	}
 
@@ -222,9 +221,11 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 		       sizeof(slurm_step_launch_callbacks_t));
 	}
 
+	ctx->launch_state->job_id = ctx->step_req->step_id.job_id;
+
 	mpi_plugin_id = mpi_g_client_init((char **)&params->mpi_plugin_name);
 	if (!mpi_plugin_id) {
-		slurm_seterrno(SLURM_MPI_PLUGIN_NAME_INVALID);
+		errno = SLURM_MPI_PLUGIN_NAME_INVALID;
 		return SLURM_ERROR;
 	}
 
@@ -234,7 +235,7 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 	if ((ctx->launch_state->mpi_state =
 	     mpi_g_client_prelaunch(ctx->launch_state->mpi_step, &mpi_env))
 	    == NULL) {
-		slurm_seterrno(SLURM_MPI_PLUGIN_PRELAUNCH_SETUP_FAILED);
+		errno = SLURM_MPI_PLUGIN_PRELAUNCH_SETUP_FAILED;
 		return SLURM_ERROR;
 	}
 
@@ -248,21 +249,6 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 	memcpy(&launch.step_id, &ctx->step_req->step_id,
 	       sizeof(launch.step_id));
 
-	if (ctx->step_resp->cred) {
-		slurm_cred_arg_t *args;
-
-		if ((args = slurm_cred_get_args(ctx->step_resp->cred))) {
-			launch.uid = args->uid;
-			launch.gid = args->gid;
-		} else {
-			/* fake cred */
-			launch.uid = getuid();
-			launch.gid = getgid();
-		}
-
-		slurm_cred_unlock_args(ctx->step_resp->cred);
-	}
-
 	launch.argc = params->argc;
 	launch.argv = params->argv;
 	launch.spank_job_env = params->spank_job_env;
@@ -275,6 +261,7 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 	launch.het_job_nnodes = params->het_job_nnodes;
 	launch.het_job_ntasks = params->het_job_ntasks;
 	launch.het_job_offset = params->het_job_offset;
+	launch.het_job_step_task_cnts = params->het_job_step_task_cnts;
 	launch.het_job_task_offset = params->het_job_task_offset;
 	launch.het_job_task_cnts = params->het_job_task_cnts;
 	launch.het_job_tids = params->het_job_tids;
@@ -302,12 +289,11 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 		launch.cwd = xstrdup(params->cwd);
 	else
 		launch.cwd = _lookup_cwd();
-	launch.alias_list	= params->alias_list;
 	launch.mpi_plugin_id = mpi_plugin_id;
 	launch.nnodes		= ctx->step_resp->step_layout->node_cnt;
 	launch.ntasks		= ctx->step_resp->step_layout->task_cnt;
 	launch.slurmd_debug	= params->slurmd_debug;
-	launch.switch_job	= ctx->step_resp->switch_job;
+	launch.switch_step = ctx->step_resp->switch_step;
 	launch.profile		= params->profile;
 	launch.task_prolog	= params->task_prolog;
 	launch.task_epilog	= params->task_epilog;
@@ -325,7 +311,11 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 	if (params->multi_prog)
 		launch.flags	|= LAUNCH_MULTI_PROG;
 	launch.cpus_per_task	= params->cpus_per_task;
+	launch.cpt_compact_array = params->cpt_compact_array;
+	launch.cpt_compact_cnt = params->cpt_compact_cnt;
+	launch.cpt_compact_reps = params->cpt_compact_reps;
 	launch.tres_per_task	= ctx->step_req->tres_per_task;
+	launch.stepmgr = xstrdup(ctx->step_resp->stepmgr);
 
 	launch.threads_per_core	= params->threads_per_core;
 	launch.ntasks_per_board = params->ntasks_per_board;
@@ -337,9 +327,14 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 		launch.flags	|= LAUNCH_NO_ALLOC;
 	if (ctx->step_req->flags & SSF_OVERCOMMIT)
 		launch.flags |= LAUNCH_OVERCOMMIT;
+	if (ctx->step_req->flags & SSF_EXT_LAUNCHER)
+		launch.flags |= LAUNCH_EXT_LAUNCHER;
+	if (ctx->step_req->flags & SSF_GRES_ALLOW_TASK_SHARING)
+		launch.flags |= LAUNCH_GRES_ALLOW_TASK_SHARING;
+	if (ctx->step_req->flags & SSF_WAIT_FOR_CHILDREN)
+		launch.flags |= LAUNCH_WAIT_FOR_CHILDREN;
 
 	launch.task_dist	= params->task_dist;
-	launch.partition	= params->partition;
 	if (params->pty)
 		launch.flags |= LAUNCH_PTY;
 	launch.acctg_freq	= params->acctg_freq;
@@ -354,20 +349,33 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 	launch.tasks_to_launch = ctx->step_resp->step_layout->tasks;
 	launch.global_task_ids = ctx->step_resp->step_layout->tids;
 
-	launch.select_jobinfo  = ctx->step_resp->select_jobinfo;
-
 	launch.ofname = params->remote_output_filename;
 	launch.efname = params->remote_error_filename;
 	launch.ifname = params->remote_input_filename;
+	if (params->oom_kill_step != NO_VAL16)
+		launch.oom_kill_step = (params->oom_kill_step == 1);
+	else
+		launch.oom_kill_step =
+			slurm_conf.task_plugin_param & OOM_KILL_STEP;
 	if (params->buffered_stdio)
 		launch.flags |= LAUNCH_BUFFERED_IO;
 	if (params->labelio)
 		launch.flags |= LAUNCH_LABEL_IO;
+
+	io_key = slurm_cred_get_signature(ctx->step_resp->cred);
+
+	if (tls_enabled()) {
+		if (!(launch.alloc_tls_cert = conn_g_get_own_public_cert())) {
+			error("Could not get self signed certificate for step IO");
+			rc = SLURM_ERROR;
+			goto fail1;
+		}
+	}
+
 	ctx->launch_state->io =
 		client_io_handler_create(params->local_fds,
 					 ctx->step_req->num_tasks,
-					 launch.nnodes,
-					 ctx->step_resp->cred,
+					 launch.nnodes, io_key,
 					 params->labelio,
 					 params->het_job_offset,
 					 params->het_job_task_offset);
@@ -381,10 +389,8 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 	 */
 	ctx->launch_state->io->sls = ctx->launch_state;
 
-	if (client_io_handler_start(ctx->launch_state->io) != SLURM_SUCCESS) {
-		rc = SLURM_ERROR;
-		goto fail1;
-	}
+	client_io_handler_start(ctx->launch_state->io);
+
 	launch.num_io_port = ctx->launch_state->io->num_listen;
 	launch.io_port = xcalloc(launch.num_io_port, sizeof(uint16_t));
 	memcpy(launch.io_port, ctx->launch_state->io->listenport,
@@ -400,17 +406,20 @@ extern int slurm_step_launch(slurm_step_ctx_t *ctx,
 	launch.resp_port = xcalloc(launch.num_resp_port, sizeof(uint16_t));
 	memcpy(launch.resp_port, ctx->launch_state->resp_port,
 	       (sizeof(uint16_t) * launch.num_resp_port));
+
 	rc = _launch_tasks(ctx, &launch, params->msg_timeout,
-			   launch.complete_nodelist);
+			   params->tree_width, launch.complete_nodelist);
 
 	/* clean up */
 	xfree(launch.resp_port);
 	xfree(launch.io_port);
 
 fail1:
-	xfree(launch.user_name);
+	xfree(io_key);
 	xfree(launch.complete_nodelist);
 	xfree(launch.cwd);
+	xfree(launch.stepmgr);
+	xfree(launch.alloc_tls_cert);
 	env_array_free(env);
 	FREE_NULL_LIST(launch.options);
 	return rc;
@@ -437,18 +446,19 @@ extern int slurm_step_launch_add(slurm_step_ctx_t *ctx,
 	uint16_t resp_port = 0;
 	bool preserve_env = params->preserve_env;
 	uint32_t mpi_plugin_id;
+	char *io_key = NULL;
 
 	debug("Entering %s", __func__);
 
 	if (!ctx || (ctx->magic != STEP_CTX_MAGIC) || !ctx->step_resp) {
 		error("%s: Not a valid slurm_step_ctx_t", __func__);
-		slurm_seterrno(EINVAL);
+		errno = EINVAL;
 		return SLURM_ERROR;
 	}
 
 	mpi_plugin_id = mpi_g_client_init((char **)&params->mpi_plugin_name);
 	if (!mpi_plugin_id) {
-		slurm_seterrno(SLURM_MPI_PLUGIN_NAME_INVALID);
+		errno = SLURM_MPI_PLUGIN_NAME_INVALID;
 		return SLURM_ERROR;
 	}
 
@@ -458,27 +468,13 @@ extern int slurm_step_launch_add(slurm_step_ctx_t *ctx,
 	memcpy(&launch.step_id, &ctx->step_req->step_id,
 	       sizeof(launch.step_id));
 
-	if (ctx->step_resp->cred) {
-		slurm_cred_arg_t *args;
-
-		if ((args = slurm_cred_get_args(ctx->step_resp->cred))) {
-			launch.uid = args->uid;
-			launch.gid = args->gid;
-		} else {
-			/* fake cred */
-			launch.uid = getuid();
-			launch.gid = getgid();
-		}
-
-		slurm_cred_unlock_args(ctx->step_resp->cred);
-	}
-
 	launch.argc = params->argc;
 	launch.argv = params->argv;
 	launch.spank_job_env = params->spank_job_env;
 	launch.spank_job_env_size = params->spank_job_env_size;
 	launch.cred = ctx->step_resp->cred;
 	launch.het_job_step_cnt = params->het_job_step_cnt;
+	launch.het_job_step_task_cnts = params->het_job_step_task_cnts;
 	launch.het_job_id = params->het_job_id;
 	launch.het_job_nnodes = params->het_job_nnodes;
 	launch.het_job_ntasks = params->het_job_ntasks;
@@ -512,12 +508,11 @@ extern int slurm_step_launch_add(slurm_step_ctx_t *ctx,
 		launch.cwd = xstrdup(params->cwd);
 	else
 		launch.cwd = _lookup_cwd();
-	launch.alias_list	= params->alias_list;
 	launch.mpi_plugin_id = mpi_plugin_id;
 	launch.nnodes		= ctx->step_resp->step_layout->node_cnt;
 	launch.ntasks		= ctx->step_resp->step_layout->task_cnt;
 	launch.slurmd_debug	= params->slurmd_debug;
-	launch.switch_job	= ctx->step_resp->switch_job;
+	launch.switch_step = ctx->step_resp->switch_step;
 	launch.profile		= params->profile;
 	launch.task_prolog	= params->task_prolog;
 	launch.task_epilog	= params->task_epilog;
@@ -535,8 +530,10 @@ extern int slurm_step_launch_add(slurm_step_ctx_t *ctx,
 	if (params->multi_prog)
 		launch.flags |= LAUNCH_MULTI_PROG;
 	launch.cpus_per_task	= params->cpus_per_task;
+	launch.cpt_compact_array = params->cpt_compact_array;
+	launch.cpt_compact_cnt = params->cpt_compact_cnt;
+	launch.cpt_compact_reps = params->cpt_compact_reps;
 	launch.task_dist	= params->task_dist;
-	launch.partition	= params->partition;
 	if (params->pty)
 		launch.flags |= LAUNCH_PTY;
 	launch.acctg_freq	= params->acctg_freq;
@@ -552,8 +549,6 @@ extern int slurm_step_launch_add(slurm_step_ctx_t *ctx,
 	launch.tasks_to_launch = ctx->step_resp->step_layout->tasks;
 	launch.global_task_ids = ctx->step_resp->step_layout->tids;
 
-	launch.select_jobinfo  = ctx->step_resp->select_jobinfo;
-
 	launch.ofname = params->remote_output_filename;
 	launch.efname = params->remote_error_filename;
 	launch.ifname = params->remote_input_filename;
@@ -561,11 +556,13 @@ extern int slurm_step_launch_add(slurm_step_ctx_t *ctx,
 		launch.flags	|= LAUNCH_BUFFERED_IO;
 	if (params->labelio)
 		launch.flags	|= LAUNCH_LABEL_IO;
+
+	io_key = slurm_cred_get_signature(ctx->step_resp->cred);
+
 	ctx->launch_state->io =
 		client_io_handler_create(params->local_fds,
 					 ctx->step_req->num_tasks,
-					 launch.nnodes,
-					 ctx->step_resp->cred,
+					 launch.nnodes, io_key,
 					 params->labelio,
 					 params->het_job_offset,
 					 params->het_job_task_offset);
@@ -579,10 +576,8 @@ extern int slurm_step_launch_add(slurm_step_ctx_t *ctx,
 	 */
 	ctx->launch_state->io->sls = ctx->launch_state;
 
-	if (client_io_handler_start(ctx->launch_state->io) != SLURM_SUCCESS) {
-		rc = SLURM_ERROR;
-		goto fail1;
-	}
+	client_io_handler_start(ctx->launch_state->io);
+
 	launch.num_io_port = ctx->launch_state->io->num_listen;
 	launch.io_port = xcalloc(launch.num_io_port, sizeof(uint16_t));
 	memcpy(launch.io_port, ctx->launch_state->io->listenport,
@@ -603,11 +598,12 @@ extern int slurm_step_launch_add(slurm_step_ctx_t *ctx,
 		       (sizeof(uint16_t) * launch.num_resp_port));
 	}
 
-	rc = _launch_tasks(ctx, &launch, params->msg_timeout, node_list);
+	rc = _launch_tasks(ctx, &launch, params->msg_timeout,
+			   params->max_threads, node_list);
 
 fail1:
 	/* clean up */
-	xfree(launch.user_name);
+	xfree(io_key);
 	xfree(launch.resp_port);
 	xfree(launch.io_port);
 
@@ -624,7 +620,7 @@ static void _step_abort(slurm_step_ctx_t *ctx)
 
 	if (!sls->abort_action_taken) {
 		slurm_kill_job_step(ctx->job_id, ctx->step_resp->job_step_id,
-				    SIGKILL);
+				    SIGKILL, 0);
 		sls->abort_action_taken = true;
 	}
 }
@@ -694,7 +690,7 @@ void slurm_step_launch_wait_finish(slurm_step_ctx_t *ctx)
 				slurm_kill_job_step(ctx->job_id,
 						    ctx->step_resp->
 						    job_step_id,
-						    SIGKILL);
+						    SIGKILL, 0);
 				sls->abort_action_taken = true;
 			}
 			if (!time_set) {
@@ -725,7 +721,7 @@ void slurm_step_launch_wait_finish(slurm_step_ctx_t *ctx)
 				 */
 				slurm_kill_job_step(ctx->job_id,
 						    ctx->step_resp->job_step_id,
-						    SIGKILL);
+						    SIGKILL, 0);
 				client_io_handler_abort(sls->io);
 				break;
 			} else if (errnum != 0) {
@@ -760,7 +756,7 @@ void slurm_step_launch_wait_finish(slurm_step_ctx_t *ctx)
 
 	slurm_mutex_unlock(&sls->lock);
 	if (sls->msg_thread)
-		pthread_join(sls->msg_thread, NULL);
+		slurm_thread_join(sls->msg_thread);
 	slurm_mutex_lock(&sls->lock);
 	pmi_kvs_free();
 
@@ -775,7 +771,7 @@ void slurm_step_launch_wait_finish(slurm_step_ctx_t *ctx)
 		slurm_cond_broadcast(&sls->cond);
 
 		slurm_mutex_unlock(&sls->lock);
-		pthread_join(sls->io_timeout_thread, NULL);
+		slurm_thread_join(sls->io_timeout_thread);
 		slurm_mutex_lock(&sls->lock);
 	}
 
@@ -824,10 +820,10 @@ extern void slurm_step_launch_fwd_signal(slurm_step_ctx_t *ctx, int signo)
 	int node_id, j, num_tasks;
 	slurm_msg_t req;
 	signal_tasks_msg_t msg;
-	hostlist_t hl;
+	hostlist_t *hl;
 	char *name = NULL;
-	List ret_list = NULL;
-	ListIterator itr;
+	list_t *ret_list = NULL;
+	list_itr_t *itr;
 	ret_data_info_t *ret_data_info = NULL;
 	int rc = SLURM_SUCCESS;
 	struct step_launch_state *sls = ctx->launch_state;
@@ -859,16 +855,9 @@ extern void slurm_step_launch_fwd_signal(slurm_step_ctx_t *ctx, int signo)
 		if (!active)
 			continue;
 
-		if (ctx->step_resp->step_layout->front_end) {
-			hostlist_push_host(hl,
-				      ctx->step_resp->step_layout->front_end);
-			break;
-		} else {
-			name = nodelist_nth_host(sls->layout->node_list,
-						 node_id);
-			hostlist_push_host(hl, name);
-			free(name);
-		}
+		name = nodelist_nth_host(sls->layout->node_list, node_id);
+		hostlist_push_host(hl, name);
+		free(name);
 	}
 
 	slurm_mutex_unlock(&sls->lock);
@@ -908,7 +897,7 @@ RESEND:	slurm_msg_t_init(&req);
 		 * probably just means the tasks exited in the meanwhile.
 		 */
 		if ((rc != 0) && (rc != ESLURM_INVALID_JOB_ID) &&
-		    (rc != ESLURMD_JOB_NOTRUNNING) && (rc != ESRCH) &&
+		    (rc != ESLURMD_STEP_NOTRUNNING) && (rc != ESRCH) &&
 		    (rc != EAGAIN) &&
 		    (rc != ESLURM_TRANSITION_STATE_NO_UPDATE)) {
 			error("Failure sending signal %d to %ps on node %s: %s",
@@ -932,7 +921,7 @@ RESEND:	slurm_msg_t_init(&req);
 }
 
 /**********************************************************************
- * Functions used by step_ctx code, but not exported throught the API
+ * Functions used by step_ctx code, but not exported through the API
  **********************************************************************/
 /*
  * Create a launch state structure for a specified step context, "ctx".
@@ -1022,36 +1011,14 @@ void step_launch_state_destroy(struct step_launch_state *sls)
 /* connect to srun_cr */
 static int _connect_srun_cr(char *addr)
 {
-	struct sockaddr_un sa;
-	unsigned int sa_len;
-	int fd, rc;
+	int fd = -1, rc;
 
 	if (!addr) {
 		error("%s: socket path name is NULL", __func__);
 		return -1;
 	}
-	if (strlen(addr) >= sizeof(sa.sun_path)) {
-		error("%s: socket path name too long (%s)", __func__, addr);
-		return -1;
-	}
-
-	fd = socket(AF_UNIX, SOCK_STREAM, 0);
-	if (fd < 0) {
-		error("failed creating cr socket: %m");
-		return -1;
-	}
-	memset(&sa, 0, sizeof(sa));
-
-	sa.sun_family = AF_UNIX;
-	strlcpy(sa.sun_path, addr, sizeof(sa.sun_path));
-	sa_len = strlen(sa.sun_path) + sizeof(sa.sun_family);
-
-	while (((rc = connect(fd, (struct sockaddr *)&sa, sa_len)) < 0) &&
-	       (errno == EINTR));
-
-	if (rc < 0) {
-		debug2("failed connecting cr socket: %m");
-		close(fd);
+	if ((rc = slurm_open_unix_stream(addr, 0, &fd))) {
+		debug2("failed connecting cr socket: %s", slurm_strerror(rc));
 		return -1;
 	}
 	return fd;
@@ -1158,7 +1125,7 @@ static int _msg_thr_create(struct step_launch_state *sls, int num_nodes)
 		eio_new_initial_obj(sls->msg_handle, obj);
 	}
 	/* finally, add the listening port that we told the slurmctld about
-	 * eariler in the step context creation phase */
+	 * earlier in the step context creation phase */
 	if (sls->slurmctld_socket_fd > -1) {
 		obj = eio_obj_create(sls->slurmctld_socket_fd,
 				     &message_socket_ops, (void *)sls);
@@ -1175,6 +1142,12 @@ _launch_handler(struct step_launch_state *sls, slurm_msg_t *resp)
 	launch_tasks_response_msg_t *msg = resp->data;
 	int i;
 
+	if (sls->job_id && (msg->step_id.job_id != sls->job_id)) {
+		verbose("Ignoring RESPONSE_LAUNCH_TASKS for JobId=%u (our JobId=%u)",
+			msg->step_id.job_id, sls->job_id);
+		return;
+	}
+
 	slurm_mutex_lock(&sls->lock);
 	if ((msg->count_of_pids > 0) &&
 	    bit_test(sls->tasks_started, msg->task_ids[0])) {
@@ -1192,6 +1165,7 @@ _launch_handler(struct step_launch_state *sls, slurm_msg_t *resp)
 			bit_set(sls->tasks_started, msg->task_ids[i]);
 			bit_set(sls->tasks_exited, msg->task_ids[i]);
 		}
+		sls->ret_code = 1;
 	} else {
 		for (i = 0; i < msg->count_of_pids; i++)
 			bit_set(sls->tasks_started, msg->task_ids[i]);
@@ -1207,7 +1181,7 @@ _launch_handler(struct step_launch_state *sls, slurm_msg_t *resp)
 static void
 _exit_handler(struct step_launch_state *sls, slurm_msg_t *exit_msg)
 {
-	task_exit_msg_t *msg = (task_exit_msg_t *) exit_msg->data;
+	task_exit_msg_t *msg = exit_msg->data;
 	void (*task_finish)(task_exit_msg_t *);
 	int i;
 
@@ -1245,8 +1219,13 @@ _exit_handler(struct step_launch_state *sls, slurm_msg_t *exit_msg)
 static void
 _job_complete_handler(struct step_launch_state *sls, slurm_msg_t *complete_msg)
 {
-	srun_job_complete_msg_t *step_msg =
-		(srun_job_complete_msg_t *) complete_msg->data;
+	srun_job_complete_msg_t *step_msg = complete_msg->data;
+
+	if (sls->job_id && (step_msg->job_id != sls->job_id)) {
+		verbose("Ignoring SRUN_JOB_COMPLETE for stray JobId=%u (our JobId=%u)",
+			step_msg->job_id, sls->job_id);
+		return;
+	}
 
 	if (step_msg->step_id == NO_VAL) {
 		verbose("Complete job %u received",
@@ -1268,8 +1247,13 @@ _job_complete_handler(struct step_launch_state *sls, slurm_msg_t *complete_msg)
 static void
 _timeout_handler(struct step_launch_state *sls, slurm_msg_t *timeout_msg)
 {
-	srun_timeout_msg_t *step_msg =
-		(srun_timeout_msg_t *) timeout_msg->data;
+	srun_timeout_msg_t *step_msg = timeout_msg->data;
+
+	if (sls->job_id && (step_msg->step_id.job_id != sls->job_id)) {
+		verbose("Ignoring SRUN_TIMEOUT for JobId=%u (our JobId=%u)",
+			step_msg->step_id.job_id, sls->job_id);
+		return;
+	}
 
 	if (sls->callback.step_timeout)
 		(sls->callback.step_timeout)(step_msg);
@@ -1289,13 +1273,19 @@ static void
 _node_fail_handler(struct step_launch_state *sls, slurm_msg_t *fail_msg)
 {
 	srun_node_fail_msg_t *nf = fail_msg->data;
-	hostlist_t fail_nodes, all_nodes;
-	hostlist_iterator_t fail_itr;
+	hostlist_t *fail_nodes, *all_nodes;
+	hostlist_iterator_t *fail_itr;
 	int num_node_ids;
 	int *node_ids;
 	int i, j;
 	int node_id, num_tasks;
 
+	if (sls->job_id && (nf->step_id.job_id != sls->job_id)) {
+		verbose("Ignoring SRUN_NODE_FAIL for JobId=%u (our JobId=%u)",
+			nf->step_id.job_id, sls->job_id);
+		return;
+	}
+
 	error("Node failure on %s", nf->nodelist);
 
 	fail_nodes = hostlist_create(nf->nodelist);
@@ -1307,9 +1297,6 @@ _node_fail_handler(struct step_launch_state *sls, slurm_msg_t *fail_msg)
 	all_nodes = hostlist_create(sls->layout->node_list);
 	/* find the index number of each down node */
 	for (i = 0; i < num_node_ids; i++) {
-#ifdef HAVE_FRONT_END
-		node_id = 0;
-#else
 		char *node = hostlist_next(fail_itr);
 		node_id = node_ids[i] = hostlist_find(all_nodes, node);
 		if (node_id < 0) {
@@ -1319,7 +1306,6 @@ _node_fail_handler(struct step_launch_state *sls, slurm_msg_t *fail_msg)
 			continue;
 		}
 		free(node);
-#endif
 
 		/* find all of the tasks that should run on this node and
 		 * mark them as having started and exited.  If they haven't
@@ -1364,8 +1350,8 @@ static void
 _step_missing_handler(struct step_launch_state *sls, slurm_msg_t *missing_msg)
 {
 	srun_step_missing_msg_t *step_missing = missing_msg->data;
-	hostlist_t fail_nodes, all_nodes;
-	hostlist_iterator_t fail_itr;
+	hostlist_t *fail_nodes, *all_nodes;
+	hostlist_iterator_t *fail_itr;
 	char *node;
 	int num_node_ids;
 	int i, j;
@@ -1374,6 +1360,12 @@ _step_missing_handler(struct step_launch_state *sls, slurm_msg_t *missing_msg)
 	int   num_tasks;
 	bool  active;
 
+	if (sls->job_id && (step_missing->step_id.job_id != sls->job_id)) {
+		verbose("Ignoring SRUN_STEP_MISSING for JobId=%u (our JobId=%u)",
+			step_missing->step_id.job_id, sls->job_id);
+		return;
+	}
+
 	debug("Step %ps missing from node(s) %s",
 	      &step_missing->step_id, step_missing->nodelist);
 
@@ -1483,6 +1475,13 @@ static void
 _step_step_signal(struct step_launch_state *sls, slurm_msg_t *signal_msg)
 {
 	job_step_kill_msg_t *step_signal = signal_msg->data;
+
+	if (sls->job_id && (step_signal->step_id.job_id != sls->job_id)) {
+		verbose("Ignoring SRUN_STEP_SIGNAL for JobId=%u (our JobId=%u)",
+			step_signal->step_id.job_id, sls->job_id);
+		return;
+	}
+
 	debug2("Signal %u requested for step %ps", step_signal->signal,
 	       &step_signal->step_id);
 	if (sls->callback.step_signal)
@@ -1561,8 +1560,8 @@ _handle_msg(void *arg, slurm_msg_t *msg)
 		slurm_send_rc_msg(msg, rc);
 		break;
 	default:
-		error("%s: received spurious message type: %u",
-		      __func__, msg->msg_type);
+		error("%s: received spurious message type: %s",
+		      __func__, rpc_num2string(msg->msg_type));
 		break;
 	}
 	return;
@@ -1582,13 +1581,8 @@ static int _fail_step_tasks(slurm_step_ctx_t *ctx, char *node, int ret_code)
 	slurm_msg_t req;
 	step_complete_msg_t msg;
 	int rc = -1;
-	int nodeid = 0;
 	struct step_launch_state *sls = ctx->launch_state;
-
-#ifndef HAVE_FRONT_END
-	/* It is always 0 for front end systems */
-	nodeid = nodelist_find(ctx->step_resp->step_layout->node_list, node);
-#endif
+	int nodeid = nodelist_find(ctx->step_resp->step_layout->node_list, node);
 
 	slurm_mutex_lock(&sls->lock);
 	for (int i = 0; i < sls->layout->tasks[nodeid]; i++) {
@@ -1623,14 +1617,11 @@ static int _fail_step_tasks(slurm_step_ctx_t *ctx, char *node, int ret_code)
 
 static int _launch_tasks(slurm_step_ctx_t *ctx,
 			 launch_tasks_request_msg_t *launch_msg,
-			 uint32_t timeout, char *nodelist)
+			 uint32_t timeout, uint16_t tree_width, char *nodelist)
 {
-#ifdef HAVE_FRONT_END
-	slurm_cred_arg_t *cred_args;
-#endif
 	slurm_msg_t msg;
-	List ret_list = NULL;
-	ListIterator ret_itr;
+	list_t *ret_list = NULL;
+	list_itr_t *ret_itr;
 	ret_data_info_t *ret_data = NULL;
 	int rc = SLURM_SUCCESS;
 	int tot_rc = SLURM_SUCCESS;
@@ -1638,7 +1629,7 @@ static int _launch_tasks(slurm_step_ctx_t *ctx,
 	debug("Entering _launch_tasks");
 	if (ctx->verbose_level) {
 		char *name = NULL;
-		hostlist_t hl = hostlist_create(nodelist);
+		hostlist_t *hl = hostlist_create(nodelist);
 		int i = 0;
 		while ((name = hostlist_shift(hl))) {
 			_print_launch_msg(launch_msg, name, i++);
@@ -1660,29 +1651,14 @@ static int _launch_tasks(slurm_step_ctx_t *ctx,
 	slurm_msg_set_r_uid(&msg, SLURM_AUTH_UID_ANY);
 	msg.msg_type = REQUEST_LAUNCH_TASKS;
 	msg.data = launch_msg;
+	msg.forward.tree_width = tree_width;
 
 	if (ctx->step_resp->use_protocol_ver)
 		msg.protocol_version = ctx->step_resp->use_protocol_ver;
 	else
 		msg.protocol_version = SLURM_PROTOCOL_VERSION;
 
-	/*
-	 * Prior to Slurm 23.02 --slurmd-debug was interpreted as offset to
-	 * LOG_LEVEL_ERROR, so we need to adjust the value.
-	 */
-	if (ctx->step_resp->use_protocol_ver <= SLURM_22_05_PROTOCOL_VERSION) {
-		launch_msg->slurmd_debug-= LOG_LEVEL_ERROR;
-	}
-
-
-#ifdef HAVE_FRONT_END
-	cred_args = slurm_cred_get_args(ctx->step_resp->cred);
-	//info("hostlist=%s", cred_args->step_hostlist);
-	ret_list = slurm_send_recv_msgs(cred_args->step_hostlist, &msg, timeout);
-	slurm_cred_unlock_args(ctx->step_resp->cred);
-#else
 	ret_list = slurm_send_recv_msgs(nodelist, &msg, timeout);
-#endif
 	if (ret_list == NULL) {
 		error("slurm_send_recv_msgs failed miserably: %m");
 		return SLURM_ERROR;
@@ -1739,7 +1715,7 @@ static void _print_launch_msg(launch_tasks_request_msg_t *msg,
 {
 	int i;
 	char *tmp_str = NULL, *task_list = NULL;
-	hostlist_t hl = hostlist_create(NULL);
+	hostlist_t *hl = hostlist_create(NULL);
 
 	for (i=0; i<msg->tasks_to_launch[nodeid]; i++) {
 		xstrfmtcat(tmp_str, "%u", msg->global_task_ids[nodeid][i]);
@@ -1752,8 +1728,6 @@ static void _print_launch_msg(launch_tasks_request_msg_t *msg,
 	info("launching %ps on host %s, %u tasks: %s",
 	     &msg->step_id, hostname, msg->tasks_to_launch[nodeid], task_list);
 	xfree(task_list);
-
-	debug3("uid:%u gid:%u cwd:%s %d", msg->uid, msg->gid, msg->cwd, nodeid);
 }
 
 /*
@@ -1782,7 +1756,7 @@ step_launch_notify_io_failure(step_launch_state_t *sls, int node_id)
 		/* FIXME
 		 * If stepd dies or we see I/O error with stepd.
 		 * Do not abort the whole job but collect all
-		 * taks on the node just like if they exited.
+		 * tasks on the node just like if they exited.
 		 *
 		 * Keep supporting 'srun -N x --pty bash'
 		 */
