@@ -49,10 +49,11 @@
 #include "slurm/slurm_errno.h"
 
 #include "src/common/pack.h"
+#include "src/interfaces/auth.h"
 #include "src/common/slurm_protocol_common.h"
 #include "src/common/slurm_protocol_defs.h"
+#include "src/common/slurm_protocol_socket.h"
 #include "src/common/slurm_protocol_util.h"
-#include "src/common/slurm_protocol_interface.h"
 
 #define MIN_NOALLOC_JOBID ((uint32_t) 0xffff0000)
 #define MAX_NOALLOC_JOBID ((uint32_t) 0xfffffffd)
@@ -123,12 +124,6 @@ extern uint16_t slurm_get_track_wckey(void);
  */
 bool slurm_with_slurmdbd(void);
 
-/* slurm_get_keep_alive_time
- * returns keep_alive_time slurm_conf object
- * RET uint16_t        - keep_alive_time
- */
-uint16_t slurm_get_keep_alive_time(void);
-
 /* slurm_get_preempt_type
  * get PreemptType from slurm_conf object
  * RET char *   - preempt type, MUST be xfreed by caller
@@ -141,46 +136,12 @@ char *slurm_get_preempt_type(void);
  */
 char *slurm_get_acct_gather_interconnect_type(void);
 
-/* slurm_get_acct_filesystem_profile_type
- * get FilesystemAccountingType from slurm_conf object
- * RET char *   - acct_gather_filesystem_type, MUST be xfreed by caller
- */
-char *slurm_get_acct_gather_filesystem_type(void);
-
-
-/* slurm_get_acct_gather_node_freq
- * returns the accounting poll frequency for requesting info from a
- * node from the slurm_conf object
- * RET int    - accounting node frequency
- */
-extern uint16_t slurm_get_acct_gather_node_freq(void);
-
-/* slurm_get_ext_sensors_type
- * get ExtSensorsType from slurm_conf object
- * RET char *   - ext_sensors type, MUST be xfreed by caller
- */
-char *slurm_get_ext_sensors_type(void);
-
-/* slurm_get_ext_sensors_freq
- * returns the external sensors sampling frequency from the slurm_conf
- * object for requesting info from a hardware component (node, switch, etc.)
- * RET int    - external sensors sampling frequency
- */
-extern uint16_t slurm_get_ext_sensors_freq(void);
-
-/* slurm_get_sched_params
- * RET char * - Value of SchedulerParameters, MUST be xfreed by caller */
-extern char *slurm_get_sched_params(void);
-
 /* slurm_get_select_type
  * get select_type from slurm_conf object
  * RET char *   - select_type, MUST be xfreed by caller
  */
 char *slurm_get_select_type(void);
 
-/** Return true if (remote) system runs Cray XT/XE */
-bool is_cray_select_type(void);
-
 /* slurm_get_srun_port_range()
  *
  * Return the array with 2 members indicating the
@@ -188,10 +149,6 @@ bool is_cray_select_type(void);
  */
 uint16_t *slurm_get_srun_port_range(void);
 
-/* slurm_get_core_spec_plugin
- * RET core_spec plugin name, must be xfreed by caller */
-char *slurm_get_core_spec_plugin(void);
-
 /*
  * slurm_get_jobcomp_type
  * returns the configured GpuFreqDef value
@@ -220,25 +177,13 @@ extern int slurm_init_msg_engine_port(uint16_t port);
  */
 extern int slurm_init_msg_engine_ports(uint16_t *);
 
-/* sock_bind_range()
- *
- * Try to bind() sock to any port in a given interval of ports
- */
-extern int sock_bind_range(int, uint16_t *, bool local);
-
 /* In the socket implementation it creates a socket, binds to it, and
  *	listens for connections.
  * IN slurm_address 	- slurm_addr_t to bind the msg server to
+ * IN permissive 	- log failure errors at "error" or "debug" level
  * RET slurm_fd		- file descriptor of the connection created
  */
-extern int slurm_init_msg_engine(slurm_addr_t * slurm_address);
-
-/* In the bsd implmentation maps directly to a accept call
- * IN open_fd		- file descriptor to accept connection on
- * OUT slurm_address 	- slurm_addr_t of the accepted connection
- * RET slurm_fd		- file descriptor of the connection created
- */
-extern int slurm_accept_msg_conn(int open_fd, slurm_addr_t * slurm_address);
+extern int slurm_init_msg_engine(slurm_addr_t *slurm_address, bool permissive);
 
 /**********************************************************************\
  * receive message functions
@@ -252,7 +197,7 @@ extern int slurm_accept_msg_conn(int open_fd, slurm_addr_t * slurm_address);
  * RET int	- returns 0 on success, -1 on failure and sets errno
  */
 
-extern int slurm_unpack_received_msg(slurm_msg_t *msg, int fd, Buf buffer);
+extern int slurm_unpack_received_msg(slurm_msg_t *msg, int fd, buf_t *buffer);
 
 /*
  *  Receive a slurm message on the open slurm descriptor "fd" waiting
@@ -261,14 +206,14 @@ extern int slurm_unpack_received_msg(slurm_msg_t *msg, int fd, Buf buffer);
  *    (msg->data) is allocated from within this function, and must be
  *    freed at some point using one of the slurm_free* functions.
  *    Also a slurm_cred is allocated (msg->auth_cred) which must be
- *    freed with g_slurm_auth_destroy() if it exists.
+ *    freed with auth_g_destroy() if it exists.
  *
- * IN open_fd	- file descriptor to receive msg on
+ * IN tls_conn
  * OUT msg	- a slurm_msg struct to be filled in by the function
  * IN timeout	- how long to wait in milliseconds
  * RET int	- returns 0 on success, -1 on failure and sets errno
  */
-int slurm_receive_msg(int fd, slurm_msg_t *msg, int timeout);
+extern int slurm_receive_msg(void *tls_conn, slurm_msg_t *msg, int timeout);
 
 /*
  *  Receive a slurm message on the open slurm descriptor "fd" waiting
@@ -277,32 +222,34 @@ int slurm_receive_msg(int fd, slurm_msg_t *msg, int timeout);
  *    returned list and must be freed at some point using the
  *    list_destroy function.
  *
- * IN open_fd	- file descriptor to receive msg on
+ * IN tls_conn
  * IN steps	- how many steps down the tree we have to wait for
  * IN timeout	- how long to wait in milliseconds
- * RET List	- List containing the responses of the children (if any) we
- *                forwarded the message to. List containing type
+ * RET list	- list containing the responses of the children (if any) we
+ *                forwarded the message to. list containing type
  *                (ret_data_info_t). NULL is returned on failure. and
  *                errno set.
  */
-List slurm_receive_msgs(int fd, int steps, int timeout);
-List slurm_receive_resp_msgs(int fd, int steps, int timeout);
+extern list_t *slurm_receive_msgs(void *tls_conn, int steps, int timeout);
+extern list_t *slurm_receive_resp_msgs(void *tls_conn, int steps, int timeout);
 
 /*
- *  Receive a slurm message on the open slurm descriptor "fd". This will also
- *  forward the message to the nodes contained in the forward_t structure
- *  inside the header of the message. The 'resp' is the actual message received
- *  and contains the ret_list of it's children and the forward_structure_t
- *  containing information about it's children also. Memory is allocated for
- *  the returned msg and the returned list both must be freed at some point
- *  using the slurm_free_functions and list_destroy function.
+ *  Unpack a Slurm message from the supplied 'buffer' and forward.
+ *  This will forward the message to the nodes contained in the forward_t
+ *  structure inside the header of the message. The 'msg' is the actual message
+ *  received and contains the ret_list of it's children and the
+ *  forward_structure_t containing information about it's children also. Memory
+ *  is allocated for the returned msg and the returned list both must be freed
+ *  at some point using the slurm_free_functions and list_destroy function.
  *
- * IN open_fd	- file descriptor to receive msg on
- * OUT resp	- a slurm_msg struct to be filled in by the function
+ * OUT msg	- a slurm_msg struct to be filled in by the function
+ * IN orig_addr - Original address of the connection msg was received on
+ * IN open_fd	- file descriptor msg was received on
  * RET int	- returns 0 on success, -1 on failure and sets errno
  */
-int slurm_receive_msg_and_forward(int fd, slurm_addr_t *orig_addr,
-				  slurm_msg_t *resp);
+extern int slurm_unpack_msg_and_forward(slurm_msg_t *msg,
+					slurm_addr_t *orig_addr, int fd,
+					buf_t *buffer);
 
 /**********************************************************************\
  * send message functions
@@ -310,46 +257,34 @@ int slurm_receive_msg_and_forward(int fd, slurm_addr_t *orig_addr,
 
 /* sends a message to an arbitrary node
  *
- * IN open_fd		- file descriptor to send msg on
+ * IN tls_conn
  * IN msg		- a slurm msg struct to be sent
  * RET int		- size of msg sent in bytes
  */
-int slurm_send_node_msg(int open_fd, slurm_msg_t *msg);
+extern int slurm_send_node_msg(void *tls_conn, slurm_msg_t *msg);
 
 /**********************************************************************\
  * msg connection establishment functions used by msg clients
 \**********************************************************************/
 
-/*
- * Calls connect to make a connection-less datagram connection to the
- *	primary or secondary slurmctld message engine
- * IN/OUT addr       - address of controller contacted
- * IN/OUT use_backup - IN: whether to try the backup first or not
- *                     OUT: set to true if connection established with backup
- * IN comm_cluster_rec	- Communication record (host/port/version)/
- * RET slurm_fd	- file descriptor of the connection created
- */
-extern int slurm_open_controller_conn(slurm_addr_t *addr, bool *use_backup,
-				      slurmdb_cluster_rec_t *comm_cluster_rec);
-
 /*
  * Calls connect to make a connection-less datagram connection to a specific
  *	primary or backup slurmctld message engine
  * IN dest      - controller to contact (0=primary, 1=backup, 2=backup2, etc.)
  * IN comm_cluster_rec	- Communication record (host/port/version)/
- * RET int      - file descriptor of the connection created
+ * RET tls_conn
  */
-extern int slurm_open_controller_conn_spec(int dest,
-				      slurmdb_cluster_rec_t *comm_cluster_rec);
+extern void *slurm_open_controller(int dest,
+				   slurmdb_cluster_rec_t *comm_cluster_rec);
 
-/* In the bsd socket implementation it creates a SOCK_STREAM socket
- *	and calls connect on it a SOCK_DGRAM socket called with connect
- *	is defined to only receive messages from the address/port pair
- *	argument of the connect call slurm_address
- * IN slurm_address 	- slurm_addr_t of the connection destination
- * RET slurm_fd		- file descriptor of the connection created
+/*
+ * Creates a SOCK_STREAM (TCP) socket and calls connect() on it.
+ * Will only receive messages from the address (w/port) argument.
+ * IN addr - slurm_addr_t of the connection destination
+ * IN tls_cert - public certificate for destination if not already trusted
+ * RET tls_conn	- TLS connection to destination
  */
-extern int slurm_open_msg_conn(slurm_addr_t * slurm_address);
+extern void *slurm_open_msg_conn(slurm_addr_t *addr, char *tls_cert);
 
 /**********************************************************************\
  * stream functions
@@ -357,21 +292,21 @@ extern int slurm_open_msg_conn(slurm_addr_t * slurm_address);
 
 /* slurm_write_stream
  * writes a buffer out a stream file descriptor
- * IN open_fd		- file descriptor to write on
+ * IN tls_conn
  * IN buffer		- buffer to send
  * IN size		- size of buffer send
- * RET size_t		- bytes sent , or -1 on errror
+ * RET size_t		- bytes sent , or -1 on error
  */
-extern size_t slurm_write_stream(int open_fd, char *buffer, size_t size);
+extern size_t slurm_write_stream(void *tls_conn, char *buffer, size_t size);
 
 /* slurm_read_stream
  * read into buffer grom a stream file descriptor
- * IN open_fd		- file descriptor to read from
+ * IN tls_conn
  * OUT buffer	- buffer to receive into
  * IN size		- size of buffer
- * RET size_t		- bytes read , or -1 on errror
+ * RET size_t		- bytes read , or -1 on error
  */
-extern size_t slurm_read_stream(int open_fd, char *buffer, size_t size);
+extern size_t slurm_read_stream(void *tls_conn, char *buffer, size_t size);
 
 /**********************************************************************\
  * address conversion and management functions
@@ -406,25 +341,6 @@ extern int slurm_get_peer_addr(int fd, slurm_addr_t * slurm_address);
  * slurm_addr_t pack routines
 \**********************************************************************/
 
-/* slurm_pack_slurm_addr_array
- * packs an array of slurm_addrs into a buffer (pre-20.11 protocol)
- * OUT slurm_address	- slurm_addr_t to pack
- * IN size_val  	- how many to pack
- * IN/OUT buffer	- buffer to pack the slurm_addr_t from
- * returns		- Slurm error code
- */
-extern void slurm_pack_slurm_addr_array(slurm_addr_t * slurm_address,
-					uint32_t size_val, Buf buffer);
-/* slurm_unpack_slurm_addr_array
- * unpacks an array of slurm_addrs from a buffer (pre-20.11 protocol)
- * OUT slurm_address	- slurm_addr_t to unpack to
- * IN size_val  	- how many to unpack
- * IN/OUT buffer	- buffer to upack the slurm_addr_t from
- * returns		- Slurm error code
- */
-extern int slurm_unpack_slurm_addr_array(slurm_addr_t ** slurm_address,
-					 uint32_t * size_val, Buf buffer);
-
 /* slurm_pack_addr_array
  * packs an array of slurm_addrs into a buffer
  * OUT addr_array	- slurm_addr_t[] to pack
@@ -439,26 +355,59 @@ extern void slurm_pack_addr_array(slurm_addr_t *addr_array, uint32_t size_val,
  * unpacks an array of slurm_addrs from a buffer
  * OUT addr_array_ptr	- slurm_addr_t[] to unpack to
  * IN/OUT size_val  	- how many to unpack
- * IN/OUT buffer	- buffer to upack the slurm_addr_t from
+ * IN/OUT buffer	- buffer to unpack the slurm_addr_t from
  * returns		- Slurm error code
  */
 extern int slurm_unpack_addr_array(slurm_addr_t **addr_array_ptr,
 				   uint32_t *size_val, buf_t *buffer);
 
+/**********************************************************************\
+ * message packing routines
+\**********************************************************************/
+
+/*
+ * Pack message into buffers to be ready to send.
+ *
+ * IN msg - message to pack
+ * IN buffers - buffers to populate with packed message
+ * IN block_for_forwarding - call the forward_wait() which blocks until
+ *    forwarding
+ * RET SLURM_SUCCESS or error
+ */
+extern int slurm_buffers_pack_msg(slurm_msg_t *msg, msg_bufs_t *buffers,
+				  bool block_for_forwarding);
+
 /**********************************************************************\
  * simplified communication routines
  * They open a connection do work then close the connection all within
  * the function
 \**********************************************************************/
 
-/* slurm_send_msg
- * given the original request message this function sends a
- *	arbitrary message back to the client that made the request
- * IN request_msg	- slurm_msg the request msg
- * IN msg_type          - message type being returned
- * IN resp_msg		- the message being returned to the client
+/*
+ * Initialize response msg
+ *
+ * NOTE: This sets SLURM_NO_AUTH_CRED on resp_msg
+ *
+ * OUT resp_msg - pointer to response message
+ * IN source_msg - create response to this message
+ * IN msg_type - RPC message type
+ * IN data - pointer to message data which corresponds to msg_type
  */
-int slurm_send_msg(slurm_msg_t *msg, uint16_t msg_type, void *resp);
+extern void slurm_resp_msg_init(slurm_msg_t *resp_msg, slurm_msg_t *msg,
+				uint16_t msg_type, void *data);
+
+/*
+ * Send message in response to a source message
+ *
+ * Handles creating response slurm_msg_t and sending based on source_msg.
+ *
+ * IN source_msg - Message to send response about
+ * IN msg_type - RPC message type
+ * IN data - pointer to message data which corresponds to msg_type
+ * RET SLURM_SUCCESS or error
+ */
+extern int send_msg_response(slurm_msg_t *source_msg, slurm_msg_type_t msg_type,
+			     void *data);
 
 /* slurm_send_rc_msg
  * given the original request message this function sends a
@@ -504,16 +453,17 @@ int slurm_send_recv_node_msg(slurm_msg_t * request_msg,
 			     int timeout);
 
 /*
- *  Send a message to the nodelist specificed using fanout
- *    Then return List containing type (ret_data_info_t).
+ *  Send a message to the nodelist specified using fanout
+ *    Then return list containing type (ret_data_info_t).
  * IN nodelist	    - list of nodes to send to.
  * IN msg           - a slurm_msg struct to be sent by the function
  * IN timeout	    - how long to wait in milliseconds
- * RET List	    - List containing the responses of the children
- *                    (if any) we forwarded the message to. List
+ * RET list	    - list containing the responses of the children
+ *                    (if any) we forwarded the message to. list
  *                    containing type (ret_types_t).
  */
-List slurm_send_recv_msgs(const char *nodelist, slurm_msg_t *msg, int timeout);
+extern list_t *slurm_send_recv_msgs(const char *nodelist, slurm_msg_t *msg,
+				    int timeout);
 
 /*
  * Sends back reroute_msg_t which directs the client to make the request to
@@ -523,19 +473,29 @@ List slurm_send_recv_msgs(const char *nodelist, slurm_msg_t *msg, int timeout);
  * IN cluster_rec - cluster to direct msg to.
  */
 int slurm_send_reroute_msg(slurm_msg_t *msg,
-			   slurmdb_cluster_rec_t *cluster_rec);
+			   slurmdb_cluster_rec_t *cluster_rec,
+			   char *stepmgr);
+
+/*
+ * Open a connection to the "address" specified in the slurm msg `req'
+ * Then, immediately close the connection w/out waiting for a reply.
+ * Ignore any errors. This should only be used when you do not care if
+ * the message is ever actually received.
+ */
+extern void slurm_send_msg_maybe(slurm_msg_t *req);
 
 /*
  *  Send a message to msg->address
- *    Then return List containing type (ret_data_info_t).
+ *    Then return list containing type (ret_data_info_t).
  * IN msg           - a slurm_msg struct to be sent by the function
  * IN name          - the name of the node the message is being sent to
  * IN timeout	    - how long to wait in milliseconds
- * RET List	    - List containing the responses of the children
- *                    (if any) we forwarded the message to. List
+ * RET list	    - list containing the responses of the children
+ *                    (if any) we forwarded the message to. list
  *                    containing type (ret_types_t).
  */
-List slurm_send_addr_recv_msgs(slurm_msg_t *msg, char *name, int timeout);
+extern list_t *slurm_send_addr_recv_msgs(slurm_msg_t *msg, char *name,
+					 int timeout);
 
 /*
  *  Same as above, but only to one node
@@ -547,7 +507,7 @@ int slurm_send_recv_rc_msg_only_one(slurm_msg_t *req, int *rc, int timeout);
 /*
  * Send message to controller and get return code.
  * Make use of slurm_send_recv_controller_msg(), which handles
- * support for backup controller and retry during transistion.
+ * support for backup controller and retry during transition.
  * IN req - request to send
  * OUT rc - return code
  * IN comm_cluster_rec	- Communication record (host/port/version)
@@ -570,34 +530,19 @@ extern int slurm_send_only_controller_msg(slurm_msg_t *req,
 /* DO NOT USE THIS. See comment in slurm_protocol_api.c for further info. */
 extern int slurm_send_only_node_msg(slurm_msg_t *request_msg);
 
-/*
- * slurm_send_msg_maybe
- * opens a connection, sends a message across while ignoring any errors,
- * then closes the connection
- * IN request_msg	- slurm_msg request
- */
-extern void slurm_send_msg_maybe(slurm_msg_t *request_msg);
-
 /* Send and recv a slurm request and response on the open slurm descriptor
  * Doesn't close the connection.
- * IN fd	- file descriptor to receive msg on
+ * IN tls_conn
  * IN req	- a slurm_msg struct to be sent by the function
  * OUT resp	- a slurm_msg struct to be filled in by the function
  * IN timeout	- how long to wait in milliseconds
  * RET int	- returns 0 on success, -1 on failure and sets errno
  */
-extern int slurm_send_recv_msg(int fd, slurm_msg_t *req,
+extern int slurm_send_recv_msg(void *tls_conn, slurm_msg_t *req,
 			       slurm_msg_t *resp, int timeout);
 
 /* Slurm message functions */
 
-/* set_span
- * build an array indicating how message fanout should occur
- * IN total - total number of nodes to communicate with
- * IN tree_width - message fanout, use system default if zero
- * NOTE: Returned array MUST be release by caller using xfree */
-extern int *set_span(int total, uint16_t tree_width);
-
 extern void slurm_free_msg_members(slurm_msg_t *msg);
 extern void slurm_free_msg(slurm_msg_t * msg);
 
@@ -614,7 +559,6 @@ extern void convert_num_unit(double num, char *buf, int buf_size,
 extern int revert_num_unit(const char *buf);
 extern int get_convert_unit_val(int base_type, char convert_to);
 extern int get_unit_type(char unit);
-extern void parse_int_to_array(int in, int *out);
 
 /*
  * slurm_job_step_create - Ask the slurm controller for a new job step
@@ -665,4 +609,13 @@ extern int slurm_hex_to_char(int v);
  */
 extern int slurm_char_to_hex(int c);
 
+/*
+ * Query assoc share info from slurmctld
+ * IN shares_req - which accounts and users to query
+ * IN/OUT shares_resp - populated result
+ * RET: SLURM_SUCCESS or error
+ */
+extern int slurm_associations_get_shares(shares_request_msg_t *shares_req,
+					 shares_response_msg_t **shares_resp);
+
 #endif
