@@ -124,7 +124,7 @@ static bool _set_max_node_gres(sock_gres_t *sock_gres, uint64_t val)
  * OUT near_gpus - Count of GPUs available on sockets with available CPUs
  * RET - 0 if job can use this node, -1 otherwise (some GRES limit prevents use)
  */
-extern int gres_select_filter_remove_unusable(List sock_gres_list,
+extern int gres_select_filter_remove_unusable(list_t *sock_gres_list,
 					      uint64_t avail_mem,
 					      uint16_t max_cpus,
 					      bool enforce_binding,
@@ -352,8 +352,9 @@ static void _pick_shared_gres_topo(sock_gres_t *sock_gres, bool use_busy_dev,
 		if (use_busy_dev && (gres_ns->topo_gres_cnt_alloc[t] == 0))
 			continue;
 		cnt_avail = gres_ns->topo_gres_cnt_avail[t] -
-			gres_ns->topo_gres_cnt_alloc[t] -
 			gres_js->gres_per_bit_select[node_inx][t];
+		if (!sock_gres->use_total_gres) /* Subtract other jobs */
+			cnt_avail -= gres_ns->topo_gres_cnt_alloc[t];
 		if  (cnt_avail < (use_single_dev ? *gres_needed : 1))
 			continue; /* Insufficient resources */
 		if (!bit_test(sock_bits, t))
@@ -431,7 +432,7 @@ static void _pick_shared_gres(uint64_t *gres_needed, uint32_t *used_sock,
 
 	if (use_single_dev && total_res_gres && (sock_with_res_cnt > 1)) {
 		/*
-		 * Have to allocate gres accross more than one socket.
+		 * Have to allocate gres across more than one socket.
 		 * This is assuming one socket per gres configuration line.
 		 */
 		*satisfy_res_gres = false;
@@ -660,7 +661,7 @@ static int _set_shared_task_bits(int node_inx,
 					      job_id, node_inx);
 					rc = ESLURM_INVALID_GRES;
 				} else if (gres_needed) {
-					error("Not enough shared gres available to satisfy gres per task request for job %u on node %d (%lu/%lu still needed)",
+					error("Not enough shared gres available to satisfy gres per task request for job %u on node %d (%"PRIu64"/%"PRIu64" still needed)",
 					      job_id, node_inx, gres_needed,
 					      gres_js->gres_per_task);
 					rc = ESLURM_INVALID_GRES;
@@ -726,9 +727,11 @@ static uint64_t _pick_gres_topo(sock_gres_t *sock_gres, int gres_needed,
 		int g = sorted_gres ? sorted_gres[i] : i;
 		if (!bit_test(sock_bits, g))
 			continue; /* GRES not on this socket */
-		if (bit_test(gres_js->gres_bit_select[node_inx], g) ||
+		if (bit_test(gres_js->gres_bit_select[node_inx], g))
+			continue; /* Already selected for this job */
+		if (!sock_gres->use_total_gres &&
 		    bit_test(gres_ns->gres_bit_alloc, g))
-			continue; /* Already allocated GRES */
+			continue; /* Already allocated to other jobs */
 		bit_set(gres_js->gres_bit_select[node_inx], g);
 		gres_js->gres_cnt_node_select[node_inx]++;
 		gres_still_needed--;
@@ -982,7 +985,7 @@ static int _set_job_bits1(int node_inx, int job_node_inx, int rem_nodes,
 		 * least one gpu on the node anyway.
 		 * For example --gpus=typeA:2,typeB:1 where there is only one
 		 * typeA on each node then two nodes are required. Because this
-		 * is not a heterogenous job a typeB gpu does have to be
+		 * is not a heterogeneous job a typeB gpu does have to be
 		 * allocated on each node even though they only requested one
 		 * for the job.
 		 */
@@ -1233,7 +1236,7 @@ static void _set_node_bits(int node_inx, int job_node_inx,
 	}
 
 	if (gres_needed) {
-		gres_needed -= _pick_gres_topo(sock_gres, 1, node_inx,
+		gres_needed -= _pick_gres_topo(sock_gres, gres_needed, node_inx,
 					       ANY_SOCK_TEST, sorted_gres,
 					       links_cnt);
 	}
@@ -1245,11 +1248,6 @@ static void _set_node_bits(int node_inx, int job_node_inx,
 		gres_needed -= _pick_gres_topo(sock_gres, gres_needed, node_inx,
 					       s, sorted_gres, links_cnt);
 	}
-	if (gres_needed) {
-		gres_needed -= _pick_gres_topo(sock_gres, gres_needed,
-					       node_inx, ANY_SOCK_TEST,
-					       sorted_gres, links_cnt);
-	}
 
 	/* Use any additional available GRES.  */
 	if (gres_needed && !enforce_binding) {
@@ -1311,7 +1309,7 @@ static void _set_task_bits(int node_inx, sock_gres_t *sock_gres,
 		gres_js->gres_per_task;
 	gres_needed -= total_res_gres;
 
-	/* First pick GRES for acitve sockets */
+	/* First pick GRES for active sockets */
 	for (s = 0; s < sock_cnt; s++) {
 		if (!tasks_per_socket[s])
 			continue;
@@ -1353,12 +1351,13 @@ static void _set_task_bits(int node_inx, sock_gres_t *sock_gres,
 }
 
 /* Build array to identify task count for each node-socket pair */
-static uint32_t **_build_tasks_per_node_sock(struct job_resources *job_res,
-					     uint8_t overcommit,
-					     gres_mc_data_t *tres_mc_ptr)
+static int _build_tasks_per_node_sock(struct job_resources *job_res,
+				      uint8_t overcommit,
+				      gres_mc_data_t *tres_mc_ptr,
+				      uint32_t ***tasks_per_node_socket_ptr)
 {
 	uint32_t **tasks_per_node_socket;
-	int j, node_cnt, job_node_inx = 0;
+	int rc = SLURM_SUCCESS, j, node_cnt, job_node_inx = 0;
 	int c, s, core_offset;
 	int cpus_per_task = 1, cpus_per_node, cpus_per_core;
 	int task_per_node_limit = 0;
@@ -1384,10 +1383,16 @@ static uint32_t **_build_tasks_per_node_sock(struct job_resources *job_res,
 		tasks_per_node_socket[i] = xcalloc(sock_cnt, sizeof(uint32_t));
 		if (tres_mc_ptr->ntasks_per_node) {
 			task_per_node_limit = tres_mc_ptr->ntasks_per_node;
+			cpus_per_task = MAX(1, job_res->cpus[job_node_inx] /
+						       job_res->tasks_per_node
+							       [job_node_inx]);
 		} else if (job_res->tasks_per_node &&
 			   job_res->tasks_per_node[job_node_inx]) {
 			task_per_node_limit =
 				job_res->tasks_per_node[job_node_inx];
+			cpus_per_task =
+				MAX(1, job_res->cpus[job_node_inx] /
+				    job_res->tasks_per_node[job_node_inx]);
 		} else {
 			/*
 			 * NOTE: We should never get here.
@@ -1499,10 +1504,13 @@ static uint32_t **_build_tasks_per_node_sock(struct job_resources *job_res,
 			}
 		}
 	}
-	if (rem_tasks > 0)	/* This should never happen */
+	if (rem_tasks > 0) { /* This should never happen */
 		error("%s: rem_tasks not zero (%d > 0)", __func__, rem_tasks);
+		rc = ESLURM_INVALID_GRES;
+	}
 
-	return tasks_per_node_socket;
+	*tasks_per_node_socket_ptr = tasks_per_node_socket;
+	return rc;
 }
 
 static void _free_tasks_per_node_sock(uint32_t **tasks_per_node_socket,
@@ -1688,8 +1696,8 @@ static int _set_res_core_bits(uint32_t **res_gres_per_sock,
 		if (*sock_with_res_cnt > 1 &&
 		    !(slurm_conf.select_type_param & MULTIPLE_SHARING_GRES_PJ)){
 			/*
-			 * Have to allocate gres accross more then one socket.
-			 * This is assuming one socket per gres configueration
+			 * Have to allocate gres across more then one socket.
+			 * This is assuming one socket per gres configuration
 			 * line.
 			 */
 			error("Restricted gpu cores on multiple sockets which requires MULTIPLE_SHARING_GRES_PJ to be set");
@@ -1782,8 +1790,12 @@ static int _select_and_set_node(void *x, void *arg)
 	}
 	if (gres_js->gres_per_task && /* Data needed */
 	    !*args->tasks_per_node_socket) { /* Not built yet */
-		*args->tasks_per_node_socket = _build_tasks_per_node_sock(
-			job_res, job_ptr->details->overcommit, tres_mc_ptr);
+		*rc = _build_tasks_per_node_sock(job_res,
+						 job_ptr->details->overcommit,
+						 tres_mc_ptr,
+						 args->tasks_per_node_socket);
+		if (*rc != SLURM_SUCCESS)
+			return -1;
 	}
 
 	tasks_per_node_socket = *args->tasks_per_node_socket;
@@ -1907,7 +1919,7 @@ static int _select_and_set_node(void *x, void *arg)
  * tres_mc_ptr IN - job's multi-core options
  * RET SLURM_SUCCESS or error code
  */
-extern int gres_select_filter_select_and_set(List *sock_gres_list,
+extern int gres_select_filter_select_and_set(list_t **sock_gres_list,
 					     job_record_t *job_ptr,
 					     gres_mc_data_t *tres_mc_ptr)
 {
