@@ -1,7 +1,7 @@
 /*****************************************************************************\
  *  common.c - definitions for functions common to all modules in sacctmgr.
  *****************************************************************************
- *  Copyright (C) 2010-2015 SchedMD LLC.
+ *  Copyright (C) SchedMD LLC.
  *  Copyright (C) 2008 Lawrence Livermore National Security.
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
@@ -39,18 +39,33 @@
 \*****************************************************************************/
 
 #include "src/sacctmgr/sacctmgr.h"
+#include "src/common/macros.h"
+#include "src/common/parse_value.h"
 #include "src/common/slurmdbd_defs.h"
-#include "src/common/slurm_auth.h"
+#include "src/interfaces/auth.h"
+#include "src/common/slurm_protocol_defs.h"
 
 #include <unistd.h>
 #include <termios.h>
 
-static pthread_t lock_warning_thread;
+static bool warn_needed = false;
+static pthread_mutex_t warn_mutex = PTHREAD_MUTEX_INITIALIZER;
+static pthread_cond_t warn_cond = PTHREAD_COND_INITIALIZER;
+static pthread_t notice_thread_handler = 0;
 
 static void *_print_lock_warn(void *no_data)
 {
-	sleep(5);
-	printf(" Database is busy or waiting for lock from other user.\n");
+	struct timespec ts = { 0, 0 };
+	ts.tv_sec = time(NULL) + 5;
+
+	slurm_mutex_lock(&warn_mutex);
+	if (warn_needed) {
+		slurm_cond_timedwait(&warn_cond, &warn_mutex, &ts);
+		if (warn_needed)
+			printf(" Database is busy or waiting for lock from other user.\n");
+		warn_needed = false;
+	}
+	slurm_mutex_unlock(&warn_mutex);
 
 	return NULL;
 }
@@ -118,6 +133,11 @@ extern char *strip_quotes(char *option, int *increased, bool make_lower)
 		quote = 1;
 		i++;
 	}
+
+	/* skip beginning spaces */
+	while (option[i] && isspace(option[i]))
+		i++;
+
 	start = i;
 
 	while(option[i]) {
@@ -134,6 +154,11 @@ extern char *strip_quotes(char *option, int *increased, bool make_lower)
 
 		i++;
 	}
+
+	/* trim end spaces */
+	while ((i - 1) && option[i - 1] && isspace(option[i - 1]))
+		i--;
+
 	end += i;
 
 	meat = xmalloc((i-start)+1);
@@ -191,9 +216,9 @@ static print_field_t *_get_print_field(char *object)
 		field->print_routine = print_fields_str;
 	} else if (!xstrncasecmp("Allowed", object, MAX(command_len, 2))) {
 		field->type = PRINT_ALLOWED;
-		field->name = xstrdup("% Allowed");
-		field->len = 10;
-		field->print_routine = print_fields_uint16;
+		field->name = xstrdup("Allowed");
+		field->len = 8;
+		field->print_routine = print_fields_uint32;
 	} else if (!xstrncasecmp("Associations", object, MAX(command_len, 2))) {
 		field->type = PRINT_ASSOC_NAME;
 		field->name = xstrdup("Assocs");
@@ -220,11 +245,16 @@ static print_field_t *_get_print_field(char *object)
 		field->name = xstrdup("Cluster");
 		field->len = 10;
 		field->print_routine = print_fields_str;
-	} else if (!xstrncasecmp("Coordinators", object, MAX(command_len, 2))) {
+	} else if (!xstrncasecmp("Coordinators", object, MAX(command_len, 3))) {
 		field->type = PRINT_COORDS;
 		field->name = xstrdup("Coord Accounts");
 		field->len = 20;
 		field->print_routine = sacctmgr_print_coord_list;
+	} else if (!xstrncasecmp("Comment", object, MAX(command_len, 3))) {
+		field->type = PRINT_COMMENT;
+		field->name = xstrdup("Comment");
+		field->len = 20;
+		field->print_routine = print_fields_str;
 	} else if (!xstrncasecmp("ControlHost", object, MAX(command_len, 8))) {
 		field->type = PRINT_CHOST;
 		field->name = xstrdup("ControlHost");
@@ -292,6 +322,11 @@ static print_field_t *_get_print_field(char *object)
 		field->name = xstrdup("Event");
 		field->len = 7;
 		field->print_routine = print_fields_str;
+	} else if (!xstrncasecmp("Extra", object, MAX(command_len, 2))) {
+		field->type = PRINT_EXTRA;
+		field->name = xstrdup("Extra");
+		field->len = 20;
+		field->print_routine = print_fields_str;
 	} else if (!xstrncasecmp("Features", object, MAX(command_len, 3))) {
 		field->type = PRINT_FEATURES;
 		field->name = xstrdup("Features");
@@ -389,16 +424,26 @@ static print_field_t *_get_print_field(char *object)
 		field->name = xstrdup("ID");
 		field->len = 6;
 		field->print_routine = print_fields_uint;
-	} else if (!xstrncasecmp("Info", object, MAX(command_len, 2))) {
+	} else if (!xstrncasecmp("Info", object, MAX(command_len, 3))) {
 		field->type = PRINT_INFO;
 		field->name = xstrdup("Info");
 		field->len = 20;
 		field->print_routine = print_fields_str;
-	} else if (!xstrncasecmp("LFT", object, MAX(command_len, 1))) {
-		field->type = PRINT_LFT;
-		field->name = xstrdup("LFT");
-		field->len = 6;
-		field->print_routine = print_fields_uint;
+	} else if (!xstrncasecmp("InstanceId", object, MAX(command_len, 9))) {
+		field->type = PRINT_INSTANCE_ID;
+		field->name = xstrdup("InstanceId");
+		field->len = 20;
+		field->print_routine = print_fields_str;
+	} else if (!xstrncasecmp("InstanceType", object, MAX(command_len, 9))) {
+		field->type = PRINT_INSTANCE_TYPE;
+		field->name = xstrdup("InstanceType");
+		field->len = 20;
+		field->print_routine = print_fields_str;
+	} else if (!xstrncasecmp("Lineage", object, MAX(command_len, 1))) {
+		field->type = PRINT_LINEAGE;
+		field->name = xstrdup("Lineage");
+		field->len = -20;
+		field->print_routine = print_fields_str;
 	} else if (!xstrncasecmp("servertype", object, MAX(command_len, 10))) {
 		field->type = PRINT_SERVERTYPE;
 		field->name = xstrdup("ServerType");
@@ -434,6 +479,8 @@ static print_field_t *_get_print_field(char *object)
 		field->print_routine = print_fields_uint;
 	} else if (!xstrncasecmp("MaxTRES", object,
 				 MAX(command_len, 7)) ||
+		   !xstrncasecmp("MaxTRESPJ", object,
+				 MAX(command_len, 9)) ||
 		   !xstrncasecmp("MaxTRESPerJob", object,
 				 MAX(command_len, 11))) {
 		field->type = PRINT_MAXT;
@@ -441,13 +488,17 @@ static print_field_t *_get_print_field(char *object)
 		field->len = 13;
 		field->print_routine = sacctmgr_print_tres;
 	} else if (!xstrncasecmp("MaxTRESPerNode", object,
-				 MAX(command_len, 11))) {
+				 MAX(command_len, 11)) ||
+		   !xstrncasecmp("MaxTRESPN", object,
+				 MAX(command_len, 9))) {
 		field->type = PRINT_MAXTN;
 		field->name = xstrdup("MaxTRESPerNode");
 		field->len = 14;
 		field->print_routine = sacctmgr_print_tres;
 	} else if (!xstrncasecmp("MaxTRESMinsPerJob", object,
-				 MAX(command_len, 8))) {
+				 MAX(command_len, 8)) ||
+		   !xstrncasecmp("MaxTRESMinsPJ", object,
+				 MAX(command_len, 13))) {
 		field->type = PRINT_MAXTM;
 		field->name = xstrdup("MaxTRESMins");
 		field->len = 13;
@@ -460,7 +511,7 @@ static print_field_t *_get_print_field(char *object)
 				 MAX(command_len, 15))) {
 		field->type = PRINT_MAXTRMA;
 		field->name = xstrdup("MaxTRESRunMinsPA");
-		field->len = 15;
+		field->len = 16;
 		field->print_routine = sacctmgr_print_tres;
 	} else if (!xstrncasecmp("MaxTRESRunMinsPerUser", object,
 				 MAX(command_len, 8)) ||
@@ -468,7 +519,7 @@ static print_field_t *_get_print_field(char *object)
 				 MAX(command_len, 8))) {
 		field->type = PRINT_MAXTRM;
 		field->name = xstrdup("MaxTRESRunMinsPU");
-		field->len = 15;
+		field->len = 16;
 		field->print_routine = sacctmgr_print_tres;
 	} else if (!xstrncasecmp("MaxTRESPerAccount", object,
 				 MAX(command_len, 11)) ||
@@ -628,12 +679,12 @@ static print_field_t *_get_print_field(char *object)
 		field->print_routine = print_fields_str;
 	} else if (!xstrncasecmp("ParentID", object, MAX(command_len, 7))) {
 		field->type = PRINT_PID;
-		field->name = xstrdup("Par ID");
-		field->len = 6;
+		field->name = xstrdup("ParentID");
+		field->len = 8;
 		field->print_routine = print_fields_uint;
 	} else if (!xstrncasecmp("ParentName", object, MAX(command_len, 7))) {
 		field->type = PRINT_PNAME;
-		field->name = xstrdup("Par Name");
+		field->name = xstrdup("ParentName");
 		field->len = 10;
 		field->print_routine = print_fields_str;
 	} else if (!xstrncasecmp("Partition", object, MAX(command_len, 4))) {
@@ -641,12 +692,6 @@ static print_field_t *_get_print_field(char *object)
 		field->name = xstrdup("Partition");
 		field->len = 10;
 		field->print_routine = print_fields_str;
-	} else if (!xstrncasecmp("PluginIDSelect", object,
-				 MAX(command_len, 2))) {
-		field->type = PRINT_SELECT;
-		field->name = xstrdup("PluginIDSelect");
-		field->len = 14;
-		field->print_routine = print_fields_uint;
 	} else if (!xstrncasecmp("PreemptMode", object, MAX(command_len, 8))) {
 		field->type = PRINT_PREEM;
 		field->name = xstrdup("PreemptMode");
@@ -657,7 +702,7 @@ static print_field_t *_get_print_field(char *object)
 		field->type = PRINT_PREE;
 		field->name = xstrdup("Preempt");
 		field->len = 10;
-		field->print_routine = sacctmgr_print_qos_bitstr;
+		field->print_routine = print_fields_str;
 	} else if (!xstrncasecmp("PreemptExemptTime", object,
 				 MAX(command_len, 8))) {
 		field->type = PRINT_PRXMPT;
@@ -678,7 +723,7 @@ static print_field_t *_get_print_field(char *object)
 		field->type = PRINT_QOS;
 		field->name = xstrdup("QOS");
 		field->len = 20;
-		field->print_routine = sacctmgr_print_qos_list;
+		field->print_routine = print_fields_str;
 	} else if (!xstrncasecmp("QOSRAWLevel", object, MAX(command_len, 4))) {
 		field->type = PRINT_QOS_RAW;
 		field->name = xstrdup("QOS_RAW");
@@ -690,11 +735,6 @@ static print_field_t *_get_print_field(char *object)
 		field->name = xstrdup("Reason");
 		field->len = 30;
 		field->print_routine = print_fields_str;
-	} else if (!xstrncasecmp("RGT", object, MAX(command_len, 1))) {
-		field->type = PRINT_RGT;
-		field->name = xstrdup("RGT");
-		field->len = 6;
-		field->print_routine = print_fields_uint;
 	} else if (!xstrncasecmp("RPC", object, MAX(command_len, 1))) {
 		field->type = PRINT_RPC_VERSION;
 		field->name = xstrdup("RPC");
@@ -778,11 +818,21 @@ static print_field_t *_get_print_field(char *object)
 		field->name = xstrdup("UsageThres");
 		field->len = 10;
 		field->print_routine = print_fields_double;
+	} else if (!xstrncasecmp("LimitFactor", object, MAX(command_len, 6))) {
+		field->type = PRINT_LF;
+		field->name = xstrdup("LimitFactor");
+		field->len = 11;
+		field->print_routine = print_fields_double;
 	} else if (!xstrncasecmp("Allocated", object, MAX(command_len, 7))) {
 		field->type = PRINT_ALLOCATED;
-		field->name = xstrdup("% Allocated");
-		field->len = 11;
-		field->print_routine = print_fields_uint16;
+		field->name = xstrdup("Allocated");
+		field->len = 9;
+		field->print_routine = print_fields_uint32;
+	} else if (!xstrncasecmp("LastConsumed", object, MAX(command_len, 8))) {
+		field->type = PRINT_LAST_CONSUMED;
+		field->name = xstrdup("LastConsumed");
+		field->len = 12;
+		field->print_routine = print_fields_uint32;
 	} else if (!xstrncasecmp("User", object, MAX(command_len, 1))) {
 		field->type = PRINT_USER;
 		field->name = xstrdup("User");
@@ -809,15 +859,23 @@ static print_field_t *_get_print_field(char *object)
 	return field;
 }
 
-extern void notice_thread_init()
+extern void notice_thread_init(void)
 {
-	slurm_thread_create_detached(&lock_warning_thread,
-				     _print_lock_warn, NULL);
+	slurm_mutex_lock(&warn_mutex);
+	warn_needed = true;
+	slurm_thread_create(&notice_thread_handler, _print_lock_warn, NULL);
+	slurm_mutex_unlock(&warn_mutex);
 }
 
-extern void notice_thread_fini()
+extern void notice_thread_fini(void)
 {
-	pthread_cancel(lock_warning_thread);
+	slurm_mutex_lock(&warn_mutex);
+	warn_needed = false;
+	slurm_cond_broadcast(&warn_cond);
+	slurm_mutex_unlock(&warn_mutex);
+
+	if (notice_thread_handler)
+		slurm_thread_join(notice_thread_handler);
 }
 
 extern int commit_check(char *warning)
@@ -863,12 +921,12 @@ extern int commit_check(char *warning)
 
 extern int sacctmgr_remove_assoc_usage(slurmdb_assoc_cond_t *assoc_cond)
 {
-	List update_list = NULL;
-	List local_assoc_list = NULL;
-	List local_cluster_list = NULL;
-	ListIterator itr = NULL;
-	ListIterator itr2 = NULL;
-	ListIterator itr3 = NULL;
+	list_t *update_list = NULL;
+	list_t *local_assoc_list = NULL;
+	list_t *local_cluster_list = NULL;
+	list_itr_t *itr = NULL;
+	list_itr_t *itr2 = NULL;
+	list_itr_t *itr3 = NULL;
 	char *account = NULL;
 	char *cluster = NULL;
 	char *user = NULL;
@@ -888,16 +946,10 @@ extern int sacctmgr_remove_assoc_usage(slurmdb_assoc_cond_t *assoc_cond)
 		assoc_cond->cluster_list = list_create(xfree_ptr);
 
 	if (!list_count(assoc_cond->cluster_list)) {
-		char *temp = slurm_get_cluster_name();
-		if (temp) {
-			printf("No cluster specified, resetting "
-			       "on local cluster %s\n", temp);
-			list_append(assoc_cond->cluster_list, temp);
-		}
-		if (!list_count(assoc_cond->cluster_list)) {
-			error("A cluster name is required to remove usage");
-			return SLURM_ERROR;
-		}
+		printf("No cluster specified, resetting on local cluster %s\n",
+		       slurm_conf.cluster_name);
+		list_append(assoc_cond->cluster_list,
+			    xstrdup(slurm_conf.cluster_name));
 	}
 
 	if (!commit_check("Would you like to reset usage?")) {
@@ -999,13 +1051,14 @@ end_it:
 	return rc;
 }
 
-extern int sacctmgr_remove_qos_usage(slurmdb_qos_cond_t *qos_cond)
+extern int sacctmgr_update_qos_usage(slurmdb_qos_cond_t *qos_cond,
+				     long double new_raw_usage)
 {
-	List update_list = NULL;
-	List cluster_list;
-	List local_qos_list = NULL;
-	List local_cluster_list = NULL;
-	ListIterator itr = NULL, itr2 = NULL;
+	list_t *update_list = NULL;
+	list_t *cluster_list;
+	list_t *local_qos_list = NULL;
+	list_t *local_cluster_list = NULL;
+	list_itr_t *itr = NULL, *itr2 = NULL;
 	char *qos_name = NULL, *cluster_name = NULL;
 	slurmdb_qos_rec_t* rec = NULL;
 	slurmdb_cluster_rec_t* cluster_rec = NULL;
@@ -1020,19 +1073,12 @@ extern int sacctmgr_remove_qos_usage(slurmdb_qos_cond_t *qos_cond)
 		cluster_list = list_create(xfree_ptr);
 
 	if (!list_count(cluster_list)) {
-		char *temp = slurm_get_cluster_name();
-		if (temp) {
-			printf("No cluster specified, resetting "
-			       "on local cluster %s\n", temp);
-			list_append(cluster_list, temp);
-		}
-		if (!list_count(cluster_list)) {
-			error("A cluster name is required to remove usage");
-			return SLURM_ERROR;
-		}
+		printf("No cluster specified, resetting on local cluster %s\n",
+		       slurm_conf.cluster_name);
+		list_append(cluster_list, xstrdup(slurm_conf.cluster_name));
 	}
 
-	if (!commit_check("Would you like to reset usage?")) {
+	if (!commit_check("Would you like to update usage?")) {
 		printf(" Changes Discarded\n");
 		return rc;
 	}
@@ -1057,7 +1103,7 @@ extern int sacctmgr_remove_qos_usage(slurmdb_qos_cond_t *qos_cond)
 
 		update_list = list_create(slurmdb_destroy_update_object);
 		update_obj = xmalloc(sizeof(slurmdb_update_object_t));
-		update_obj->type = SLURMDB_REMOVE_QOS_USAGE;
+		update_obj->type = SLURMDB_UPDATE_QOS_USAGE;
 		update_obj->objects = list_create(NULL);
 
 		while ((qos_name = list_next(itr2))) {
@@ -1069,6 +1115,9 @@ extern int sacctmgr_remove_qos_usage(slurmdb_qos_cond_t *qos_cond)
 				slurmdb_destroy_update_object(update_obj);
 				goto end_it;
 			}
+			if (!rec->usage)
+				rec->usage = xmalloc(sizeof(*rec->usage));
+			rec->usage->usage_raw = new_raw_usage;
 			list_append(update_obj->objects, rec);
 		}
 		list_iterator_reset(itr2);
@@ -1102,7 +1151,7 @@ extern slurmdb_assoc_rec_t *sacctmgr_find_account_base_assoc(
 	slurmdb_assoc_rec_t *assoc = NULL;
 	char *temp = "root";
 	slurmdb_assoc_cond_t assoc_cond;
-	List assoc_list = NULL;
+	list_t *assoc_list = NULL;
 
 	if (!cluster)
 		return NULL;
@@ -1142,7 +1191,7 @@ extern slurmdb_user_rec_t *sacctmgr_find_user(char *name)
 	slurmdb_user_rec_t *user = NULL;
 	slurmdb_user_cond_t user_cond;
 	slurmdb_assoc_cond_t assoc_cond;
-	List user_list = NULL;
+	list_t *user_list = NULL;
 
 	if (!name)
 		return NULL;
@@ -1170,7 +1219,7 @@ extern slurmdb_account_rec_t *sacctmgr_find_account(char *name)
 	slurmdb_account_rec_t *account = NULL;
 	slurmdb_account_cond_t account_cond;
 	slurmdb_assoc_cond_t assoc_cond;
-	List account_list = NULL;
+	list_t *account_list = NULL;
 
 	if (!name)
 		return NULL;
@@ -1197,7 +1246,7 @@ extern slurmdb_cluster_rec_t *sacctmgr_find_cluster(char *name)
 {
 	slurmdb_cluster_rec_t *cluster = NULL;
 	slurmdb_cluster_cond_t cluster_cond;
-	List cluster_list = NULL;
+	list_t *cluster_list = NULL;
 
 	if (!name)
 		return NULL;
@@ -1219,10 +1268,10 @@ extern slurmdb_cluster_rec_t *sacctmgr_find_cluster(char *name)
 }
 
 extern slurmdb_assoc_rec_t *sacctmgr_find_assoc_from_list(
-	List assoc_list, char *user, char *account,
+	list_t *assoc_list, char *user, char *account,
 	char *cluster, char *partition)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_assoc_rec_t * assoc = NULL;
 
 	if (!assoc_list)
@@ -1256,9 +1305,9 @@ extern slurmdb_assoc_rec_t *sacctmgr_find_assoc_from_list(
 }
 
 extern slurmdb_assoc_rec_t *sacctmgr_find_account_base_assoc_from_list(
-	List assoc_list, char *account, char *cluster)
+	list_t *assoc_list, char *account, char *cluster)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_assoc_rec_t *assoc = NULL;
 	char *temp = "root";
 
@@ -1285,9 +1334,9 @@ extern slurmdb_assoc_rec_t *sacctmgr_find_account_base_assoc_from_list(
 }
 
 extern slurmdb_qos_rec_t *sacctmgr_find_qos_from_list(
-	List qos_list, char *name)
+	list_t *qos_list, char *name)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_qos_rec_t *qos = NULL;
 	char *working_name = NULL;
 
@@ -1311,9 +1360,9 @@ extern slurmdb_qos_rec_t *sacctmgr_find_qos_from_list(
 }
 
 extern slurmdb_res_rec_t *sacctmgr_find_res_from_list(
-	List res_list, uint32_t id, char *name, char *server)
+	list_t *res_list, uint32_t id, char *name, char *server)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_res_rec_t *res = NULL;
 
 	if ((id == NO_VAL) && (!name || !res_list))
@@ -1333,9 +1382,9 @@ extern slurmdb_res_rec_t *sacctmgr_find_res_from_list(
 }
 
 extern slurmdb_user_rec_t *sacctmgr_find_user_from_list(
-	List user_list, char *name)
+	list_t *user_list, char *name)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_user_rec_t *user = NULL;
 
 	if (!name || !user_list)
@@ -1353,9 +1402,9 @@ extern slurmdb_user_rec_t *sacctmgr_find_user_from_list(
 }
 
 extern slurmdb_account_rec_t *sacctmgr_find_account_from_list(
-	List acct_list, char *name)
+	list_t *acct_list, char *name)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_account_rec_t *account = NULL;
 
 	if (!name || !acct_list)
@@ -1373,9 +1422,9 @@ extern slurmdb_account_rec_t *sacctmgr_find_account_from_list(
 }
 
 extern slurmdb_cluster_rec_t *sacctmgr_find_cluster_from_list(
-	List cluster_list, char *name)
+	list_t *cluster_list, char *name)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_cluster_rec_t *cluster = NULL;
 
 	if (!name || !cluster_list)
@@ -1392,9 +1441,9 @@ extern slurmdb_cluster_rec_t *sacctmgr_find_cluster_from_list(
 }
 
 extern slurmdb_wckey_rec_t *sacctmgr_find_wckey_from_list(
-	List wckey_list, char *user, char *name, char *cluster)
+	list_t *wckey_list, char *user, char *name, char *cluster)
 {
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	slurmdb_wckey_rec_t * wckey = NULL;
 
 	if (!wckey_list)
@@ -1495,16 +1544,15 @@ extern int get_uint64(char *in_value, uint64_t *out_value, char *type)
 
 extern int get_double(char *in_value, double *out_value, char *type)
 {
-	char *ptr = NULL, *meat = NULL;
+	char *meat = NULL;
 	double num;
 
 	if (!(meat = strip_quotes(in_value, NULL, 1))) {
 		error("Problem with strip_quotes");
 		return SLURM_ERROR;
 	}
-	num = strtod(meat, &ptr);
-	if ((num == 0) && ptr && ptr[0]) {
-		error("Invalid value for %s (%s)", type, meat);
+
+	if (s_p_handle_double(&num, type, meat)) {
 		xfree(meat);
 		return SLURM_ERROR;
 	}
@@ -1517,113 +1565,53 @@ extern int get_double(char *in_value, double *out_value, char *type)
 	return SLURM_SUCCESS;
 }
 
-extern int addto_action_char_list(List char_list, char *names)
+static int _addto_action_char_list_internal(list_t *char_list, char *name,
+					    void *x)
 {
-	int i=0, start=0;
-	char *name = NULL, *tmp_char = NULL;
-	ListIterator itr = NULL;
-	char quote_c = '\0';
-	int quote = 0;
-	uint32_t id=0;
-	int count = 0;
+	uint32_t id = 0;
+	char *tmp_name = NULL;
 
+	id = str_2_slurmdbd_msg_type(name);
+	if (id == NO_VAL) {
+		error("You gave a bad action '%s'.", name);
+		list_flush(char_list);
+		return SLURM_ERROR;
+	}
+
+	tmp_name = xstrdup_printf("%u", id);
+
+	if (!list_find_first(char_list, slurm_find_char_in_list, tmp_name)) {
+		list_append(char_list, tmp_name);
+		return 1;
+	} else {
+		xfree(tmp_name);
+		return 0;
+	}
+}
+
+extern int addto_action_char_list(list_t *char_list, char *names)
+{
 	if (!char_list) {
 		error("No list was given to fill in");
 		return 0;
 	}
 
-	itr = list_iterator_create(char_list);
-	if (names) {
-		if (names[i] == '\"' || names[i] == '\'') {
-			quote_c = names[i];
-			quote = 1;
-			i++;
-		}
-		start = i;
-		while(names[i]) {
-			if (quote && names[i] == quote_c)
-				break;
-			else if (names[i] == '\"' || names[i] == '\'')
-				names[i] = '`';
-			else if (names[i] == ',') {
-				if ((i-start) > 0) {
-					name = xmalloc((i-start+1));
-					memcpy(name, names+start, (i-start));
-
-					id = str_2_slurmdbd_msg_type(name);
-					if (id == NO_VAL) {
-						error("You gave a bad action "
-						      "'%s'.", name);
-						xfree(name);
-						break;
-					}
-					xfree(name);
-
-					name = xstrdup_printf("%u", id);
-					while((tmp_char = list_next(itr))) {
-						if (!xstrcasecmp(tmp_char,
-								 name))
-							break;
-					}
-					list_iterator_reset(itr);
-
-					if (!tmp_char) {
-						list_append(char_list, name);
-						count++;
-					} else
-						xfree(name);
-				}
-
-				i++;
-				start = i;
-				if (!names[i]) {
-					error("There is a problem with "
-					      "your request.  It appears you "
-					      "have spaces inside your list.");
-					break;
-				}
-			}
-			i++;
-		}
-		if ((i-start) > 0) {
-			name = xmalloc((i-start)+1);
-			memcpy(name, names+start, (i-start));
-
-			id = str_2_slurmdbd_msg_type(name);
-			if (id == NO_VAL)  {
-				error("You gave a bad action '%s'.",
-				      name);
-				xfree(name);
-				goto end_it;
-			}
-			xfree(name);
-
-			name = xstrdup_printf("%u", id);
-			while((tmp_char = list_next(itr))) {
-				if (!xstrcasecmp(tmp_char, name))
-					break;
-			}
-
-			if (!tmp_char) {
-				list_append(char_list, name);
-				count++;
-			} else
-				xfree(name);
-		}
-	}
-end_it:
-	list_iterator_destroy(itr);
-	return count;
+	return slurm_parse_char_list(char_list, names, NULL,
+				     _addto_action_char_list_internal);
 }
 
 extern void sacctmgr_print_coord_list(
-	print_field_t *field, List value, int last)
+	print_field_t *field, void *input, int last)
 {
 	int abs_len = abs(field->len);
-	ListIterator itr = NULL;
+	list_itr_t *itr = NULL;
 	char *print_this = NULL;
+	list_t *value = NULL;
 	slurmdb_coord_rec_t *object = NULL;
 
+	if (input)
+		value = *(list_t **)input;
+
 	if (!value || !list_count(value)) {
 		if (print_fields_parsable_print)
 			print_this = xstrdup("");
@@ -1659,68 +1647,19 @@ extern void sacctmgr_print_coord_list(
 	xfree(print_this);
 }
 
-extern void sacctmgr_print_qos_list(print_field_t *field, List qos_list,
-				    List value, int last)
-{
-	int abs_len = abs(field->len);
-	char *print_this = NULL;
-
-	print_this = get_qos_complete_str(qos_list, value);
-
-	if (print_fields_parsable_print == PRINT_FIELDS_PARSABLE_NO_ENDING
-	    && last)
-		printf("%s", print_this);
-	else if (print_fields_parsable_print)
-		printf("%s|", print_this);
-	else {
-		if (strlen(print_this) > abs_len)
-			print_this[abs_len-1] = '+';
-
-		if (field->len == abs_len)
-			printf("%*.*s ", abs_len, abs_len, print_this);
-		else
-			printf("%-*.*s ", abs_len, abs_len, print_this);
-	}
-	xfree(print_this);
-}
-
-extern void sacctmgr_print_qos_bitstr(print_field_t *field, List qos_list,
-				      bitstr_t *value, int last)
-{
-	int abs_len = abs(field->len);
-	char *print_this = NULL;
-
-	print_this = get_qos_complete_str_bitstr(qos_list, value);
-
-	if (print_fields_parsable_print == PRINT_FIELDS_PARSABLE_NO_ENDING
-	    && last)
-		printf("%s", print_this);
-	else if (print_fields_parsable_print)
-		printf("%s|", print_this);
-	else {
-		if (strlen(print_this) > abs_len)
-			print_this[abs_len-1] = '+';
-
-		if (field->len == abs_len)
-			printf("%*.*s ", abs_len, abs_len, print_this);
-		else
-			printf("%-*.*s ", abs_len, abs_len, print_this);
-	}
-	xfree(print_this);
-}
-
-extern void sacctmgr_print_tres(print_field_t *field, char *tres_simple_str,
-				int last)
+extern void sacctmgr_print_tres(print_field_t *field, void *input, int last)
 {
 	int abs_len = abs(field->len);
 	char *print_this;
+	char *value = NULL;
+
+	if (input)
+		value = *(char **) input;
 
 	sacctmgr_initialize_g_tres_list();
 
 	print_this = slurmdb_make_tres_string_from_simple(
-		tres_simple_str, g_tres_list, NO_VAL, CONVERT_NUM_UNIT_EXACT, 0,
-		NULL);
-
+		value, g_tres_list, NO_VAL, CONVERT_NUM_UNIT_EXACT, 0, NULL);
 
 	if (!print_this)
 		print_this = xstrdup("");
@@ -1898,6 +1837,10 @@ extern void sacctmgr_print_assoc_limits(slurmdb_assoc_rec_t *assoc)
 		       slurmdb_qos_str(g_qos_list, assoc->def_qos_id));
 	}
 
+	/* This should be last because it might be long */
+	if (assoc->comment)
+		printf("  Comment       = %s\n", assoc->comment);
+
 }
 
 static int _print_cluster_features(void *object, void *arg)
@@ -1960,7 +1903,7 @@ extern void sacctmgr_print_federation(slurmdb_federation_rec_t *fed)
 		xfree(tmp_flags);
 	}
 	if (fed->cluster_list) {
-		ListIterator itr = list_iterator_create(fed->cluster_list);
+		list_itr_t *itr = list_iterator_create(fed->cluster_list);
 		slurmdb_cluster_rec_t *cluster = NULL;
 		while ((cluster = list_next(itr))) {
 			char *tmp_name = cluster->name;
@@ -2203,6 +2146,11 @@ extern void sacctmgr_print_qos_limits(slurmdb_qos_rec_t *qos)
 	else if (qos->usage_thres != NO_VAL)
 		printf("  UsageThreshold           = %.4lf\n", qos->usage_thres);
 
+	if (qos->limit_factor == INFINITE)
+		printf("  LimitFactor              = NONE\n");
+	else if(qos->limit_factor != NO_VAL)
+		printf("  LimitFactor              = %.4lf\n", qos->limit_factor);
+
 }
 
 extern int sort_coord_list(void *a, void *b)
@@ -2221,10 +2169,10 @@ extern int sort_coord_list(void *a, void *b)
 	return 0;
 }
 
-extern List sacctmgr_process_format_list(List format_list)
+extern list_t *sacctmgr_process_format_list(list_t *format_list)
 {
-	List print_fields_list = list_create(destroy_print_field);
-	ListIterator itr = list_iterator_create(format_list);
+	list_t *print_fields_list = list_create(destroy_print_field);
+	list_itr_t *itr = list_iterator_create(format_list);
 	print_field_t *field = NULL;
 	char *object = NULL;
 
@@ -2239,12 +2187,12 @@ extern List sacctmgr_process_format_list(List format_list)
 	return print_fields_list;
 }
 
-extern int sacctmgr_validate_cluster_list(List cluster_list)
+extern int sacctmgr_validate_cluster_list(list_t *cluster_list)
 {
-	List temp_list = NULL;
+	list_t *temp_list = NULL;
 	char *cluster = NULL;
 	int rc = SLURM_SUCCESS;
-	ListIterator itr = NULL, itr_c = NULL;
+	list_itr_t *itr = NULL, *itr_c = NULL;
 
 	xassert(cluster_list);
 
