@@ -3,7 +3,7 @@
  *****************************************************************************
  *  Copyright (C) 2002-2007 The Regents of the University of California.
  *  Copyright (C) 2008-2009 Lawrence Livermore National Security.
- *  Copyright (C) 2010-2015 SchedMD LLC.
+ *  Copyright (C) SchedMD LLC.
  *  Produced at Lawrence Livermore National Laboratory (cf, DISCLAIMER).
  *  Written by Mark Grondona <grondona1@llnl.gov>, et. al.
  *  CODE-OCEC-09-009. All rights reserved.
@@ -52,6 +52,7 @@
 
 #include "src/common/log.h"
 #include "src/common/macros.h"
+#include "src/common/read_config.h"
 #include "src/common/slurm_protocol_defs.h"
 #include "src/common/xmalloc.h"
 #include "src/common/xstring.h"
@@ -66,6 +67,8 @@
 #define OPT_LONG_CTLD    0x102
 #define OPT_LONG_WCKEY   0x103
 #define OPT_LONG_SIBLING 0x104
+#define OPT_LONG_ME      0x105
+#define OPT_LONG_AUTOCOMP 0x106
 
 /* forward declarations of static functions
  *
@@ -123,6 +126,7 @@ extern bool has_default_opt(void)
 {
 	if (opt.account == NULL
 	    && opt.batch == false
+	    && !opt.ctld
 	    && opt.interactive == false
 	    && opt.job_name == NULL
 	    && opt.partition == NULL
@@ -179,19 +183,8 @@ static void _opt_default(void)
 	opt.account	= NULL;
 	opt.batch	= false;
 	opt.clusters    = NULL;
-#ifdef HAVE_FRONT_END
-	opt.ctld	= true;
-#else
-{
-	char *launch_type = slurm_get_launch_type();
-	/* do this for all but slurm (poe, aprun, etc...) */
-	if (xstrcmp(launch_type, "launch/slurm"))
-		opt.ctld	= true;
-	else
-		opt.ctld	= false;
-	xfree(launch_type);
-}
-#endif
+	opt.ctld	= false;
+	opt.cron = false;
 	opt.full	= false;
 	opt.hurry	= false;
 	opt.interactive	= false;
@@ -211,6 +204,19 @@ static void _opt_default(void)
 	opt.wckey	= NULL;
 }
 
+static void _opt_clusters(char *clusters)
+{
+	opt.ctld = true;
+	FREE_NULL_LIST(opt.clusters);
+
+	if (slurm_get_cluster_info(&(opt.clusters), clusters, 0)) {
+		print_db_notok(clusters, 0);
+		fatal("Could not get cluster information");
+	}
+
+	working_cluster_rec = list_peek(opt.clusters);
+}
+
 /*
  * opt_env(): used by initialize_and_process_args to set options via
  *            environment variables. See comments above for how to
@@ -221,7 +227,8 @@ static void _opt_env(void)
 	char *val;
 
 	if ( (val=getenv("SCANCEL_ACCOUNT")) ) {
-		opt.account = xstrtolower(xstrdup(val));
+		opt.account = xstrdup(val);
+		xstrtolower(opt.account);
 	}
 
 	if ( (val=getenv("SCANCEL_BATCH")) ) {
@@ -240,6 +247,9 @@ static void _opt_env(void)
 	if (getenv("SCANCEL_CTLD"))
 		opt.ctld = true;
 
+	if (getenv("SCANCEL_CRON"))
+		opt.cron = true;
+
 	if ( (val=getenv("SCANCEL_FULL")) ) {
 		if (xstrcasecmp(val, "true") == 0)
 			opt.full       = true;
@@ -279,7 +289,8 @@ static void _opt_env(void)
 	}
 
 	if ( (val=getenv("SCANCEL_QOS")) ) {
-		opt.qos = xstrtolower(xstrdup(val));
+		opt.qos = xstrdup(val);
+		xstrtolower(opt.qos);
 	}
 
 	if ( (val=getenv("SCANCEL_STATE")) ) {
@@ -307,6 +318,28 @@ static void _opt_env(void)
 	if ( (val=getenv("SCANCEL_WCKEY")) ) {
 		opt.wckey = xstrdup(val);
 	}
+
+	if ((val = getenv("SLURM_CLUSTERS"))) {
+		/*
+		 * We must pass in a modifiable string, and we don't want to
+		 * modify the environment.
+		 */
+		char *valdup = xstrdup(val);
+		_opt_clusters(valdup);
+		xfree(valdup);
+	}
+}
+
+static void _handle_nodelist(void)
+{
+	/* If nodelist contains a '/', treat it as a file name */
+	if (strchr(opt.nodelist, '/') != NULL) {
+		char *reallist = slurm_read_hostfile(opt.nodelist, NO_VAL);
+		if (reallist) {
+			xfree(opt.nodelist);
+			opt.nodelist = reallist;
+		}
+	}
 }
 
 /*
@@ -317,9 +350,11 @@ static void _opt_args(int argc, char **argv)
 	char **rest = NULL;
 	int opt_char, option_index;
 	static struct option long_options[] = {
+		{"autocomplete", required_argument, 0, OPT_LONG_AUTOCOMP},
 		{"account",	required_argument, 0, 'A'},
 		{"batch",	no_argument,       0, 'b'},
 		{"ctld",	no_argument,	   0, OPT_LONG_CTLD},
+		{"cron",	no_argument,	   0, 'c'},
 		{"full",	no_argument,       0, 'f'},
 		{"help",        no_argument,       0, OPT_LONG_HELP},
 		{"hurry",       no_argument,       0, 'H'},
@@ -327,6 +362,7 @@ static void _opt_args(int argc, char **argv)
 		{"cluster",     required_argument, 0, 'M'},
 		{"clusters",    required_argument, 0, 'M'},
 		{"jobname",     required_argument, 0, 'n'},
+		{"me",		no_argument,       0, OPT_LONG_ME},
 		{"name",        required_argument, 0, 'n'},
 		{"nodelist",    required_argument, 0, 'w'},
 		{"partition",   required_argument, 0, 'p'},
@@ -344,7 +380,8 @@ static void _opt_args(int argc, char **argv)
 		{NULL,          0,                 0, 0}
 	};
 
-	while ((opt_char = getopt_long(argc, argv, "A:bfHiM:n:p:Qq:R:s:t:u:vVw:",
+	while ((opt_char = getopt_long(argc, argv,
+				       "A:bcfHiM:n:p:Qq:R:s:t:u:vVw:",
 				       long_options, &option_index)) != -1) {
 		switch (opt_char) {
 		case (int)'?':
@@ -354,7 +391,8 @@ static void _opt_args(int argc, char **argv)
 			exit(1);
 			break;
 		case (int)'A':
-			opt.account = xstrtolower(xstrdup(optarg));
+			opt.account = xstrdup(optarg);
+			xstrtolower(opt.account);
 			break;
 		case (int)'b':
 			opt.batch = true;
@@ -362,6 +400,9 @@ static void _opt_args(int argc, char **argv)
 		case OPT_LONG_CTLD:
 			opt.ctld = true;
 			break;
+		case (int)'c':
+			opt.cron = true;
+			break;
 		case (int)'f':
 			opt.full = true;
 			break;
@@ -372,14 +413,10 @@ static void _opt_args(int argc, char **argv)
 			opt.interactive = true;
 			break;
 		case (int)'M':
-			opt.ctld = true;
-			FREE_NULL_LIST(opt.clusters);
-			opt.clusters = slurmdb_get_info_cluster(optarg);
-			if (!opt.clusters) {
-				print_db_notok(optarg, 0);
-				exit(1);
-			}
-			working_cluster_rec = list_peek(opt.clusters);
+			_opt_clusters(optarg);
+			break;
+		case OPT_LONG_ME:
+			opt.user_name = xstrdup_printf("%u", getuid());
 			break;
 		case (int)'n':
 			opt.job_name = xstrdup(optarg);
@@ -391,7 +428,8 @@ static void _opt_args(int argc, char **argv)
 			opt.verbose = -1;
 			break;
 		case (int)'q':
-			opt.qos = xstrtolower(xstrdup(optarg));
+			opt.qos = xstrdup(optarg);
+			xstrtolower(opt.qos);
 			break;
 		case (int)'R':
 			opt.reservation = xstrdup(optarg);
@@ -431,6 +469,10 @@ static void _opt_args(int argc, char **argv)
 		case OPT_LONG_USAGE:
 			_usage();
 			exit(0);
+		case OPT_LONG_AUTOCOMP:
+			suggest_completion(long_options, optarg);
+			exit(0);
+			break;
 		}
 	}
 
@@ -439,6 +481,8 @@ static void _opt_args(int argc, char **argv)
 
 	if (rest)
 		opt.job_list = _xlate_job_step_ids(rest);
+	if (opt.nodelist)
+		_handle_nodelist();
 
 	if (!_opt_verify())
 		exit(1);
@@ -479,7 +523,7 @@ _xlate_job_step_ids(char **rest)
 		opt.job_id[buf_offset] = job_id;
 
 		if ((next_str[0] == '_') && (next_str[1] == '[')) {
-			hostlist_t hl;
+			hostlist_t *hl;
 			char save_char, *next_elem;
 			char *end_char = strchr(next_str + 2, ']');
 			if (!end_char || (end_char[1] != '\0')) {
@@ -576,7 +620,14 @@ _opt_verify(void)
 	bool verified = true;
 
 	if (opt.user_name) {	/* translate to user_id */
-		if ( uid_from_string( opt.user_name, &opt.user_id ) != 0 ) {
+		int rc;
+
+		/*
+		 * Allow numeric uids that no longer exist on underlying system
+		 * so that any old jobs that still use them can be identified.
+		 */
+		rc = uid_from_string(opt.user_name, &opt.user_id);
+		if ((rc != SLURM_SUCCESS) && (rc != ESLURM_USER_ID_UNKNOWN)) {
 			error("Invalid user name: %s", opt.user_name);
 			return false;
 		}
@@ -608,6 +659,7 @@ static void _opt_list(void)
 	info("account        : %s", opt.account);
 	info("batch          : %s", tf_(opt.batch));
 	info("ctld           : %s", tf_(opt.ctld));
+	info("cron           : %s", tf_(opt.cron));
 	info("full           : %s", tf_(opt.full));
 	info("hurry          : %s", tf_(opt.hurry));
 	info("interactive    : %s", tf_(opt.interactive));
@@ -638,16 +690,26 @@ static void _opt_list(void)
 				     i, opt.job_id[i], opt.array_id[i]);
 			}
 		} else {
+			char tmp_char[23];
+			slurm_step_id_t tmp_step_id = {
+				.job_id = opt.job_id[i],
+				.step_het_comp = NO_VAL,
+				.step_id = opt.step_id[i],
+			};
+			log_build_step_id_str(&tmp_step_id, tmp_char,
+					      sizeof(tmp_char),
+					      (STEP_ID_FLAG_NO_PREFIX |
+					       STEP_ID_FLAG_NO_JOB));
 			if (opt.array_id[i] == NO_VAL) {
-				info("job_step_id[%d] : %u.%u",
-				     i, opt.job_id[i], opt.step_id[i]);
+				info("job_step_id[%d] : %u.%s",
+				     i, opt.job_id[i], tmp_char);
 			} else if (opt.array_id[i] == INFINITE) {
-				info("job_step_id[%d] : %u_*.%u",
-				     i, opt.job_id[i], opt.step_id[i]);
+				info("job_step_id[%d] : %u_*.%s",
+				     i, opt.job_id[i], tmp_char);
 			} else {
-				info("job_step_id[%d] : %u_%u.%u",
+				info("job_step_id[%d] : %u_%u.%s",
 				     i, opt.job_id[i], opt.array_id[i],
-				     opt.step_id[i]);
+				     tmp_char);
 			}
 		}
 	}
@@ -655,8 +717,8 @@ static void _opt_list(void)
 
 static void _usage(void)
 {
-	printf("Usage: scancel [-A account] [--batch] [--full] [--interactive] [-n job_name]\n");
-	printf("               [-p partition] [-Q] [-q qos] [-R reservation][-s signal | integer]\n");
+	printf("Usage: scancel [-A account] [--batch] [--ctld] [--full] [--interactive] [-n job_name]\n");
+	printf("               [-p partition] [-Q] [-q qos] [-R reservation] [-s signal | integer]\n");
 	printf("               [-t PENDING | RUNNING | SUSPENDED] [--usage] [-u user_name]\n");
 	printf("               [--hurry] [-V] [-v] [-w hosts...] [--wckey=wckey]\n");
 	printf("               [job_id[_array_id][.step_id]]\n");
@@ -667,7 +729,8 @@ static void _help(void)
 	printf("Usage: scancel [OPTIONS] [job_id[_array_id][.step_id]]\n");
 	printf("  -A, --account=account           act only on jobs charging this account\n");
 	printf("  -b, --batch                     signal batch shell for specified job\n");
-/*	printf("      --ctld                      send request directly to slurmctld\n"); */
+	printf("      --ctld                      send request directly to slurmctld\n");
+	printf("  -c, --cron                      cancel an scrontab job\n");
 	printf("  -f, --full                      signal batch shell and all steps for specified job\n");
 	printf("  -H, --hurry                     avoid burst buffer stage out\n");
 	printf("  -i, --interactive               require response from user for each job\n");
