@@ -72,7 +72,7 @@ static int  _confirmation(job_info_t *job_ptr, uint32_t step_id,
 			  uint32_t array_id);
 static void _filter_job_records(void);
 static void _load_job_records (void);
-static int  _multi_cluster(List clusters);
+static int _multi_cluster(list_t *clusters);
 static int  _proc_cluster(void);
 static int  _signal_job_by_str(void);
 static int  _verify_job_ids(void);
@@ -316,8 +316,7 @@ static int _ctld_signal_jobs(void)
 }
 
 /* _multi_cluster - process job cancellation across a list of clusters */
-static int
-_multi_cluster(List clusters)
+static int _multi_cluster(list_t *clusters)
 {
 	list_itr_t *itr;
 	int rc = 0, rc2;
@@ -342,7 +341,11 @@ _proc_cluster(void)
 		rc = _signal_job_by_str();
 		return rc;
 	}
-	if (opt.ctld && !has_job_steps())
+	/*
+	 * TODO:
+	 * Remove sibling restriction once --ctld has this logic implemented.
+	 */
+	if (opt.ctld && !(opt.sibling || has_job_steps()))
 		return _ctld_signal_jobs();
 
 	_load_job_records();
@@ -407,7 +410,7 @@ static int _verify_job_ids(void)
 	opt.job_pend  = xmalloc(sizeof(bool) * opt.job_cnt);
 	job_ptr = job_buffer_ptr->job_array;
 	for (i = 0; i < job_buffer_ptr->record_count; i++, job_ptr++) {
-		/* NOTE: We re-use the job's "assoc_id" value as a flag to
+		/* NOTE: We reuse the job's "assoc_id" value as a flag to
 		 * record if the job is referenced in the job list supplied
 		 * by the user. */
 		job_ptr->assoc_id = 0;
@@ -923,12 +926,12 @@ _cancel_job_id (void *ci)
 		sleep(5 + i);
 	}
 	if (error_code) {
-		error_code = slurm_get_errno();
+		error_code = errno;
 		if ((opt.verbose > 0) ||
 		    ((error_code != ESLURM_ALREADY_DONE) &&
 		     (error_code != ESLURM_INVALID_JOB_ID))) {
 			_log_kill_job_error(cancel_info->job_id_str,
-					    slurm_strerror(slurm_get_errno()));
+					    slurm_strerror(error_code));
 		}
 		if (((error_code == ESLURM_ALREADY_DONE) ||
 		     (error_code == ESLURM_INVALID_JOB_ID)) &&
@@ -1014,11 +1017,11 @@ _cancel_step_id (void *ci)
 		sleep(5 + i);
 	}
 	if (error_code) {
-		error_code = slurm_get_errno();
+		error_code = errno;
 		if ((opt.verbose > 0) || (error_code != ESLURM_ALREADY_DONE))
 			error("Kill job error on job step id %s: %s",
 		 	      cancel_info->job_id_str,
-			      slurm_strerror(slurm_get_errno()));
+			      slurm_strerror(error_code));
 
 		if ((error_code == ESLURM_ALREADY_DONE) &&
 		    (cancel_info->sig == SIGKILL)) {
