@@ -1,8 +1,7 @@
 /*****************************************************************************\
  *  container.c - slurmstepd container handling
  *****************************************************************************
- *  Copyright (C) 2021 SchedMD LLC.
- *  Written by Nathan Rini <nate@schedmd.com>
+ *  Copyright (C) SchedMD LLC.
  *
  *  This file is part of Slurm, a resource management program.
  *  For details, see <https://slurm.schedmd.com/>.
@@ -64,6 +63,12 @@
 #define SLURM_CONTAINER_STDOUT "/tmp/slurm/stdout"
 #define SLURM_CONTAINER_STDERR "/tmp/slurm/stderr"
 
+#define SLURM_CONTAINER_BATCH_STEP_PATTERN "oci-job%j-batch"
+#define SLURM_CONTAINER_INTERACT_STEP_PATTERN "oci-job%j-interactive"
+#define SLURM_CONTAINER_EXTERN_STEP_PATTERN "oci-job%j-extern"
+#define SLURM_CONTAINER_STEP_PATTERN "oci-job%j-%s"
+#define SLURM_CONTAINER_TASK_PATTERN "task-%t"
+
 oci_conf_t *oci_conf = NULL;
 
 static char *create_argv[] = {
@@ -151,7 +156,7 @@ static char *_generate_pattern(const char *pattern, stepd_step_rec_t *step,
 				break;
 			case 'e':
 				xstrfmtcatat(buffer, &offset, "%s/%s",
-					     c->spool_dir,
+					     c->task_spool_dir,
 					     SLURM_CONTAINER_ENV_FILE);
 				break;
 			case 'j':
@@ -378,9 +383,8 @@ static int _modify_config(stepd_step_rec_t *step, stepd_step_task_info_t *task)
 
 	xassert(c->magic == STEP_CONTAINER_MAGIC);
 
-	/* Disable terminal to ensure stdin/err/out are used */
 	data_set_bool(data_define_dict_path(c->config, "/process/terminal/"),
-		      false);
+		      (step->flags & LAUNCH_PTY));
 
 	/* point to correct rootfs */
 	data_set_string(data_define_dict_path(c->config, "/root/path/"),
@@ -558,11 +562,11 @@ static int _generate_container_paths(stepd_step_rec_t *step)
 			_generate_pattern(oci_conf->mount_spool_dir, step,
 					  step->task[0]->id, NULL);
 	} else {
-		c->mount_spool_dir = xstrdup("/var/run/slurm/");
+		c->mount_spool_dir = _generate_spooldir(step, NULL);
 	}
 
-	xfree(c->spool_dir);
-	c->spool_dir = _generate_spooldir(step, NULL);
+	if (!c->spool_dir)
+		c->spool_dir = _generate_spooldir(step, NULL);
 
 	if ((rc = _mkpath(c->spool_dir, step->uid, step->gid)))
 		fatal("%s: unable to create spool directory %s: %s",
@@ -590,21 +594,52 @@ static bool _pattern_has_taskid(const char *pattern)
 	return false;
 }
 
+static char *_generate_spooldir_pattern(stepd_step_rec_t *step,
+					stepd_step_task_info_t *task)
+{
+	const char *step_id = NULL;
+	const char *task_id = NULL;
+	char *parent = NULL;
+
+	if (oci_conf->container_path) {
+		parent = xstrdup(oci_conf->container_path);
+	} else if (conf->spooldir) {
+#ifdef MULTIPLE_SLURMD
+		parent = slurm_conf_expand_slurmd_path(conf->spooldir,
+						       conf->node_name,
+						       conf->hostname);
+#else /* !MULTIPLE_SLURMD */
+		parent = xstrdup(conf->spooldir);
+#endif /* !MULTIPLE_SLURMD */
+	} else {
+		parent = xstrdup(DEFAULT_SPOOLDIR);
+	}
+
+	if (step->step_id.step_id == SLURM_BATCH_SCRIPT) {
+		step_id = SLURM_CONTAINER_BATCH_STEP_PATTERN;
+	} else if (step->step_id.step_id == SLURM_INTERACTIVE_STEP) {
+		step_id = SLURM_CONTAINER_INTERACT_STEP_PATTERN;
+	} else if (step->step_id.step_id == SLURM_EXTERN_CONT) {
+		step_id = SLURM_CONTAINER_EXTERN_STEP_PATTERN;
+	} else {
+		step_id = SLURM_CONTAINER_STEP_PATTERN;
+	}
+
+	if (task)
+		task_id = SLURM_CONTAINER_TASK_PATTERN;
+	else
+		task_id = "";
+
+	return xstrdup_printf("%s%s%s%s%s", parent, (step_id[0] ? "/" : ""),
+			      step_id, (task_id[0] ? "/" : ""), task_id);
+}
+
 static char *_generate_spooldir(stepd_step_rec_t *step,
 				stepd_step_task_info_t *task)
 {
 	int id = -1;
-	char **argv = NULL, *pattern, *path;
-
-	if (oci_conf->container_path) {
-		pattern = xstrdup(oci_conf->container_path);
-	} else if (step->step_id.step_id == SLURM_BATCH_SCRIPT) {
-		pattern = xstrdup("%m/oci-job%j-batch/task-%t/");
-	} else if (step->step_id.step_id == SLURM_INTERACTIVE_STEP) {
-		pattern = xstrdup("%m/oci-job%j-interactive/task-%t/");
-	} else {
-		pattern = xstrdup("%m/oci-job%j-%s/task-%t/");
-	}
+	char **argv = NULL, *path = NULL;
+	char *pattern = _generate_spooldir_pattern(step, task);
 
 	if (task) {
 		id = task->id;
@@ -665,15 +700,15 @@ extern void container_task_init(stepd_step_rec_t *step,
 		return;
 	}
 
-	xassert(c->spool_dir);
-	xfree(c->spool_dir);
+	if (c->task_spool_dir || !c->spool_dir)
+		fatal_abort("task spool dir already set or spool dir not set");
 
-	/* re-generate out the spool_dir now we know the task */
-	c->spool_dir = _generate_spooldir(step, task);
+	/* generate the task_spool_dir now we know the task */
+	c->task_spool_dir = _generate_spooldir(step, task);
 
-	if ((rc = _mkpath(c->spool_dir, step->uid, step->gid)))
+	if ((rc = _mkpath(c->task_spool_dir, step->uid, step->gid)))
 		fatal("%s: unable to create spool directory %s: %s",
-		      __func__, c->spool_dir, slurm_strerror(rc));
+		      __func__, c->task_spool_dir, slurm_strerror(rc));
 }
 
 static char *_get_config_path(stepd_step_rec_t *step)
@@ -686,7 +721,7 @@ static char *_get_config_path(stepd_step_rec_t *step)
 
 	xassert(c->magic == STEP_CONTAINER_MAGIC);
 
-	/* OCI runtime spec reqires config.json to be in root of bundle */
+	/* OCI runtime spec requires config.json to be in root of bundle */
 	xstrfmtcat(path, "%s/config.json", c->bundle);
 
 	return path;
@@ -752,17 +787,7 @@ extern int setup_container(stepd_step_rec_t *step)
 		return ESLURM_CONTAINER_NOT_CONFIGURED;
 	}
 
-	if ((rc = data_init())) {
-		error("Unable to init data structures: %s",
-		      slurm_strerror(rc));
-		goto error;
-	}
-
-	if ((rc = serializer_g_init(MIME_TYPE_JSON_PLUGIN, NULL))) {
-		error("Unable to load JSON plugin: %s",
-		      slurm_strerror(rc));
-		goto error;
-	}
+	serializer_required(MIME_TYPE_JSON);
 
 	if (!oci_conf->ignore_config_json) {
 		if ((rc = _load_config(step)))
@@ -873,7 +898,7 @@ static void _kill_container()
 			 * use exp backoff up to 1s to wait for the container to
 			 * cleanup.
 			 *
-			 * OCI runtime doesnt provide any way but to poll to see
+			 * OCI runtime doesn't provide any way but to poll to see
 			 * if the container has been squashed
 			 */
 			debug("%s: sleeping %dusec to query state again",
@@ -991,7 +1016,7 @@ static void _create_start(stepd_step_rec_t *step,
 
 		xfree(status);
 
-		/* increate wait times exp */
+		/* increase wait times exp */
 		if (stime > 1000000)
 			stime = 1000000;
 		else
@@ -1012,15 +1037,12 @@ static void _create_start(stepd_step_rec_t *step,
 static void _generate_patterns(stepd_step_rec_t *step,
 			       stepd_step_task_info_t *task)
 {
-	static bool generated = false;
 	char *gen;
 	int id = -1;
 	char **argv = NULL;
 
-	if (generated)
-		return;
-
-	generated = true;
+	debug2("%s: %ps TaskId=%d",
+	       __func__, &step->step_id, (task ? task->id : -1));
 
 	if (task) {
 		id = task->id;
@@ -1028,29 +1050,58 @@ static void _generate_patterns(stepd_step_rec_t *step,
 	}
 
 	gen = _generate_pattern(oci_conf->runtime_create, step, id, argv);
-	if (gen)
+	if (gen) {
+		static bool set = false;
+		if (set)
+			xfree(create_argv[2]);
 		create_argv[2] = gen;
+		set = true;
+	}
 
 	gen = _generate_pattern(oci_conf->runtime_delete, step, id, argv);
-	if (gen)
+	if (gen) {
+		static bool set = false;
+		if (set)
+			xfree(delete_argv[2]);
 		delete_argv[2] = gen;
+		set = true;
+	}
 
 	gen = _generate_pattern(oci_conf->runtime_kill, step, id, argv);
-	if (gen)
+	if (gen) {
+		static bool set = false;
+		if (set)
+			xfree(kill_argv[2]);
 		kill_argv[2] = gen;
+		set = true;
+	}
 
 	gen = _generate_pattern(oci_conf->runtime_query, step, id, argv);
-	if (gen)
+	if (gen) {
+		static bool set = false;
+		if (set)
+			xfree(query_argv[2]);
 		query_argv[2] = gen;
+		set = true;
+	}
 
 	gen = _generate_pattern(oci_conf->runtime_run, step, id, argv);
-
-	if (gen)
+	if (gen) {
+		static bool set = false;
+		if (set)
+			xfree(run_argv[2]);
 		run_argv[2] = gen;
+		set = true;
+	}
 
 	gen = _generate_pattern(oci_conf->runtime_start, step, id, argv);
-	if (gen)
+	if (gen) {
+		static bool set = false;
+		if (set)
+			xfree(start_argv[2]);
 		start_argv[2] = gen;
+		set = true;
+	}
 }
 
 extern void container_run(stepd_step_rec_t *step,
@@ -1067,6 +1118,9 @@ extern void container_run(stepd_step_rec_t *step,
 		return;
 	}
 
+	if (!c->spool_dir || !c->task_spool_dir)
+		fatal_abort("spool directory not populated");
+
 	if (oci_conf->env_exclude_set) {
 		char **env = env_array_exclude((const char **) step->env,
 					       &oci_conf->env_exclude);
@@ -1082,7 +1136,7 @@ extern void container_run(stepd_step_rec_t *step,
 		char *jconfig = NULL;
 
 		/* create new config.json in spooldir */
-		xstrfmtcat(jconfig, "%s/config.json", c->spool_dir);
+		xstrfmtcat(jconfig, "%s/config.json", c->task_spool_dir);
 
 		if ((rc = _modify_config(step, task)))
 			fatal("%s: configuring container failed: %s",
@@ -1090,7 +1144,7 @@ extern void container_run(stepd_step_rec_t *step,
 
 		if ((rc = serialize_g_data_to_string(&out, NULL, c->config,
 						     MIME_TYPE_JSON,
-						     SER_FLAGS_PRETTY))) {
+						     SER_FLAGS_NONE))) {
 			fatal("%s: serialization of config failed: %s",
 			      __func__, slurm_strerror(rc));
 		}
@@ -1108,7 +1162,7 @@ extern void container_run(stepd_step_rec_t *step,
 		 * correct config.json
 		 */
 		xfree(c->bundle);
-		c->bundle = xstrdup(c->spool_dir);
+		c->bundle = xstrdup(c->task_spool_dir);
 
 		xfree(out);
 		xfree(jconfig);
@@ -1120,7 +1174,7 @@ extern void container_run(stepd_step_rec_t *step,
 			   NEWLINE_TERMINATED_ENV_FILE);
 
 		/* keep _generate_pattern() in sync with this path */
-		xstrfmtcat(envfile, "%s/%s", c->spool_dir,
+		xstrfmtcat(envfile, "%s/%s", c->task_spool_dir,
 			   SLURM_CONTAINER_ENV_FILE);
 
 		if ((rc = env_array_to_file(envfile, (const char **) step->env,
@@ -1150,10 +1204,10 @@ extern void container_run(stepd_step_rec_t *step,
 		environ = env;
 	}
 
-	debug4("%s: setting cwd from %s to %s",
-	       __func__, step->cwd, c->spool_dir);
+	debug4("%s: setting cwd from %s to task spooldir: %s",
+	       __func__, step->cwd, c->task_spool_dir);
 	xfree(step->cwd);
-	step->cwd = xstrdup(c->spool_dir);
+	step->cwd = xstrdup(c->task_spool_dir);
 
 	_generate_patterns(step, task);
 
@@ -1177,23 +1231,29 @@ extern void cleanup_container(stepd_step_rec_t *step)
 
 	/* cleanup may be called without ever setting up container */
 
+	_generate_patterns(step, NULL);
 	_kill_container();
 
 	if (oci_conf->disable_cleanup)
 		goto done;
 
+	if (!c->spool_dir)
+		c->spool_dir = _generate_spooldir(step, NULL);
+
 	if (step->node_tasks > 0) {
 		/* clear every config.json and task dir */
 		for (int i = 0; i < step->node_tasks; i++) {
-			xfree(c->spool_dir);
-			c->spool_dir = _generate_spooldir(step, step->task[i]);
+			xfree(c->task_spool_dir);
+			c->task_spool_dir =
+				_generate_spooldir(step, step->task[i]);
+
 			_generate_patterns(step, step->task[i]);
 
 			if (!oci_conf->ignore_config_json) {
 				char *jconfig = NULL;
 
 				xstrfmtcat(jconfig, "%s/config.json",
-					   c->spool_dir);
+					   c->task_spool_dir);
 
 				if ((unlink(jconfig) < 0) && (errno != ENOENT))
 					error("unlink(%s): %m", jconfig);
@@ -1203,7 +1263,7 @@ extern void cleanup_container(stepd_step_rec_t *step)
 			if (oci_conf->create_env_file) {
 				char *envfile = NULL;
 
-				xstrfmtcat(envfile, "%s/%s", c->spool_dir,
+				xstrfmtcat(envfile, "%s/%s", c->task_spool_dir,
 					   SLURM_CONTAINER_ENV_FILE);
 
 				if (unlink(envfile) && (errno != ENOENT))
@@ -1212,15 +1272,12 @@ extern void cleanup_container(stepd_step_rec_t *step)
 				xfree(envfile);
 			}
 
-			if (rmdir(c->spool_dir) && (errno != ENOENT))
-				error("rmdir(%s): %m", c->spool_dir);
-			xfree(c->spool_dir);
+			if (rmdir(c->task_spool_dir) && (errno != ENOENT))
+				error("rmdir(%s): %m", c->task_spool_dir);
 		}
-	}
 
-	/* swap to non-task spool_dir */
-	xfree(c->spool_dir);
-	c->spool_dir = _generate_spooldir(step, NULL);
+		xfree(c->task_spool_dir);
+	}
 
 	if (rmdir(c->spool_dir) && (errno != ENOENT))
 		error("rmdir(%s): %m", c->spool_dir);
