@@ -70,21 +70,40 @@
 #include "src/common/read_config.h"
 #include "src/common/write_labelled_message.h"
 #include "src/common/xmalloc.h"
-#include "src/common/xsignal.h"
 #include "src/common/xstring.h"
 
+#include "src/interfaces/conn.h"
+
 #include "src/slurmd/common/fname.h"
 #include "src/slurmd/slurmd/slurmd.h"
 #include "src/slurmd/slurmstepd/io.h"
 #include "src/slurmd/slurmstepd/slurmstepd.h"
 
+/*
+ * The message cache uses up free message buffers, so STDIO_MAX_MSG_CACHE
+ * must be a number smaller than STDIO_MAX_FREE_BUF.
+ */
+#define STDIO_MAX_FREE_BUF 1024
+#define STDIO_MAX_MSG_CACHE 128
+
+#define STDIO_FILE_RETRIES 10
+
+struct io_buf {
+	int ref_count;
+	uint32_t length;
+	void *data;
+};
+
+static struct io_buf *_alloc_io_buf(void);
+static void _free_io_buf(struct io_buf *buf);
+
 /**********************************************************************
  * IO client socket declarations
  **********************************************************************/
 static bool _client_readable(eio_obj_t *);
 static bool _client_writable(eio_obj_t *);
-static int  _client_read(eio_obj_t *, List);
-static int  _client_write(eio_obj_t *, List);
+static int _client_read(eio_obj_t *, list_t *);
+static int _client_write(eio_obj_t *, list_t *);
 
 struct io_operations client_ops = {
 	.readable = &_client_readable,
@@ -105,7 +124,7 @@ struct client_io_info {
 	bool in_eof;
 
 	/* outgoing variables */
-	List msg_queue;
+	list_t *msg_queue;
 	struct io_buf *out_msg;
 	int32_t out_remaining;
 	bool out_eof;
@@ -122,7 +141,7 @@ struct client_io_info {
 
 
 static bool _local_file_writable(eio_obj_t *);
-static int  _local_file_write(eio_obj_t *, List);
+static int _local_file_write(eio_obj_t *, list_t *);
 
 struct io_operations local_file_ops = {
 	.writable = &_local_file_writable,
@@ -134,8 +153,8 @@ struct io_operations local_file_ops = {
  * Task write declarations
  **********************************************************************/
 static bool _task_writable(eio_obj_t *);
-static int  _task_write(eio_obj_t *, List);
-static int _task_write_error(eio_obj_t *obj, List objs);
+static int _task_write(eio_obj_t *, list_t *);
+static int _task_write_error(eio_obj_t *obj, list_t *objs);
 
 struct io_operations task_write_ops = {
 	.writable = &_task_writable,
@@ -148,7 +167,7 @@ struct task_write_info {
 	int              magic;
 	stepd_step_rec_t *step; /* pointer back to step data */
 
-	List msg_queue;
+	list_t *msg_queue;
 	struct io_buf *msg;
 	int32_t remaining;
 };
@@ -157,7 +176,7 @@ struct task_write_info {
  * Task read declarations
  **********************************************************************/
 static bool _task_readable(eio_obj_t *);
-static int  _task_read(eio_obj_t *, List);
+static int _task_read(eio_obj_t *, list_t *);
 
 struct io_operations task_read_ops = {
 	.readable = &_task_readable,
@@ -182,7 +201,7 @@ struct task_read_info {
 struct window_info {
 	stepd_step_task_info_t *task;
 	stepd_step_rec_t *step;
-	int pty_fd;
+	void *tls_conn;
 };
 #ifdef HAVE_PTY_H
 static void  _spawn_window_manager(stepd_step_task_info_t *task, stepd_step_rec_t *step);
@@ -193,8 +212,8 @@ static void *_window_manager(void *arg);
  * General declarations
  **********************************************************************/
 static void *_io_thr(void *);
-static int _send_io_init_msg(int sock, srun_info_t *srun, stepd_step_rec_t *step,
-			     bool init);
+static int _send_io_init_msg(int sock, void *tls_conn, srun_info_t *srun,
+			     stepd_step_rec_t *step, bool init);
 static void _send_eof_msg(struct task_read_info *out);
 static struct io_buf *_task_build_message(struct task_read_info *out,
 					  stepd_step_rec_t *step, cbuf_t *cbuf);
@@ -202,7 +221,7 @@ static void *_io_thr(void *arg);
 static void _route_msg_task_to_client(eio_obj_t *obj);
 static void _free_outgoing_msg(struct io_buf *msg, stepd_step_rec_t *step);
 static void _free_incoming_msg(struct io_buf *msg, stepd_step_rec_t *step);
-static void _free_all_outgoing_msgs(List msg_queue, stepd_step_rec_t *step);
+static void _free_all_outgoing_msgs(list_t *msg_queue, stepd_step_rec_t *step);
 static bool _incoming_buf_free(stepd_step_rec_t *step);
 static bool _outgoing_buf_free(stepd_step_rec_t *step);
 static int  _send_connection_okay_response(stepd_step_rec_t *step);
@@ -260,7 +279,7 @@ _client_writable(eio_obj_t *obj)
 
 	/* If this is a newly attached client its msg_queue needs
 	 * to be initialized from the outgoing_cache, and then "obj" needs
-	 * to be added to the List of clients.
+	 * to be added to the list of clients.
 	 */
 	if (client->msg_queue == NULL) {
 		list_itr_t *msgs;
@@ -290,8 +309,7 @@ _client_writable(eio_obj_t *obj)
 	return false;
 }
 
-static int
-_client_read(eio_obj_t *obj, List objs)
+static int _client_read(eio_obj_t *obj, list_t *objs)
 {
 	struct client_io_info *client = (struct client_io_info *) obj->arg;
 	void *buf;
@@ -311,7 +329,7 @@ _client_read(eio_obj_t *obj, List objs)
 			debug5("  _client_read free_incoming is empty");
 			return SLURM_SUCCESS;
 		}
-		n = io_hdr_read_fd(obj->fd, &client->header);
+		n = io_hdr_read_fd(obj->fd, obj->tls_conn, &client->header);
 		if (n <= 0) { /* got eof or fatal error */
 			debug5("  got eof or error _client_read header, n=%d", n);
 			client->in_eof = true;
@@ -321,9 +339,9 @@ _client_read(eio_obj_t *obj, List objs)
 			return SLURM_SUCCESS;
 		}
 		debug5("client->header.length = %u", client->header.length);
-		if (client->header.length > MAX_MSG_LEN)
+		if (client->header.length > SLURM_IO_MAX_MSG_LEN)
 			error("Message length of %u exceeds maximum of %u",
-			      client->header.length, MAX_MSG_LEN);
+			      client->header.length, SLURM_IO_MAX_MSG_LEN);
 		client->in_remaining = client->header.length;
 		client->in_msg->length = client->header.length;
 	}
@@ -358,7 +376,13 @@ _client_read(eio_obj_t *obj, List objs)
 		buf = client->in_msg->data +
 			(client->in_msg->length - client->in_remaining);
 	again:
-		if ((n = read(obj->fd, buf, client->in_remaining)) < 0) {
+		if (obj->tls_conn) {
+			n = conn_g_recv(obj->tls_conn, buf,
+					client->in_remaining);
+		} else {
+			n = read(obj->fd, buf, client->in_remaining);
+		}
+		if (n < 0) {
 			if (errno == EINTR)
 				goto again;
 			if (errno == EAGAIN || errno == EWOULDBLOCK) {
@@ -427,8 +451,7 @@ _client_read(eio_obj_t *obj, List objs)
 /*
  * Write outgoing packed messages to the client socket.
  */
-static int
-_client_write(eio_obj_t *obj, List objs)
+static int _client_write(eio_obj_t *obj, list_t *objs)
 {
 	struct client_io_info *client = (struct client_io_info *) obj->arg;
 	void *buf;
@@ -461,7 +484,12 @@ _client_write(eio_obj_t *obj, List objs)
 	buf = client->out_msg->data +
 		(client->out_msg->length - client->out_remaining);
 again:
-	if ((n = write(obj->fd, buf, client->out_remaining)) < 0) {
+	if (obj->tls_conn) {
+		n = conn_g_send(obj->tls_conn, buf, client->out_remaining);
+	} else {
+		n = write(obj->fd, buf, client->out_remaining);
+	}
+	if (n < 0) {
 		if (errno == EINTR) {
 			goto again;
 		} else if ((errno == EAGAIN) || (errno == EWOULDBLOCK)) {
@@ -510,8 +538,7 @@ _local_file_writable(eio_obj_t *obj)
 /*
  * The slurmstepd writes I/O to a file, possibly adding a label.
  */
-static int
-_local_file_write(eio_obj_t *obj, List objs)
+static int _local_file_write(eio_obj_t *obj, list_t *objs)
 {
 	struct client_io_info *client = (struct client_io_info *) obj->arg;
 	void *buf;
@@ -529,8 +556,8 @@ _local_file_write(eio_obj_t *obj, List objs)
 		if (client->out_msg == NULL) {
 			return SLURM_SUCCESS;
 		}
-		client->out_remaining = client->out_msg->length -
-					io_hdr_packed_size();
+		client->out_remaining =
+			(client->out_msg->length - IO_HDR_PACKET_BYTES);
 	}
 
 	/*
@@ -625,8 +652,7 @@ _task_writable(eio_obj_t *obj)
 	return false;
 }
 
-static int
-_task_write_error(eio_obj_t *obj, List objs)
+static int _task_write_error(eio_obj_t *obj, list_t *objs)
 {
 	debug4("Called _task_write_error, closing fd %d", obj->fd);
 
@@ -636,8 +662,7 @@ _task_write_error(eio_obj_t *obj, List objs)
 	return SLURM_SUCCESS;
 }
 
-static int
-_task_write(eio_obj_t *obj, List objs)
+static int _task_write(eio_obj_t *obj, list_t *objs)
 {
 	struct task_write_info *in = (struct task_write_info *) obj->arg;
 	void *buf;
@@ -712,7 +737,8 @@ _create_task_out_eio(int fd, uint16_t type,
 	out->gtaskid = task->gtid;
 	out->ltaskid = task->id;
 	out->step = step;
-	out->buf = cbuf_create(MAX_MSG_LEN, MAX_MSG_LEN*4);
+	out->buf = cbuf_create(SLURM_IO_MAX_MSG_LEN,
+			       (SLURM_IO_MAX_MSG_LEN * 4));
 	out->eof = false;
 	out->eof_msg_sent = false;
 	if (cbuf_opt_set(out->buf, CBUF_OPT_OVERWRITE, CBUF_NO_DROP) == -1)
@@ -749,8 +775,7 @@ _task_readable(eio_obj_t *obj)
  * allows whole lines to be packed into messages if line buffering
  * is requested.
  */
-static int
-_task_read(eio_obj_t *obj, List objs)
+static int _task_read(eio_obj_t *obj, list_t *objs)
 {
 	struct task_read_info *out = (struct task_read_info *)obj->arg;
 	int len;
@@ -809,7 +834,7 @@ static void *_window_manager(void *arg)
 	struct pollfd ufds;
 	char buf[4];
 
-	ufds.fd = win_info->pty_fd;
+	ufds.fd = conn_g_get_fd(win_info->tls_conn);
 	ufds.events = POLLIN;
 
 	while (1) {
@@ -824,7 +849,7 @@ static void *_window_manager(void *arg)
 			 *  (ufds.revents & POLLERR)) */
 			break;
 		}
-		len = slurm_read_stream(win_info->pty_fd, buf, 4);
+		len = slurm_read_stream(win_info->tls_conn, buf, 4);
 		if ((len == -1) && ((errno == EINTR) || (errno == EAGAIN)))
 			continue;
 		if (len < 4) {
@@ -854,8 +879,9 @@ static void *_window_manager(void *arg)
 static void
 _spawn_window_manager(stepd_step_task_info_t *task, stepd_step_rec_t *step)
 {
+	void *tls_conn = NULL;
+	char *tls_cert = NULL;
 	char *host, *port, *rows, *cols;
-	int pty_fd;
 	slurm_addr_t pty_addr;
 	uint16_t port_u;
 	struct window_info *win_info;
@@ -891,9 +917,15 @@ _spawn_window_manager(stepd_step_task_info_t *task, stepd_step_rec_t *step)
 
 	port_u = atoi(port);
 	slurm_set_addr(&pty_addr, port_u, host);
-	pty_fd = slurm_open_msg_conn(&pty_addr);
-	if (pty_fd < 0) {
-		error("slurm_open_msg_conn(pty_conn) %s,%u: %m",
+
+	if (tls_enabled()) {
+		srun_info_t *srun = list_peek(step->sruns);
+		if (srun)
+			tls_cert = srun->tls_cert;
+	}
+
+	if (!(tls_conn = slurm_open_msg_conn(&pty_addr, tls_cert))) {
+		error("slurm_open_stream(pty_conn) %s,%u: %m",
 			host, port_u);
 		return;
 	}
@@ -901,13 +933,13 @@ _spawn_window_manager(stepd_step_task_info_t *task, stepd_step_rec_t *step)
 	win_info = xmalloc(sizeof(struct window_info));
 	win_info->task   = task;
 	win_info->step    = step;
-	win_info->pty_fd = pty_fd;
+	win_info->tls_conn = tls_conn;
 	slurm_thread_create_detached(_window_manager, win_info);
 }
 #endif
 
 /**********************************************************************
- * General fuctions
+ * General functions
  **********************************************************************/
 
 /*
@@ -980,10 +1012,27 @@ _init_task_stdio_fds(stepd_step_task_info_t *task, stepd_step_rec_t *step)
 		/* open file on task's stdin */
 		debug5("  stdin file name = %s", task->ifname);
 		do {
-			task->stdin_fd = open(task->ifname, (O_RDONLY |
-							     O_CLOEXEC));
-			++count;
-		} while (task->stdin_fd == -1 && errno == EINTR && count < 10);
+			if ((task->stdin_fd = open(task->ifname,
+						   (O_RDONLY | O_CLOEXEC)))
+									!= -1)
+				break;
+
+			/* "Retry-able" errors. */
+			if (errno == EINTR) {
+				debug("%s: Could not open stdin file '%s': '%s'. Attempt [%d/%d], retrying.",
+				      __func__,
+				      task->ifname,
+				      strerror(errno),
+				      (count + 1),
+				      STDIO_FILE_RETRIES);
+				count++;
+				continue;
+			}
+
+			/* Non-"retryable" errors. */
+			break;
+
+		} while (count < STDIO_FILE_RETRIES);
 		if (task->stdin_fd == -1) {
 			error("Could not open stdin file %s: %m", task->ifname);
 			return SLURM_ERROR;
@@ -1034,18 +1083,51 @@ _init_task_stdio_fds(stepd_step_task_info_t *task, stepd_step_rec_t *step)
 	    (((step->flags & LAUNCH_LABEL_IO) == 0) ||
 	     xstrcmp(task->ofname, "/dev/null") == 0)) {
 #endif
-		int count = 0;
+		int count = 0, mkdir_rc;
+		bool tried_mkdir = false;
 		/* open file on task's stdout */
 		debug5("  stdout file name = %s", task->ofname);
 		do {
-			task->stdout_fd = open(task->ofname,
-					       file_flags | O_CLOEXEC, 0666);
-			if (!count && (errno == ENOENT)) {
-				mkdirpath(task->ofname, 0755, false);
-				errno = EINTR;
+			if ((task->stdout_fd = open(task->ofname,
+						    file_flags | O_CLOEXEC,
+						    0666)) != -1)
+				break;
+
+			/* "Retry-able" errors. */
+			if (errno == EINTR) {
+				debug("%s: Could not open stdout file '%s': '%s'. Attempt [%d/%d], retrying.",
+				      __func__,
+				      task->ofname,
+				      strerror(count + 1),
+				      (count + 1),
+				      STDIO_FILE_RETRIES);
+				count++;
+				continue;
 			}
-			++count;
-		} while (task->stdout_fd == -1 && errno == EINTR && count < 10);
+
+			if (errno == ENOENT && !tried_mkdir) {
+				mkdir_rc = mkdirpath(task->ofname, 0755, false);
+				tried_mkdir = true;
+				if (mkdir_rc == SLURM_SUCCESS) {
+					debug("%s: Could not open stdout file '%s': '%s'. Retrying after successful path creation.",
+					      __func__,
+					      task->ofname,
+					      strerror(ENOENT));
+					continue;
+				} else {
+					error("%s: Could not open stdout file '%s': '%s'. Recursive path creation failed: '%s'.",
+					      __func__,
+					      task->ofname,
+					      strerror(ENOENT),
+					      strerror(mkdir_rc));
+					return SLURM_ERROR;
+				}
+			}
+
+			/* Non-"retryable" errors. */
+			break;
+
+		} while (count < STDIO_FILE_RETRIES);
 		if (task->stdout_fd == -1) {
 			error("Could not open stdout file %s: %m",
 			      task->ofname);
@@ -1133,18 +1215,51 @@ _init_task_stdio_fds(stepd_step_task_info_t *task, stepd_step_rec_t *step)
 	    (((step->flags & LAUNCH_LABEL_IO) == 0) ||
 	     (xstrcmp(task->efname, "/dev/null") == 0))) {
 #endif
-		int count = 0;
+		int count = 0, mkdir_rc;
+		bool tried_mkdir = false;
 		/* open file on task's stdout */
 		debug5("  stderr file name = %s", task->efname);
 		do {
-			task->stderr_fd = open(task->efname,
-					       file_flags | O_CLOEXEC, 0666);
-			if (!count && (errno == ENOENT)) {
-				mkdirpath(task->efname, 0755, false);
-				errno = EINTR;
+			if ((task->stderr_fd = open(task->efname,
+						    file_flags | O_CLOEXEC,
+						    0666)) != -1)
+				break;
+
+			/* "Retry-able" errors. */
+			if (errno == EINTR) {
+				debug("%s: Could not open stderr file '%s': '%s'. Attempt [%d/%d], retrying.",
+				      __func__,
+				      task->efname,
+				      strerror(errno),
+				      (count + 1),
+				      STDIO_FILE_RETRIES);
+				count++;
+				continue;
 			}
-			++count;
-		} while (task->stderr_fd == -1 && errno == EINTR && count < 10);
+
+			if (errno == ENOENT && !tried_mkdir) {
+				mkdir_rc = mkdirpath(task->efname, 0755, false);
+				tried_mkdir = true;
+				if (mkdir_rc == SLURM_SUCCESS) {
+					debug("%s: Could not open stderr file '%s': '%s'. Retrying after successful path creation.",
+					      __func__,
+					      task->efname,
+					      strerror(ENOENT));
+					continue;
+				} else {
+					error("%s: Could not open stderr file '%s': '%s'. Recursive path creation failed: '%s'.",
+					      __func__,
+					      task->efname,
+					      strerror(ENOENT),
+					      strerror(mkdir_rc));
+					return SLURM_ERROR;
+				}
+			}
+
+			/* Non-"retryable" errors. */
+			break;
+
+		} while (count < STDIO_FILE_RETRIES);
 		if (task->stderr_fd == -1) {
 			error("Could not open stderr file %s: %m",
 			      task->efname);
@@ -1195,8 +1310,7 @@ extern void io_thread_start(stepd_step_rec_t *step)
 	slurm_mutex_unlock(&step->io_mutex);
 }
 
-void
-_shrink_msg_cache(List cache, stepd_step_rec_t *step)
+static void _shrink_msg_cache(list_t *cache, stepd_step_rec_t *step)
 {
 	struct io_buf *msg;
 	int over = 0;
@@ -1267,13 +1381,13 @@ _build_connection_okay_message(stepd_step_rec_t *step)
 	header.gtaskid = 0;  /* Unused */
 	header.length = 0;
 
-	packbuf = create_buf(msg->data, io_hdr_packed_size());
+	packbuf = create_buf(msg->data, IO_HDR_PACKET_BYTES);
 	if (!packbuf) {
 		fatal("Failure to allocate memory for a message header");
 		return msg;	/* Fix for CLANG false positive error */
 	}
 	io_hdr_pack(&header, packbuf);
-	msg->length = io_hdr_packed_size();
+	msg->length = IO_HDR_PACKET_BYTES;
 	msg->ref_count = 0; /* make certain it is initialized */
 
 	/* free packbuf, but not the memory to which it points */
@@ -1340,7 +1454,7 @@ _free_incoming_msg(struct io_buf *msg, stepd_step_rec_t *step)
 {
 	msg->ref_count--;
 	if (msg->ref_count == 0) {
-		/* Put the message back on the free List */
+		/* Put the message back on the free list */
 		list_enqueue(step->free_incoming, msg);
 
 		/* Kick the event IO engine */
@@ -1355,7 +1469,7 @@ _free_outgoing_msg(struct io_buf *msg, stepd_step_rec_t *step)
 
 	msg->ref_count--;
 	if (msg->ref_count == 0) {
-		/* Put the message back on the free List */
+		/* Put the message back on the free list */
 		list_enqueue(step->free_outgoing, msg);
 
 		/* Try packing messages from tasks' output cbufs */
@@ -1378,8 +1492,7 @@ _free_outgoing_msg(struct io_buf *msg, stepd_step_rec_t *step)
 	}
 }
 
-static void
-_free_all_outgoing_msgs(List msg_queue, stepd_step_rec_t *step)
+static void _free_all_outgoing_msgs(list_t *msg_queue, stepd_step_rec_t *step)
 {
 	list_itr_t *msgs;
 	struct io_buf *msg;
@@ -1469,18 +1582,8 @@ static void *
 _io_thr(void *arg)
 {
 	stepd_step_rec_t *step = (stepd_step_rec_t *) arg;
-	sigset_t set;
 	int rc;
 
-	/* A SIGHUP signal signals a reattach to the mgr thread.  We need
-	 * to block SIGHUP from being delivered to this thread so the mgr
-	 * thread will see the signal.
-	 */
-	sigemptyset(&set);
-	sigaddset(&set, SIGHUP);
-	sigaddset(&set, SIGPIPE);
-	pthread_sigmask(SIG_BLOCK, &set, NULL);
-
 	debug("IO handler started pid=%lu", (unsigned long) getpid());
 	rc = eio_handle_mainloop(step->eio);
 	debug("IO handler exited, rc=%d", rc);
@@ -1554,6 +1657,7 @@ io_initial_client_connect(srun_info_t *srun, stepd_step_rec_t *step,
 	int sock = -1;
 	struct client_io_info *client;
 	eio_obj_t *obj;
+	void *tls_conn = NULL;
 
 	debug4 ("adding IO connection (logical node rank %d)", step->nodeid);
 
@@ -1565,7 +1669,7 @@ io_initial_client_connect(srun_info_t *srun, stepd_step_rec_t *step,
 		debug4("connecting IO back to %pA", &srun->ioaddr);
 	}
 
-	if ((sock = (int) slurm_open_stream(&srun->ioaddr, true)) < 0) {
+	if (!(tls_conn = slurm_open_msg_conn(&srun->ioaddr, srun->tls_cert))) {
 		error("connect io: %m");
 		/* XXX retry or silently fail?
 		 *     fail for now.
@@ -1573,8 +1677,10 @@ io_initial_client_connect(srun_info_t *srun, stepd_step_rec_t *step,
 		return SLURM_ERROR;
 	}
 
+	sock = conn_g_get_fd(tls_conn);
+
 	fd_set_blocking(sock);  /* just in case... */
-	_send_io_init_msg(sock, srun, step, true);
+	_send_io_init_msg(sock, tls_conn, srun, step, true);
 
 	debug5("  back from _send_io_init_msg");
 	fd_set_nonblocking(sock);
@@ -1592,6 +1698,7 @@ io_initial_client_connect(srun_info_t *srun, stepd_step_rec_t *step,
 	client->is_local_file = false;
 
 	obj = eio_obj_create(sock, &client_ops, (void *)client);
+	obj->tls_conn = tls_conn;
 	list_append(step->clients, (void *)obj);
 	eio_new_initial_obj(step->eio, (void *)obj);
 	debug5("Now handling %d IO Client object(s)",
@@ -1612,6 +1719,7 @@ io_client_connect(srun_info_t *srun, stepd_step_rec_t *step)
 	int sock = -1;
 	struct client_io_info *client;
 	eio_obj_t *obj;
+	void *tls_conn = NULL;
 
 	debug4 ("adding IO connection (logical node rank %d)", step->nodeid);
 
@@ -1627,8 +1735,22 @@ io_client_connect(srun_info_t *srun, stepd_step_rec_t *step)
 		return SLURM_ERROR;
 	}
 
+	if (tls_enabled()) {
+		conn_args_t tls_args = {
+			.input_fd = sock,
+			.output_fd = sock,
+			.mode = TLS_CONN_CLIENT,
+			.cert = srun->tls_cert,
+		};
+
+		if (!(tls_conn = conn_g_create(&tls_args))) {
+			error("Could not create client TLS connection for step IO");
+			return SLURM_ERROR;
+		}
+	}
+
 	fd_set_blocking(sock);  /* just in case... */
-	_send_io_init_msg(sock, srun, step, false);
+	_send_io_init_msg(sock, tls_conn, srun, step, false);
 
 	debug5("  back from _send_io_init_msg");
 	fd_set_nonblocking(sock);
@@ -1648,6 +1770,7 @@ io_client_connect(srun_info_t *srun, stepd_step_rec_t *step)
 	/* client object adds itself to step->clients in _client_writable */
 
 	obj = eio_obj_create(sock, &client_ops, (void *)client);
+	obj->tls_conn = tls_conn;
 	eio_new_obj(step->eio, (void *)obj);
 
 	debug5("New IO Client object added");
@@ -1655,8 +1778,8 @@ io_client_connect(srun_info_t *srun, stepd_step_rec_t *step)
 	return SLURM_SUCCESS;
 }
 
-static int
-_send_io_init_msg(int sock, srun_info_t *srun, stepd_step_rec_t *step, bool init)
+static int _send_io_init_msg(int sock, void *tls_conn, srun_info_t *srun,
+			     stepd_step_rec_t *step, bool init)
 {
 	io_init_msg_t msg;
 
@@ -1680,7 +1803,7 @@ _send_io_init_msg(int sock, srun_info_t *srun, stepd_step_rec_t *step, bool init
 	else
 		msg.stderr_objs = list_count(step->stderr_eio_objs);
 
-	if (io_init_msg_write_to_fd(sock, &msg) != SLURM_SUCCESS) {
+	if (io_init_msg_write_to_fd(sock, tls_conn, &msg) != SLURM_SUCCESS) {
 		error("Couldn't sent slurm_io_init_msg");
 		xfree(msg.io_key);
 		return SLURM_ERROR;
@@ -1743,7 +1866,7 @@ _send_eof_msg(struct task_read_info *out)
 		   a poll returns POLLHUP on the incoming task pipe,
 		   put there are no outgoing message buffers available,
 		   the slurmstepd will start spinning. */
-		msg = alloc_io_buf();
+		msg = _alloc_io_buf();
 	}
 
 	header.type = out->type;
@@ -1751,14 +1874,14 @@ _send_eof_msg(struct task_read_info *out)
 	header.gtaskid = out->gtaskid;
 	header.length = 0; /* eof */
 
-	packbuf = create_buf(msg->data, io_hdr_packed_size());
+	packbuf = create_buf(msg->data, IO_HDR_PACKET_BYTES);
 	if (!packbuf) {
 		fatal("Failure to allocate memory for a message header");
 		return;	/* Fix for CLANG false positive error */
 	}
 
 	io_hdr_pack(&header, packbuf);
-	msg->length = io_hdr_packed_size() + header.length;
+	msg->length = IO_HDR_PACKET_BYTES + header.length;
 	msg->ref_count = 0; /* make certain it is initialized */
 
 	/* free packbuf, but not the memory to which it points */
@@ -1778,7 +1901,7 @@ _send_eof_msg(struct task_read_info *out)
 	}
 	list_iterator_destroy(clients);
 	if (msg->ref_count == 0)
-		free_io_buf(msg);
+		_free_io_buf(msg);
 
 	debug4("Leaving  _send_eof_msg");
 }
@@ -1805,13 +1928,13 @@ static struct io_buf *_task_build_message(struct task_read_info *out,
 		return NULL;
 	}
 
-	ptr = msg->data + io_hdr_packed_size();
+	ptr = msg->data + IO_HDR_PACKET_BYTES;
 
 	if (buffered_stdio) {
-		avail = cbuf_peek_line(cbuf, ptr, MAX_MSG_LEN, 1);
-		if (avail >= MAX_MSG_LEN)
+		avail = cbuf_peek_line(cbuf, ptr, SLURM_IO_MAX_MSG_LEN, 1);
+		if (avail >= SLURM_IO_MAX_MSG_LEN)
 			must_truncate = true;
-		else if (avail == 0 && cbuf_used(cbuf) >= MAX_MSG_LEN)
+		else if (avail == 0 && cbuf_used(cbuf) >= SLURM_IO_MAX_MSG_LEN)
 			must_truncate = true;
 	}
 
@@ -1826,9 +1949,9 @@ static struct io_buf *_task_build_message(struct task_read_info *out,
 	 * Hence the "|| out->eof".
 	 */
 	if (must_truncate || !buffered_stdio || out->eof) {
-		n = cbuf_read(cbuf, ptr, MAX_MSG_LEN);
+		n = cbuf_read(cbuf, ptr, SLURM_IO_MAX_MSG_LEN);
 	} else {
-		n = cbuf_read_line(cbuf, ptr, MAX_MSG_LEN, -1);
+		n = cbuf_read_line(cbuf, ptr, SLURM_IO_MAX_MSG_LEN, -1);
 		if (n == 0) {
 			debug5("  partial line in buffer, ignoring");
 			debug4("Leaving  _task_build_message");
@@ -1843,13 +1966,13 @@ static struct io_buf *_task_build_message(struct task_read_info *out,
 	header.length = n;
 
 	debug4("%s: header.length = %d", __func__, n);
-	packbuf = create_buf(msg->data, io_hdr_packed_size());
+	packbuf = create_buf(msg->data, IO_HDR_PACKET_BYTES);
 	if (!packbuf) {
 		fatal("Failure to allocate memory for a message header");
 		return msg;	/* Fix for CLANG false positive error */
 	}
 	io_hdr_pack(&header, packbuf);
-	msg->length = io_hdr_packed_size() + header.length;
+	msg->length = IO_HDR_PACKET_BYTES + header.length;
 	msg->ref_count = 0; /* make certain it is initialized */
 
 	/* free packbuf, but not the memory to which it points */
@@ -1860,8 +1983,7 @@ static struct io_buf *_task_build_message(struct task_read_info *out,
 	return msg;
 }
 
-struct io_buf *
-alloc_io_buf(void)
+static struct io_buf *_alloc_io_buf(void)
 {
 	struct io_buf *buf = xmalloc(sizeof(*buf));
 
@@ -1869,13 +1991,12 @@ alloc_io_buf(void)
 	buf->length = 0;
 	/* The following "+ 1" is just temporary so I can stick a \0 at
 	   the end and do a printf of the data pointer */
-	buf->data = xmalloc(MAX_MSG_LEN + io_hdr_packed_size() + 1);
+	buf->data = xmalloc(SLURM_IO_MAX_MSG_LEN + IO_HDR_PACKET_BYTES + 1);
 
 	return buf;
 }
 
-void
-free_io_buf(struct io_buf *buf)
+static void _free_io_buf(struct io_buf *buf)
 {
 	if (buf) {
 		if (buf->data)
@@ -1893,7 +2014,7 @@ _incoming_buf_free(stepd_step_rec_t *step)
 	if (list_count(step->free_incoming) > 0) {
 		return true;
 	} else if (step->incoming_count < STDIO_MAX_FREE_BUF) {
-		buf = alloc_io_buf();
+		buf = _alloc_io_buf();
 		list_enqueue(step->free_incoming, buf);
 		step->incoming_count++;
 		return true;
@@ -1910,7 +2031,7 @@ _outgoing_buf_free(stepd_step_rec_t *step)
 	if (list_count(step->free_outgoing) > 0) {
 		return true;
 	} else if (step->outgoing_count < STDIO_MAX_FREE_BUF) {
-		buf = alloc_io_buf();
+		buf = _alloc_io_buf();
 		list_enqueue(step->free_outgoing, buf);
 		step->outgoing_count++;
 		return true;
@@ -2044,13 +2165,10 @@ io_get_file_flags(stepd_step_rec_t *step)
 		file_flags = O_CREAT|O_WRONLY|O_APPEND;
 	else if (step->open_mode == OPEN_MODE_TRUNCATE)
 		file_flags = O_CREAT|O_WRONLY|O_APPEND|O_TRUNC;
-	else {
-		slurm_conf_t *conf = slurm_conf_lock();
-		if (conf->job_file_append)
-			file_flags = O_CREAT|O_WRONLY|O_APPEND;
-		else
-			file_flags = O_CREAT|O_WRONLY|O_APPEND|O_TRUNC;
-		slurm_conf_unlock();
-	}
+	else if (slurm_conf.job_file_append)
+		file_flags = O_CREAT|O_WRONLY|O_APPEND;
+	else
+		file_flags = O_CREAT|O_WRONLY|O_APPEND|O_TRUNC;
+
 	return file_flags;
 }
