@@ -41,7 +41,6 @@
 #include <grp.h>
 #include <limits.h>
 #include <netdb.h>
-#include <sched.h>
 #include <signal.h>
 #include <sys/socket.h>
 #include <sys/stat.h>
@@ -61,6 +60,7 @@
 #include "src/common/proc_args.h"
 #include "src/common/read_config.h"
 #include "src/common/ref.h"
+#include "src/common/run_in_daemon.h"
 #include "src/common/slurm_protocol_defs.h"
 #include "src/common/uid.h"
 #include "src/common/xmalloc.h"
@@ -94,6 +94,8 @@
 
 decl_static_data(usage_txt);
 
+uint32_t slurm_daemon = IS_SLURMRESTD;
+
 typedef struct {
 	bool stdin_tty; /* running with a TTY for stdin */
 	bool stdin_socket; /* running with a socket for stdin */
@@ -112,7 +114,7 @@ static run_mode_t run_mode = { 0 };
 static list_t *socket_listen = NULL;
 static char *slurm_conf_filename = NULL;
 /* Number of requested threads */
-static int thread_count = 20;
+static int thread_count = 0;
 /* Max number of connections */
 static int max_connections = 124;
 /* User to become once loaded */
@@ -138,6 +140,12 @@ static http_status_code_t *response_status_codes = NULL;
 extern parsed_host_port_t *parse_host_port(const char *str);
 extern void free_parse_host_port(parsed_host_port_t *parsed);
 
+static void _plugrack_foreach_list(const char *full_type, const char *fq_path,
+				   const plugin_handle_t id, void *arg)
+{
+	fprintf(stdout, "%s\n", full_type);
+}
+
 /* SIGPIPE handler - mostly a no-op */
 static void _sigpipe_handler(conmgr_callback_args_t conmgr_args, void *arg)
 {
@@ -208,6 +216,9 @@ static void _parse_env(void)
 						"disable_unshare_files")) {
 				unshare_files = false;
 			} else if (!xstrcasecmp(token, "disable_user_check")) {
+#ifdef NDEBUG
+				fatal_abort("SLURMRESTD_SECURITY=disable_user_check should only be used for development. Disabling the user check to run slurmrestd as root or SlurmUser will allow anyone to run any command on the cluster as root.");
+#endif /* NDEBUG */
 				check_user = false;
 			} else if (!xstrcasecmp(token, "become_user")) {
 				become_user = true;
@@ -220,44 +231,6 @@ static void _parse_env(void)
 		xfree(toklist);
 	}
 
-	if ((buffer = getenv("SLURMRESTD_JSON"))) {
-		char *token = NULL, *save_ptr = NULL;
-		char *toklist = xstrdup(buffer);
-
-		token = strtok_r(toklist, ",", &save_ptr);
-		while (token) {
-			if (!xstrcasecmp(token, "compact")) {
-				json_flags = SER_FLAGS_COMPACT;
-			} else if (!xstrcasecmp(token, "pretty")) {
-				json_flags = SER_FLAGS_PRETTY;
-			} else {
-				fatal("Unexpected value in SLURMRESTD_JSON=%s",
-				      token);
-			}
-			token = strtok_r(NULL, ",", &save_ptr);
-		}
-		xfree(toklist);
-	}
-
-	if ((buffer = getenv("SLURMRESTD_YAML"))) {
-		char *token = NULL, *save_ptr = NULL;
-		char *toklist = xstrdup(buffer);
-
-		token = strtok_r(toklist, ",", &save_ptr);
-		while (token) {
-			if (!xstrcasecmp(token, "compact")) {
-				yaml_flags = SER_FLAGS_COMPACT;
-			} else if (!xstrcasecmp(token, "pretty")) {
-				yaml_flags = SER_FLAGS_PRETTY;
-			} else {
-				fatal("Unexpected value in SLURMRESTD_YAML=%s",
-				      token);
-			}
-			token = strtok_r(NULL, ",", &save_ptr);
-		}
-		xfree(toklist);
-	}
-
 	if ((buffer = getenv("SLURMRESTD_RESPONSE_STATUS_CODES"))) {
 		char *token = NULL, *save_ptr = NULL;
 		char *toklist = xstrdup(buffer);
@@ -374,6 +347,7 @@ static void _usage(void)
 __attribute__((noreturn))
 static void dump_spec(int argc, char **argv)
 {
+	const char *dump_mime_types[] = { MIME_TYPE_JSON, NULL };
 	int rc = SLURM_SUCCESS;
 	data_t *spec = data_new();
 	char *output = NULL;
@@ -394,8 +368,7 @@ static void dump_spec(int argc, char **argv)
 		      slurm_conf_filename, slurm_strerror(rc));
 	}
 
-	if (serializer_g_init(MIME_TYPE_JSON_PLUGIN, NULL))
-		fatal("Unable to initialize JSON serializer");
+	serializer_required(MIME_TYPE_JSON);
 
 	if (!(parsers = data_parser_g_new_array(NULL, NULL, NULL, NULL, NULL,
 						NULL, NULL, NULL,
@@ -410,7 +383,7 @@ static void dump_spec(int argc, char **argv)
 	if (init_openapi(oas_specs, NULL, parsers, response_status_codes))
 		fatal("Unable to initialize OpenAPI structures");
 
-	if ((rc = generate_spec(spec)))
+	if ((rc = generate_spec(spec, dump_mime_types)))
 		fatal("Unable to generate OpenAPI Specification: %s",
 		      slurm_strerror(rc));
 
@@ -462,6 +435,15 @@ static void _parse_commandline(int argc, char **argv)
 			rest_auth = xstrdup(optarg);
 			break;
 		case 'd':
+			if (!xstrcasecmp(optarg, "list")) {
+				fprintf(stderr, "Possible data_parser plugins:\n");
+				parsers = data_parser_g_new_array(
+					NULL, NULL, NULL, NULL, NULL, NULL,
+					NULL, NULL, optarg,
+					_plugrack_foreach_list, false);
+				exit(SLURM_SUCCESS);
+			}
+
 			xfree(data_parser_plugins);
 			data_parser_plugins = xstrdup(optarg);
 			break;
@@ -517,6 +499,59 @@ static void _parse_commandline(int argc, char **argv)
 	}
 }
 
+/*
+ * Check for supplementary group that could result in an unintended privilege
+ * escalation
+ */
+static void _check_gids(void)
+{
+	gid_t *gids = NULL;
+	bool need_drop = false;
+	int gid_count = getgroups(0, NULL);
+
+	if (gid_count < 0)
+		fatal("%s: getgroups(0, NULL) failed: %m", __func__);
+
+	if (!gid_count)
+		return;
+
+	gids = xcalloc(gid_count, sizeof(*gids));
+
+	if ((gid_count = getgroups(gid_count, gids)) < 0)
+		fatal("%s: getgroups() failed: %m", __func__);
+
+	for (int i = 0; i < gid_count; i++) {
+		/*
+		 * Ignore same gid being in supplementary groups
+		 * as it won't change permissions
+		 */
+		if (gids[i] == gid)
+			continue;
+
+		need_drop = true;
+		debug("%s: Supplementary group %d needs to be dropped",
+		      __func__, gids[i]);
+	}
+
+	xfree(gids);
+
+	if (!need_drop)
+		return;
+
+	debug("%s: Dropping all supplementary groups", __func__);
+
+	if (!setgroups(0, NULL))
+		return;
+
+#ifdef __linux__
+	if (errno == EPERM)
+		fatal("slurmrestd process lacks CAP_SETGID to drop supplementary groups. Supplementary groups must be removed from slurmrestd user (uid=%d,gid=%d) prior to starting slurmrestd.",
+		      uid, gid);
+#endif /* __linux__ */
+
+	fatal("Unable to drop supplementary groups: %m");
+}
+
 /*
  * slurmrestd is merely a translator from REST to Slurm.
  * Try to lock down any extra unneeded permissions.
@@ -536,10 +571,10 @@ static void _lock_down(void)
 	if (unshare_files && unshare(CLONE_FILES))
 		fatal("Unable to unshare file descriptors: %m");
 
-	if (gid && setgroups(0, NULL))
-		fatal("Unable to drop supplementary groups: %m");
 	if (uid != 0 && (gid == 0))
 		gid = gid_from_uid(uid);
+	if (gid)
+		_check_gids();
 	if (gid != 0 && setgid(gid))
 		fatal("Unable to setgid: %m");
 	if (uid != 0 && setuid(uid))
@@ -579,9 +614,9 @@ static void _check_user(void)
 	if (gid_from_uid(slurm_conf.slurm_user_id) == getgid())
 		fatal("slurmrestd should not be run with SlurmUser's group.");
 
-	if (!getuid() && !become_user)
+	if (!getuid())
 		fatal("slurmrestd should not be run as the root user.");
-	if (!getgid() && !become_user)
+	if (!getgid())
 		fatal("slurmrestd should not be run with the root group.");
 
 	if ((gid_count = getgroups(0, NULL)) > 0) {
@@ -594,7 +629,7 @@ static void _check_user(void)
 			if (list[i] == slurm_conf.slurm_user_id)
 				fatal("slurmrestd should not be run with SlurmUser's group.");
 
-			if (!list[i] && !become_user)
+			if (!list[i])
 				fatal("slurmrestd should not be run with the root group.");
 
 			if (list[i] == SLURM_AUTH_NOBODY)
@@ -631,12 +666,6 @@ static void _auth_plugrack_foreach(const char *full_type, const char *fq_path,
 	       __func__, full_type, fq_path);
 }
 
-static void _plugrack_foreach_list(const char *full_type, const char *fq_path,
-				   const plugin_handle_t id, void *arg)
-{
-	fprintf(stdout, "%s\n", full_type);
-}
-
 static void _on_signal_interrupt(conmgr_callback_args_t conmgr_args, void *arg)
 {
 	info("%s: caught SIGINT. Shutting down.", __func__);
@@ -658,6 +687,7 @@ int main(int argc, char **argv)
 		.on_data = parse_http,
 		.on_connection = _setup_http_context,
 		.on_finish = on_http_connection_finish,
+		.on_fingerprint = on_fingerprint_tls,
 	};
 	static const conmgr_events_t inet_events = {
 		.on_data = parse_http,
@@ -688,13 +718,28 @@ int main(int argc, char **argv)
 	slurm_init(slurm_conf_filename);
 	_check_user();
 
-	if (serializer_g_init(NULL, NULL))
-		fatal("Unable to initialize serializers");
+	/* Load serializers if they are present */
+	serializer_required(MIME_TYPE_JSON);
+	if (getenv("SLURMRESTD_YAML"))
+		serializer_required(MIME_TYPE_YAML);
+	serializer_required(MIME_TYPE_URL_ENCODED);
 
 	/* This checks if slurmrestd is running in inetd mode */
 	conmgr_init((run_mode.listen ? thread_count : CONMGR_THREAD_COUNT_MIN),
 		    max_connections, callbacks);
 
+	/*
+	 * Check if TLS is available after conmgr_init() as it will attempt to
+	 * load the TLS plugin. If the TLS plugin fails to load the slurmrestd
+	 * daemon certificate, then unload the TLS plugin to avoid
+	 * tls_available() returning true but TLS not being possible.
+	 */
+	if (tls_available() && (rc = tls_g_load_own_cert(NULL, 0, NULL, 0))) {
+		warning("Disabling TLS support due to failure loading TLS certificate: %s",
+			slurm_strerror(rc));
+		(void) tls_g_fini();
+	}
+
 	conmgr_add_work_signal(SIGINT, _on_signal_interrupt, NULL);
 	conmgr_add_work_signal(SIGPIPE, _sigpipe_handler, NULL);
 
@@ -745,18 +790,10 @@ int main(int argc, char **argv)
 	if (init_rest_auth(become_user, auth_plugin_handles, auth_plugin_count))
 		fatal("Unable to initialize rest authentication");
 
-	if (data_parser_plugins && !xstrcasecmp(data_parser_plugins, "list")) {
-		fprintf(stderr, "Possible data_parser plugins:\n");
-		parsers = data_parser_g_new_array(NULL, NULL, NULL, NULL,
-						  NULL, NULL, NULL, NULL,
-						  data_parser_plugins,
-						  _plugrack_foreach_list,
-						  false);
-		exit(SLURM_SUCCESS);
-	} else if (!(parsers = data_parser_g_new_array(NULL, NULL, NULL, NULL,
-						       NULL, NULL, NULL, NULL,
-						       data_parser_plugins,
-						       NULL, false))) {
+	if (!(parsers = data_parser_g_new_array(NULL, NULL, NULL, NULL, NULL,
+						NULL, NULL, NULL,
+						data_parser_plugins, NULL,
+						false))) {
 		fatal("Unable to initialize data_parser plugins");
 	}
 	xfree(data_parser_plugins);
@@ -793,7 +830,7 @@ int main(int argc, char **argv)
 		if ((rc = conmgr_process_fd(CON_TYPE_RAW, STDIN_FILENO,
 					    STDOUT_FILENO, &inet_events,
 					    CON_FLAG_NONE, NULL, 0,
-					    operations_router)))
+					    NULL, operations_router)))
 			fatal("%s: unable to process stdin: %s",
 			      __func__, slurm_strerror(rc));
 
@@ -802,8 +839,8 @@ int main(int argc, char **argv)
 	} else if (run_mode.listen) {
 		mode_t mask = umask(0);
 
-		if (conmgr_create_listen_sockets(CON_TYPE_RAW, socket_listen,
-						 &conmgr_events,
+		if (conmgr_create_listen_sockets(CON_TYPE_RAW, CON_FLAG_NONE,
+						 socket_listen, &conmgr_events,
 						 operations_router))
 			fatal("Unable to create sockets");
 
@@ -842,10 +879,9 @@ int main(int argc, char **argv)
 
 	xfree(auth_plugin_handles);
 	acct_storage_g_fini();
-	select_g_fini();
 	slurm_fini();
 	hash_g_fini();
-	tls_g_fini();
+	conn_g_fini();
 	cred_g_fini();
 	auth_g_fini();
 	log_fini();
