@@ -60,7 +60,7 @@
 #include "src/common/print_fields.h"
 #include "src/squeue/squeue.h"
 
-static void	_combine_pending_array_tasks(List l);
+static void _combine_pending_array_tasks(list_t *l);
 static bool _filter_job(job_info_t *job);
 static int	_filter_job_part(char *part_name);
 static int	_filter_step(job_step_info_t * step);
@@ -121,13 +121,13 @@ static void _create_priority_list(list_t *l,
 	list_t *part_names = list_create(xfree_ptr);
 
 	/*
-	 * If the job requests multiple partitions then priority_array_parts
+	 * If the job requests multiple partitions then priority_array_names
 	 * will exist. When a job is running the controller only sends the
 	 * partition that the job is running in in job_ptr->partition so we can
 	 * just use job_ptr->partition and job_ptr->priority.
 	 */
-	if (IS_JOB_PENDING(job_ptr) && job_ptr->priority_array_parts)
-		tmp = job_ptr->priority_array_parts;
+	if (IS_JOB_PENDING(job_ptr) && job_ptr->priority_array_names)
+		tmp = job_ptr->priority_array_names;
 	else
 		tmp = job_ptr->partition;
 	slurm_addto_char_list(part_names, tmp);
@@ -142,7 +142,7 @@ extern void print_jobs_array(job_info_t *jobs, int size, list_t *format)
 {
 	squeue_job_rec_t *job_rec_ptr;
 	int i;
-	List l;
+	list_t *l;
 
 	l = list_create(_job_list_del);
 	if (!params.no_header)
@@ -191,7 +191,7 @@ extern void print_steps_array(job_step_info_t *steps, int size, list_t *format)
 
 	if (size > 0) {
 		int i;
-		List step_list;
+		list_t *step_list;
 
 		step_list = list_create(NULL);
 
@@ -254,7 +254,7 @@ static void _merge_job_reason(job_info_t *job_ptr, job_info_t *task_ptr)
 /* Combine pending tasks of a job array into a single record.
  * The tasks may have been split into separate job records because they were
  * modified or started, but the records can be re-combined if pending. */
-static void _combine_pending_array_tasks(List job_list)
+static void _combine_pending_array_tasks(list_t *job_list)
 {
 	squeue_job_rec_t *job_rec_ptr, *task_rec_ptr;
 	list_itr_t *job_iterator, *task_iterator;
@@ -464,7 +464,7 @@ int _print_time(time_t t, int level, int width, bool right)
 /*****************************************************************************
  * Job Print Functions
  *****************************************************************************/
-static int _print_one_job_from_format(job_info_t * job, List list)
+static int _print_one_job_from_format(job_info_t *job, list_t *list)
 {
 	list_itr_t *iter = list_iterator_create(list);
 	job_format_t *current;
@@ -487,7 +487,7 @@ static int _print_job_from_format(void *x, void *arg)
 	int i, i_first, i_last;
 	bitstr_t *bitmap;
 	squeue_job_rec_t *job_rec_ptr = (squeue_job_rec_t *) x;
-	List list = (List) arg;
+	list_t *list = arg;
 
 	if (!job_rec_ptr) {
 		_print_one_job_from_format(NULL, list);
@@ -530,9 +530,8 @@ static int _print_job_from_format(void *x, void *arg)
 	return SLURM_SUCCESS;
 }
 
-int
-job_format_add_function(List list, int width, bool right, char *suffix,
-			int (*function) (job_info_t *, int, bool, char*))
+int job_format_add_function(list_t *list, int width, bool right, char *suffix,
+			    int (*function)(job_info_t *, int, bool, char *))
 {
 	job_format_t *tmp = (job_format_t *) xmalloc(sizeof(job_format_t));
 	tmp->function = function;
@@ -802,6 +801,19 @@ int _print_job_licenses(job_info_t * job, int width, bool right, char* suffix)
 	return SLURM_SUCCESS;
 }
 
+int _print_job_licenses_alloc(job_info_t *job, int width, bool right,
+			      char *suffix)
+{
+	if (job == NULL) /* Print the Header instead */
+		_print_str("LICENSES_ALLOC", width, right, true);
+	else
+		_print_str(job->licenses_allocated, width, right, true);
+
+	if (suffix)
+		printf("%s", suffix);
+	return SLURM_SUCCESS;
+}
+
 int _print_job_wckey(job_info_t * job, int width, bool right, char* suffix)
 {
 	if (job == NULL)	/* Print the Header instead */
@@ -2143,6 +2155,21 @@ int _print_job_restart_cnt(job_info_t * job, int width,
 	return SLURM_SUCCESS;
 }
 
+int _print_job_segment_size(job_info_t *job, int width, bool right_justify,
+			    char *suffix)
+{
+	if (job == NULL)
+		_print_str("SEGMENT_SIZE", width, right_justify, true);
+	else if (job->segment_size)
+		_print_int(job->segment_size, width, right_justify, true);
+	else
+		_print_str("N/A", width, right_justify, true);
+
+	if (suffix)
+		printf("%s", suffix);
+	return SLURM_SUCCESS;
+}
+
 int _print_job_sockets_per_board(job_info_t * job, int width,
 				 bool right_justify, char* suffix)
 {
@@ -2157,28 +2184,14 @@ int _print_job_sockets_per_board(job_info_t * job, int width,
 
 }
 
-static char *_expand_std_patterns(char *path, job_info_t *job)
-{
-	job_std_pattern_t job_stp;
-
-	job_stp.array_task_id = job->array_task_id;
-	job_stp.first_step_name = "batch";
-	job_stp.first_step_node = job->batch_host;
-	job_stp.jobid = job->job_id;
-	job_stp.jobname = job->name;
-	job_stp.user = job->user_name;
-	job_stp.work_dir = job->work_dir;
-
-	return expand_stdio_fields(path, &job_stp);
-}
-
 int _print_job_std_err(job_info_t * job, int width,
 		       bool right_justify, char* suffix)
 {
 	if (job == NULL)
 		_print_str("STDERR", width, right_justify, true);
 	else if (params.expand_patterns) {
-		char *tmp_str = _expand_std_patterns(job->std_err, job);
+		char *tmp_str = slurm_expand_job_stdio_fields(job->std_err,
+							      job);
 		_print_str(tmp_str, width, right_justify, true);
 		xfree(tmp_str);
 	} else
@@ -2195,7 +2208,7 @@ int _print_job_std_in(job_info_t * job, int width,
 	if (job == NULL)
 		_print_str("STDIN", width, right_justify, true);
 	else if (params.expand_patterns) {
-		char *tmp_str = _expand_std_patterns(job->std_in, job);
+		char *tmp_str = slurm_expand_job_stdio_fields(job->std_in, job);
 		_print_str(tmp_str, width, right_justify, true);
 		xfree(tmp_str);
 	} else
@@ -2225,7 +2238,8 @@ int _print_job_std_out(job_info_t * job, int width,
 	if (job == NULL)
 		_print_str("STDOUT", width, right_justify, true);
 	else if (params.expand_patterns) {
-		char *tmp_str = _expand_std_patterns(job->std_out, job);
+		char *tmp_str = slurm_expand_job_stdio_fields(job->std_out,
+							      job);
 		_print_str(tmp_str, width, right_justify, true);
 		xfree(tmp_str);
 	} else
@@ -2456,7 +2470,7 @@ int _print_job_mcs_label(job_info_t * job, int width,
 static int _print_step_from_format(void *x, void *arg)
 {
 	job_step_info_t *job_step = (job_step_info_t *) x;
-	List list = (List) arg;
+	list_t *list = arg;
 	list_itr_t *i = list_iterator_create(list);
 	step_format_t *current;
 
@@ -2473,10 +2487,9 @@ static int _print_step_from_format(void *x, void *arg)
 	return SLURM_SUCCESS;
 }
 
-int
-step_format_add_function(List list, int width, bool right_justify,
-			 char* suffix,
-			 int (*function) (job_step_info_t *, int, bool, char*))
+int step_format_add_function(
+	list_t *list, int width, bool right_justify, char *suffix,
+	int (*function) (job_step_info_t *, int, bool, char *))
 {
 	step_format_t *tmp =
 	    (step_format_t *) xmalloc(sizeof(step_format_t));
@@ -2576,6 +2589,72 @@ int _print_step_prefix(job_step_info_t * step, int width, bool right,
 	return SLURM_SUCCESS;
 }
 
+int _print_step_std_err(job_step_info_t *step, int width, bool right_justify,
+			char *suffix)
+{
+	if (step == NULL) {
+		_print_str("STDERR", width, right_justify, true);
+	} else if (!step->std_err) {
+		_print_str("", width, right_justify, true);
+	} else if (params.expand_patterns) {
+		char *tmp_str = slurm_expand_step_stdio_fields(step->std_err,
+							       step);
+		_print_str(tmp_str, width, right_justify, true);
+		xfree(tmp_str);
+	} else {
+		_print_str(step->std_err, width, right_justify, true);
+	}
+
+	if (suffix)
+		printf("%s", suffix);
+
+	return SLURM_SUCCESS;
+}
+
+int _print_step_std_in(job_step_info_t *step, int width, bool right_justify,
+		       char *suffix)
+{
+	if (step == NULL) {
+		_print_str("STDIN", width, right_justify, true);
+	} else if (!step->std_in) {
+		_print_str("", width, right_justify, true);
+	} else if (params.expand_patterns) {
+		char *tmp_str = slurm_expand_step_stdio_fields(step->std_in,
+							       step);
+		_print_str(tmp_str, width, right_justify, true);
+		xfree(tmp_str);
+	} else {
+		_print_str(step->std_in, width, right_justify, true);
+	}
+
+	if (suffix)
+		printf("%s", suffix);
+
+	return SLURM_SUCCESS;
+}
+
+int _print_step_std_out(job_step_info_t *step, int width, bool right_justify,
+			char *suffix)
+{
+	if (step == NULL) {
+		_print_str("STDOUT", width, right_justify, true);
+	} else if (!step->std_out) {
+		_print_str("", width, right_justify, true);
+	} else if (params.expand_patterns) {
+		char *tmp_str = slurm_expand_step_stdio_fields(step->std_out,
+							       step);
+		_print_str(tmp_str, width, right_justify, true);
+		xfree(tmp_str);
+	} else {
+		_print_str(step->std_out, width, right_justify, true);
+	}
+
+	if (suffix)
+		printf("%s", suffix);
+
+	return SLURM_SUCCESS;
+}
+
 int _print_step_user_id(job_step_info_t * step, int width, bool right,
 			char* suffix)
 {
@@ -3045,7 +3124,6 @@ static bool _filter_job(job_info_t *job)
 		iterator = list_iterator_create(params.state_list);
 		while ((state_id = list_next(iterator))) {
 			bool match = false;
-			job->job_state &= ~JOB_UPDATE_DB;
 			if (*state_id &  JOB_STATE_FLAGS) {
 				if (*state_id &  job->job_state)
 					match = true;
@@ -3274,3 +3352,18 @@ int _print_com_invalid(void * p, int width, bool right, char* suffix)
 		printf("%s", suffix);
 	return SLURM_SUCCESS;
 }
+
+int _print_job_cron_flag(job_info_t *job, int width, bool right_justify,
+			 char *suffix)
+{
+	if (!job)
+		_print_str("CRON_JOB", width, right_justify, true);
+	else if (job->bitflags & CRON_JOB)
+		_print_str("Yes", width, right_justify, true);
+	else
+		_print_str("No", width, right_justify, true);
+
+	if (suffix)
+		printf("%s", suffix);
+	return SLURM_SUCCESS;
+}
