@@ -1,36 +1,36 @@
-/*  launch.c - Define job launch plugin functions.
-*****************************************************************************
-*  Copyright (C) 2012 SchedMD LLC
-*  Written by Danny Auble <da@schedmd.com>
-*
-*  This file is part of Slurm, a resource management program.
-*  For details, see <https://slurm.schedmd.com/>.
-*  Please also read the included file: DISCLAIMER.
-*
-*  Slurm is free software; you can redistribute it and/or modify it under
-*  the terms of the GNU General Public License as published by the Free
-*  Software Foundation; either version 2 of the License, or (at your option)
-*  any later version.
-*
-*  In addition, as a special exception, the copyright holders give permission
-*  to link the code of portions of this program with the OpenSSL library under
-*  certain conditions as described in each individual source file, and
-*  distribute linked combinations including the two. You must obey the GNU
-*  General Public License in all respects for all of the code used other than
-*  OpenSSL. If you modify file(s) with this exception, you may extend this
-*  exception to your version of the file(s), but you are not obligated to do
-*  so. If you do not wish to do so, delete this exception statement from your
-*  version.  If you delete this exception statement from all source files in
-*  the program, then also delete it here.
-*
-*  Slurm is distributed in the hope that it will be useful, but WITHOUT ANY
-*  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-*  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
-*  details.
-*
-*  You should have received a copy of the GNU General Public License along
-*  with Slurm; if not, write to the Free Software Foundation, Inc.,
-*  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
+/*****************************************************************************\
+ *  launch.c - Define job launch plugin functions.
+ *****************************************************************************
+ *  Copyright (C) SchedMD LLC.
+ *
+ *  This file is part of Slurm, a resource management program.
+ *  For details, see <https://slurm.schedmd.com/>.
+ *  Please also read the included file: DISCLAIMER.
+ *
+ *  Slurm is free software; you can redistribute it and/or modify it under
+ *  the terms of the GNU General Public License as published by the Free
+ *  Software Foundation; either version 2 of the License, or (at your option)
+ *  any later version.
+ *
+ *  In addition, as a special exception, the copyright holders give permission
+ *  to link the code of portions of this program with the OpenSSL library under
+ *  certain conditions as described in each individual source file, and
+ *  distribute linked combinations including the two. You must obey the GNU
+ *  General Public License in all respects for all of the code used other than
+ *  OpenSSL. If you modify file(s) with this exception, you may extend this
+ *  exception to your version of the file(s), but you are not obligated to do
+ *  so. If you do not wish to do so, delete this exception statement from your
+ *  version.  If you delete this exception statement from all source files in
+ *  the program, then also delete it here.
+ *
+ *  Slurm is distributed in the hope that it will be useful, but WITHOUT ANY
+ *  WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
+ *  FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
+ *  details.
+ *
+ *  You should have received a copy of the GNU General Public License along
+ *  with Slurm; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA.
 \*****************************************************************************/
 
 #include "config.h"
@@ -50,6 +50,7 @@
 
 #include "src/common/env.h"
 #include "src/common/fd.h"
+#include "src/common/forward.h"
 #include "src/common/net.h"
 #include "src/common/xstring.h"
 #include "src/common/plugin.h"
@@ -70,7 +71,7 @@
 
 #define MAX_STEP_RETRIES 4
 
-static List local_job_list = NULL;
+static list_t *local_job_list = NULL;
 static uint32_t *local_global_rc = NULL;
 static pthread_mutex_t launch_lock = PTHREAD_MUTEX_INITIALIZER;
 static pthread_mutex_t het_job_lock = PTHREAD_MUTEX_INITIALIZER;
@@ -78,7 +79,7 @@ static pthread_cond_t  start_cond  = PTHREAD_COND_INITIALIZER;
 static pthread_mutex_t start_mutex = PTHREAD_MUTEX_INITIALIZER;
 static slurm_opt_t *opt_save = NULL;
 
-static List task_state_list = NULL;
+static list_t *task_state_list = NULL;
 static time_t launch_start_time;
 static bool retry_step_begin = false;
 static int  retry_step_cnt = 0;
@@ -88,7 +89,7 @@ extern char **environ;
 static int _step_signal(int signal)
 {
 	srun_job_t *my_srun_job;
-	ListIterator iter;
+	list_itr_t *iter;
 	int rc = SLURM_SUCCESS, rc2;
 
 	if (!local_job_list) {
@@ -100,7 +101,8 @@ static int _step_signal(int signal)
 	while ((my_srun_job = (srun_job_t *) list_next(iter))) {
 		info("Terminating %ps", &my_srun_job->step_id);
 		rc2 = slurm_kill_job_step(my_srun_job->step_id.job_id,
-					  my_srun_job->step_id.step_id, signal);
+					  my_srun_job->step_id.step_id, signal,
+					  0);
 		if (rc2)
 			rc = rc2;
 	}
@@ -108,7 +110,7 @@ static int _step_signal(int signal)
 	return rc;
 }
 
-static char *_hostset_to_string(hostset_t hs)
+static char *_hostset_to_string(hostset_t *hs)
 {
 	size_t n = 1024;
 	size_t maxsize = 1024 * 64;
@@ -135,7 +137,7 @@ static char *_task_ids_to_host_list(int ntasks, uint32_t *taskids,
 				    srun_job_t *my_srun_job)
 {
 	int i, task_cnt = 0;
-	hostset_t hs;
+	hostset_t *hs;
 	char *hosts;
 	slurm_step_layout_t *sl;
 
@@ -244,7 +246,7 @@ static int _is_openmpi_port_error(int errcode)
 {
 	if (errcode != OPEN_MPI_PORT_ERROR)
 		return 0;
-	if (opt_save && (opt_save->srun_opt->resv_port_cnt == NO_VAL))
+	if (opt_save && (opt_save->resv_port_cnt == NO_VAL))
 		return 0;
 	if (difftime(time(NULL), launch_start_time) > slurm_conf.msg_timeout)
 		return 0;
@@ -269,7 +271,7 @@ _handle_openmpi_port_error(const char *tasks, const char *hosts,
 	      hosts, tasks, msg);
 
 	info("Terminating job step %ps", &step_id);
-	slurm_kill_job_step(step_id.job_id, step_id.step_id, SIGKILL);
+	slurm_kill_job_step(step_id.job_id, step_id.step_id, SIGKILL, 0);
 }
 
 static char *_mpir_get_host_name(char *node_name)
@@ -491,7 +493,7 @@ static void _task_finish(task_exit_msg_t *msg)
 /*
  * Load the multi_prog config file into argv, pass the  entire file contents
  * in order to avoid having to read the file on every node. We could parse
- * the infomration here too for loading the MPIR records for TotalView
+ * the information here too for loading the MPIR records for TotalView
  */
 static void _load_multi(int *argc, char **argv)
 {
@@ -585,7 +587,8 @@ static char **_build_user_env(srun_job_t *job, slurm_opt_t *opt_local)
 	else if (all)
 		env_array_merge(&dest_array, (const char **) job->env);
 	else
-		env_array_merge_slurm(&dest_array, (const char **) job->env);
+		env_array_merge_slurm_spank(&dest_array,
+					    (const char **) job->env);
 
 	return dest_array;
 }
@@ -627,16 +630,6 @@ static void _wait_all_het_job_comps_started(slurm_opt_t *opt_local)
 	slurm_mutex_unlock(&start_mutex);
 }
 
-/*
- * Initialize context for plugin
- */
-extern int launch_init(void)
-{
-	int retval = SLURM_SUCCESS;
-
-	return retval;
-}
-
 extern int location_fini(void)
 {
 	FREE_NULL_LIST(task_state_list);
@@ -650,13 +643,37 @@ extern slurm_step_layout_t *launch_common_get_slurm_step_layout(srun_job_t *job)
 		NULL : job->step_ctx->step_resp->step_layout;
 }
 
+static int _parse_gpu_request(char *in_str)
+{
+	char *save_ptr = NULL, *tmp_str, *tok, *sep;
+	int gpus_val = 0;
+
+	tmp_str = xstrdup(in_str);
+
+	tok = strtok_r(tmp_str, ",", &save_ptr);
+	while (tok) {
+		int tmp = 0;
+		sep = xstrchr(tok, ':');
+		if (sep)
+			tmp += atoi(sep + 1);
+		else
+			tmp += atoi(tok);
+		if (tmp > 0)
+			gpus_val += tmp;
+		tok = strtok_r(NULL, ",", &save_ptr);
+	}
+	xfree(tmp_str);
+
+	return gpus_val;
+}
+
 static job_step_create_request_msg_t *_create_job_step_create_request(
 	slurm_opt_t *opt_local, bool use_all_cpus, srun_job_t *job)
 {
-	char *add_tres = NULL;
+	char *add_tres = NULL, *pos;
 	srun_opt_t *srun_opt = opt_local->srun_opt;
 	job_step_create_request_msg_t *step_req = xmalloc(sizeof(*step_req));
-	List tmp_gres_list = NULL;
+	list_t *tmp_gres_list = NULL;
 	int rc;
 
 	xassert(srun_opt);
@@ -667,8 +684,14 @@ static job_step_create_request_msg_t *_create_job_step_create_request(
 	step_req->cpu_freq_gov = opt_local->cpu_freq_gov;
 
 	if (opt_local->cpus_per_gpu) {
-		xstrfmtcat(step_req->cpus_per_tres, "gres:gpu:%d",
+		xstrfmtcat(step_req->cpus_per_tres, "gres/gpu:%d",
 			   opt_local->cpus_per_gpu);
+		/* Like cpus_per_task, imply --exact */
+		if (srun_opt->whole)
+			info("Ignoring --whole since --cpus-per-gpu used");
+		else if (!srun_opt->exact)
+			verbose("Implicitly setting --exact, because --cpus-per-gpu given.");
+		srun_opt->exact = true;
 	}
 
 	step_req->exc_nodes = xstrdup(opt_local->exclude);
@@ -686,6 +709,14 @@ static job_step_create_request_msg_t *_create_job_step_create_request(
 		debug("interactive step launch request");
 		step_req->flags |= SSF_INTERACTIVE;
 	}
+	if (srun_opt->external_launcher) {
+		debug("external launcher step request");
+		step_req->flags |= SSF_EXT_LAUNCHER;
+	}
+	if (opt_local->job_flags & GRES_ALLOW_TASK_SHARING)
+		step_req->flags |= SSF_GRES_ALLOW_TASK_SHARING;
+	if (srun_opt->wait_for_children)
+		step_req->flags |= SSF_WAIT_FOR_CHILDREN;
 
 	if (opt_local->immediate == 1)
 		step_req->immediate = opt_local->immediate;
@@ -696,7 +727,7 @@ static job_step_create_request_msg_t *_create_job_step_create_request(
 		step_req->max_nodes = opt_local->max_nodes;
 
 	if (opt_local->mem_per_gpu != NO_VAL64)
-		xstrfmtcat(step_req->mem_per_tres, "gres:gpu:%"PRIu64,
+		xstrfmtcat(step_req->mem_per_tres, "gres/gpu:%"PRIu64,
 			   opt.mem_per_gpu);
 
 	step_req->min_nodes = job->nhosts;
@@ -704,6 +735,10 @@ static job_step_create_request_msg_t *_create_job_step_create_request(
 	    (opt_local->min_nodes < step_req->min_nodes))
 		step_req->min_nodes = opt_local->min_nodes;
 
+	if (opt_local->gres)
+		add_tres = opt_local->gres;
+	else
+		add_tres = getenv("SLURM_STEP_GRES");
 	/*
 	 * If the number of CPUs was specified (cpus_set==true), then we need to
 	 * set exact = true. Otherwise the step will be allocated the wrong
@@ -716,34 +751,74 @@ static job_step_create_request_msg_t *_create_job_step_create_request(
 		else
 			step_req->cpu_count = step_req->min_nodes;
 	} else if (opt_local->cpus_set) {
-		step_req->cpu_count =
-			opt_local->ntasks * opt_local->cpus_per_task;
+		if (opt_local->ntasks == NO_VAL)
+			step_req->cpu_count = NO_VAL;
+		else
+			step_req->cpu_count = opt_local->ntasks *
+				opt_local->cpus_per_task;
 		if (srun_opt->whole)
 			info("Ignoring --whole since -c/--cpus-per-task used");
 		else if (!srun_opt->exact)
 			verbose("Implicitly setting --exact, because -c/--cpus-per-task given.");
 		srun_opt->exact = true;
-	} else if (opt_local->gpus_per_task && opt_local->cpus_per_gpu) {
-		char *save_ptr = NULL, *tmp_str, *tok, *sep;
-		int gpus_per_task = 0;
+	} else if (opt_local->cpus_per_gpu) {
+		if (opt_local->gpus) {
+			int gpus_per_step;
 
-		tmp_str = xstrdup(opt_local->gpus_per_task);
+			gpus_per_step = _parse_gpu_request(opt_local->gpus);
+			step_req->cpu_count = gpus_per_step *
+				opt_local->cpus_per_gpu;
+		} else if (opt_local->gpus_per_node) {
+			int gpus_per_node;
 
-		tok = strtok_r(tmp_str, ",", &save_ptr);
-		while (tok) {
-			int tmp = 0;
-			sep = xstrchr(tok, ':');
-			if (sep)
-				tmp += atoi(sep + 1);
-			else
-				tmp += atoi(tok);
-			if (tmp > 0)
-				gpus_per_task += tmp;
-			tok = strtok_r(NULL, ",", &save_ptr);
+			gpus_per_node =
+				_parse_gpu_request(opt_local->gpus_per_node);
+			/* Use a minimum value for requested cpus */
+			step_req->cpu_count = opt_local->min_nodes *
+				gpus_per_node * opt_local->cpus_per_gpu;
+		} else if (opt_local->tres_per_task &&
+			   (pos = xstrstr(opt_local->tres_per_task,
+					  "gres/gpu:"))) {
+			pos += 9; /* Don't include "gres/gpu:" */
+			step_req->cpu_count = opt_local->ntasks *
+					      _parse_gpu_request(pos) *
+					      opt_local->cpus_per_gpu;
+		} else if (add_tres) {
+			int rc = SLURM_SUCCESS;
+			uint64_t gpus_per_node = 0;
+			char *save_ptr = NULL;
+
+			while (slurm_option_get_tres_per_tres(
+				       opt_local->gres, "gpu",
+				       &gpus_per_node, &save_ptr,
+				       &rc));
+			if (rc != SLURM_SUCCESS) {
+				/* Failed to parse, error already logged */
+				slurm_free_job_step_create_request_msg(
+							step_req);
+				return NULL;
+			}
+			/*
+			 * Same math as gpus_per_node
+			 *
+			 * If gpus_per_node == 0, then the step did not request
+			 * gpus, but the step may still inherit gpus from the
+			 * job. This math will set requested cpus to zero in
+			 * case gpus_per_node == 0.
+			 */
+			step_req->cpu_count = opt_local->min_nodes *
+				gpus_per_node * opt_local->cpus_per_gpu;
+		} else if (opt_local->gpus_per_socket) {
+			/*
+			 * gpus_per_socket is not fully supported for steps and
+			 * does not affect the gpus allocated to the step:
+			 * slurmctld may inherit gres from the job.
+			 */
 		}
-		xfree(tmp_str);
-		step_req->cpu_count = opt_local->ntasks * gpus_per_task *
-				      opt_local->cpus_per_gpu;
+		/*
+		 * If none of these are requested, the step may inherit gres
+		 * from the job.
+		 */
 	} else if (opt_local->ntasks_set ||
 		   (opt_local->ntasks_per_tres != NO_VAL) ||
 		   (opt_local->ntasks_per_gpu != NO_VAL)) {
@@ -785,19 +860,10 @@ static job_step_create_request_msg_t *_create_job_step_create_request(
 
 	step_req->relative = srun_opt->relative;
 
-	if (srun_opt->resv_port_cnt != NO_VAL) {
-		step_req->resv_port_cnt = srun_opt->resv_port_cnt;
+	if (opt_local->resv_port_cnt != NO_VAL) {
+		step_req->resv_port_cnt = opt_local->resv_port_cnt;
 	} else {
-#if defined(HAVE_NATIVE_CRAY)
-		/*
-		 * On Cray systems default to reserving one port, or one
-		 * more than the number of multi prog commands, for Cray PMI
-		 */
-		step_req->resv_port_cnt = (srun_opt->multi_prog ?
-					   srun_opt->multi_prog_cmds + 1 : 1);
-#else
 		step_req->resv_port_cnt = NO_VAL16;
-#endif
 	}
 
 	step_req->srun_pid = (uint32_t) getpid();
@@ -806,6 +872,10 @@ static job_step_create_request_msg_t *_create_job_step_create_request(
 	memcpy(&step_req->step_id, &job->step_id, sizeof(step_req->step_id));
 	step_req->array_task_id = srun_opt->array_task_id;
 
+	step_req->cwd = xstrdup(opt_local->chdir);
+	step_req->std_err = xstrdup(opt_local->efname);
+	step_req->std_in = xstrdup(opt_local->ifname);
+	step_req->std_out = xstrdup(opt_local->ofname);
 	step_req->submit_line = xstrdup(opt_local->submit_line);
 
 	if (opt_local->threads_per_core != NO_VAL) {
@@ -813,48 +883,44 @@ static job_step_create_request_msg_t *_create_job_step_create_request(
 	} else
 		step_req->threads_per_core = NO_VAL16;
 
-	/*
-	 * FIXME: tres_bind is really gres_bind. This should be fixed in the
-	 * future.
-	 */
-
 	if (!opt_local->tres_bind &&
 	    ((opt_local->ntasks_per_tres != NO_VAL) ||
 	     (opt_local->ntasks_per_gpu != NO_VAL))) {
 		/* Implicit single GPU binding with ntasks-per-tres/gpu */
 		if (opt_local->ntasks_per_tres != NO_VAL)
-			xstrfmtcat(opt_local->tres_bind, "gpu:single:%d",
+			xstrfmtcat(opt_local->tres_bind, "gres/gpu:single:%d",
 				   opt_local->ntasks_per_tres);
 		else
-			xstrfmtcat(opt_local->tres_bind, "gpu:single:%d",
+			xstrfmtcat(opt_local->tres_bind, "gres/gpu:single:%d",
 				   opt_local->ntasks_per_gpu);
 	}
 
-	/*
-	 * FIXME: tres_per_task Should be handled in src/common/slurm_opt.c
-	 * _validate_tres_per_task(). But we should probably revisit this to get
-	 * rid of gpus_per_task completely.
-	 */
-	if (!opt_local->tres_bind && opt_local->gpus_per_task) {
-		/* Implicit GPU binding with gpus_per_task */
-		xstrfmtcat(opt_local->tres_bind, "gpu:per_task:%s",
-			   opt_local->gpus_per_task);
-	}
-
+	step_req->tres_per_task = xstrdup(opt_local->tres_per_task);
 	step_req->tres_bind = xstrdup(opt_local->tres_bind);
 	step_req->tres_freq = xstrdup(opt_local->tres_freq);
 
 	xstrfmtcat(step_req->tres_per_step, "%scpu:%u",
 		   step_req->tres_per_step ? "," : "",
 		   step_req->cpu_count);
-	xfmt_tres(&step_req->tres_per_step, "gres:gpu", opt_local->gpus);
+	xfmt_tres(&step_req->tres_per_step, "gres/gpu", opt_local->gpus);
 
-	xfmt_tres(&step_req->tres_per_node, "gres:gpu",
-		  opt_local->gpus_per_node);
+	/* add_tres set from opt_local->gres or environment above */
 	if (opt_local->gres)
 		add_tres = opt_local->gres;
 	else
 		add_tres = getenv("SLURM_STEP_GRES");
+
+	/*
+	 * If --gres=none was requested, we need to send exactly
+	 * tres_per_node="none" to slurmctld to not inherit gres from the job.
+	 * Do not also send --gpus-per-node which could have been set from the
+	 * environment.
+	 */
+	if (!add_tres || xstrcasecmp(add_tres, "NONE")) {
+		xfmt_tres(&step_req->tres_per_node, "gres/gpu",
+			  opt_local->gpus_per_node);
+	}
+
 	if (add_tres) {
 		if (step_req->tres_per_node) {
 			xstrfmtcat(step_req->tres_per_node, ",%s", add_tres);
@@ -862,15 +928,13 @@ static job_step_create_request_msg_t *_create_job_step_create_request(
 			step_req->tres_per_node = xstrdup(add_tres);
 	}
 
-	xfmt_tres(&step_req->tres_per_socket, "gres:gpu",
+	xfmt_tres(&step_req->tres_per_socket, "gres/gpu",
 		  opt_local->gpus_per_socket);
 
 	if (opt_local->cpus_set)
 		xstrfmtcat(step_req->tres_per_task, "%scpu:%u",
 			   step_req->tres_per_task ? "," : "",
 			   opt_local->cpus_per_task);
-	xfmt_tres(&step_req->tres_per_task, "gres:gpu",
-		  opt_local->gpus_per_task);
 
 	if (opt_local->time_limit != NO_VAL)
 		step_req->time_limit = opt_local->time_limit;
@@ -890,7 +954,7 @@ static job_step_create_request_msg_t *_create_job_step_create_request(
 				     step_req->ntasks_per_tres,
 				     step_req->min_nodes,
 				     &tmp_gres_list,
-				     NULL, job->step_id.job_id,
+				     job->step_id.job_id,
 				     NO_VAL, &step_req->num_tasks,
 				     &step_req->cpu_count, NULL);
 	FREE_NULL_LIST(tmp_gres_list);
@@ -923,7 +987,7 @@ static job_step_create_request_msg_t *_create_job_step_create_request(
 	{
 		uint16_t base_dist;
 		/* Leave distribution set to unknown if taskcount <= nodes and
-		 * memory is set to 0. step_mgr will handle the mem=0 case. */
+		 * memory is set to 0. stepmgr will handle the mem=0 case. */
 		if (!opt_local->mem_per_cpu || !opt_local->pn_min_memory ||
 		    srun_opt->interactive)
 			base_dist = SLURM_DIST_UNKNOWN;
@@ -1124,13 +1188,11 @@ extern int launch_g_create_job_step(srun_job_t *job, bool use_all_cpus,
 		       opt_local->min_nodes, opt_local->max_nodes);
 		return SLURM_ERROR;
 	}
-#if !defined HAVE_FRONT_END
 	if (opt_local->min_nodes && (opt_local->min_nodes > job->nhosts)) {
 		error ("Minimum node count > allocated node count (%d > %d)",
 		       opt_local->min_nodes, job->nhosts);
 		return SLURM_ERROR;
 	}
-#endif
 
 	step_req = _create_job_step_create_request(
 		opt_local, use_all_cpus, job);
@@ -1152,7 +1214,12 @@ extern int launch_g_create_job_step(srun_job_t *job, bool use_all_cpus,
 	      step_req->name, step_req->relative);
 
 	for (i = 0; (!(*destroy_job)); i++) {
+		bool timed_out = false;
 		if (srun_opt->no_alloc) {
+			if (step_req->num_tasks == NO_VAL) {
+				step_req->num_tasks = job->ntasks;
+				step_req->cpu_count = job->cpu_count;
+			}
 			job->step_ctx = step_ctx_create_no_alloc(
 				step_req, job->step_id.step_id);
 		} else {
@@ -1167,8 +1234,9 @@ extern int launch_g_create_job_step(srun_job_t *job, bool use_all_cpus,
 				step_wait = ((getpid() % 10) +
 					     slurmctld_timeout) * 1000;
 			}
-			job->step_ctx = step_ctx_create_timeout(
-						step_req, step_wait);
+			job->step_ctx = step_ctx_create_timeout(step_req,
+								step_wait,
+								&timed_out);
 		}
 		if (job->step_ctx != NULL) {
 			job->step_ctx->verbose_level = opt_local->verbose;
@@ -1178,7 +1246,7 @@ extern int launch_g_create_job_step(srun_job_t *job, bool use_all_cpus,
 			}
 			break;
 		}
-		rc = slurm_get_errno();
+		rc = errno;
 
 		if (((opt_local->immediate != 0) &&
 		     ((opt_local->immediate == 1) ||
@@ -1198,9 +1266,13 @@ extern int launch_g_create_job_step(srun_job_t *job, bool use_all_cpus,
 					"being configured, please wait",
 					step_req->step_id.job_id);
 			} else {
-				info("Job %u step creation temporarily disabled, retrying (%s)",
-				     step_req->step_id.job_id,
-				     slurm_strerror(rc));
+				if (timed_out)
+					info("Job %u step creation temporarily disabled, retrying (%s)",
+					     step_req->step_id.job_id,
+					     slurm_strerror(rc));
+				else
+					verbose("Step completed in JobId=%u, retrying",
+						step_req->step_id.job_id);
 			}
 			xsignal_unblock(sig_array);
 			for (j = 0; sig_array[j]; j++)
@@ -1210,10 +1282,15 @@ extern int launch_g_create_job_step(srun_job_t *job, bool use_all_cpus,
 				verbose("Job %u step creation still disabled, retrying (%s)",
 					step_req->step_id.job_id,
 					slurm_strerror(rc));
-			else
-				info("Job %u step creation still disabled, retrying (%s)",
-				     step_req->step_id.job_id,
-				     slurm_strerror(rc));
+			else {
+				if (timed_out)
+					info("Job %u step creation still disabled, retrying (%s)",
+					     step_req->step_id.job_id,
+					     slurm_strerror(rc));
+				else
+					verbose("Step completed in JobId=%u, retrying",
+						step_req->step_id.job_id);
+			}
 		}
 
 		if (*destroy_job) {
@@ -1241,7 +1318,10 @@ extern int launch_g_create_job_step(srun_job_t *job, bool use_all_cpus,
 		slurm_free_job_step_create_request_msg(step_req);
 		return SLURM_ERROR;
 	}
-
+	fwd_set_alias_addrs(step_layout->alias_addrs);
+	if (opt_local->cpus_set && (job->cpu_count == NO_VAL))
+		job->cpu_count = step_layout->task_cnt *
+			opt_local->cpus_per_task;
 	if (job->ntasks != step_layout->task_cnt)
 		job->ntasks = step_layout->task_cnt;
 
@@ -1275,6 +1355,7 @@ extern int launch_g_step_launch(srun_job_t *job, slurm_step_io_fds_t *cio_fds,
 	srun_opt_t *srun_opt = opt_local->srun_opt;
 	slurm_step_launch_params_t launch_params;
 	slurm_step_launch_callbacks_t callbacks;
+	slurm_step_layout_t *layout;
 	int rc = SLURM_SUCCESS;
 	task_state_t *task_state;
 	bool first_launch = false;
@@ -1305,7 +1386,6 @@ extern int launch_g_step_launch(srun_job_t *job, slurm_step_io_fds_t *cio_fds,
 		task_state_alter(task_state, job->ntasks);
 	}
 
-	launch_params.alias_list = job->alias_list;
 	launch_params.argc = opt_local->argc;
 	launch_params.argv = opt_local->argv;
 	launch_params.multi_prog = srun_opt->multi_prog ? true : false;
@@ -1323,20 +1403,21 @@ extern int launch_g_step_launch(srun_job_t *job, slurm_step_io_fds_t *cio_fds,
 	launch_params.het_job_ntasks = job->het_job_ntasks;
 	launch_params.het_job_offset = job->het_job_offset;
 	launch_params.het_job_step_cnt = srun_opt->het_step_cnt;
+	launch_params.het_job_step_task_cnts = job->het_job_step_task_cnts;
 	launch_params.het_job_task_offset = job->het_job_task_offset;
 	launch_params.het_job_task_cnts = job->het_job_task_cnts;
 	launch_params.het_job_tids = job->het_job_tids;
 	launch_params.het_job_tid_offsets = job->het_job_tid_offsets;
 	launch_params.het_job_node_list = job->het_job_node_list;
-	launch_params.partition = job->partition;
 	launch_params.profile = opt_local->profile;
 	launch_params.task_prolog = srun_opt->task_prolog;
 	launch_params.task_epilog = srun_opt->task_epilog;
 
-	if (!srun_opt->cpu_bind_type &&
+	if (!(srun_opt->cpu_bind_type & (~CPU_BIND_VERBOSE)) &&
 	    job->step_ctx->step_resp->def_cpu_bind_type)
 		srun_opt->cpu_bind_type =
-			job->step_ctx->step_resp->def_cpu_bind_type;
+			job->step_ctx->step_resp->def_cpu_bind_type |
+			srun_opt->cpu_bind_type;
 	if (get_log_level() >= LOG_LEVEL_VERBOSE) {
 		slurm_sprint_cpu_bind_type(tmp_str, srun_opt->cpu_bind_type);
 		verbose("CpuBindType=%s", tmp_str);
@@ -1355,6 +1436,11 @@ extern int launch_g_step_launch(srun_job_t *job, slurm_step_io_fds_t *cio_fds,
 		launch_params.cpus_per_task	= opt_local->cpus_per_task;
 	else
 		launch_params.cpus_per_task	= 1;
+
+	layout = job->step_ctx->step_resp->step_layout;
+	launch_params.cpt_compact_array = layout->cpt_compact_array;
+	launch_params.cpt_compact_cnt = layout->cpt_compact_cnt;
+	launch_params.cpt_compact_reps = layout->cpt_compact_reps;
 	launch_params.threads_per_core   = opt_local->threads_per_core;
 	launch_params.cpu_freq_min       = opt_local->cpu_freq_min;
 	launch_params.cpu_freq_max       = opt_local->cpu_freq_max;
@@ -1372,6 +1458,8 @@ extern int launch_g_step_launch(srun_job_t *job, slurm_step_io_fds_t *cio_fds,
 	launch_params.no_alloc           = srun_opt->no_alloc;
 	launch_params.mpi_plugin_name = srun_opt->mpi_type;
 	launch_params.env = _build_user_env(job, opt_local);
+	launch_params.tree_width = srun_opt->tree_width;
+	launch_params.oom_kill_step = opt_local->oom_kill_step;
 
 	memcpy(&launch_params.local_fds, cio_fds, sizeof(slurm_step_io_fds_t));
 
@@ -1501,7 +1589,7 @@ extern void launch_g_print_status(void)
 extern void launch_g_fwd_signal(int signal)
 {
 	srun_job_t *my_srun_job;
-	ListIterator iter;
+	list_itr_t *iter;
 
 	if (!local_job_list) {
 		debug("%s: local_job_list does not exist yet", __func__);
